#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import json
import subprocess
import logging
import time

SUCCESS = 0
WARN = 1
CRITICAL = 2

CodeToStatus_map = {SUCCESS: "Success", WARN: "Warning", CRITICAL: "Critical"}

StatusToCode_map = {"Success": SUCCESS, "Warning": WARN, "Critical": CRITICAL}


# Exception
class CheckerError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


# Base class for checker implemention
class baseChecker(object):
    def __init__(self, logger, timeout):
        self.checkStatus = WARN
        self.logger = logger
        self.cmdHandle = cmdHandler(self.logger, timeout)

    # Internal method to generate check result structure
    def _checkResult(self, corrAction="", msg="", msgId="", msgReplcmt=[],
                     status=""):
        result_dict = {}
        result_dict["CorrectiveAction"] = corrAction
        result_dict["Message"] = msg
        result_dict["MessageId"] = msgId
        result_dict["MsgSubstitutes"] = msgReplcmt
        result_dict["Status"] = status
        result_dict["Verbose"] = "null"
        result_dict["TimeStamp"] = getCurrSysTime()
        return result_dict


class cmdHandler:
    def __init__(self, logger, timedOut=60):
        self.logger = logger
        self.timedOut = timedOut

    # Internal method to execute a command
    # RC 0 - Cmd execution with zero return code
    # RC 1 - Cmd executed with non zero return code
    # RC 2 - Runtime error / exception
    def cmd_run(self, cmd):
        cmd_output = ''
        cmd_err = ''
        rc = 0

        self.logger.info("CommmandExec \"{0}\"".format(cmd))
        try:
            #TODO: Defect 47986 - this was flagged in a Bandit Scan.
            #Will need to fix subprocess issue in 2Q
            process = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE, shell=True)
            (cmd_output, cmd_err) = process.communicate()
#        except subprocess.TimeoutExpired as err:
#            cmd_err = "Command {0} Timedout".format(cmd)
#            self.logger.error(cmd_err)
#            rc = 2
        except Exception as err:
            self.logger.error("Command execution failed")
            self.logger.error(err)
            cmd_err = "Command execution failed"
            rc = 2
        else:
            if process.returncode == 0:
                rc = 0
            else:
                rc = 1
            self.logger.info("CmdReturnCode \"{0}\"".format(str(rc)))
            self.logger.info("CmdOutput \"{0}\"".format(cmd_output))
            self.logger.info("CmdError \"{0}\"".format(cmd_err))
        return (rc, cmd_output, cmd_err)


# CheckerLogger class will handle the logging
class checkerLogger:
    def __init__(self, log_file=None):
        self.logger = logging.getLogger(__name__)
        self.log_file = log_file
        self.logger.setLevel(logging.DEBUG)
        handler = logging.FileHandler(self.log_file, "w")
        handler.setLevel(logging.INFO)
        handler.setLevel(logging.ERROR)
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s -\
            %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)

    def getLogger(self):
        return self.logger

# validator class provides methods to execute the checkers and
# generates the JSON output


class validator:
    # JSON output dictionary
    output_dict = {
        "ValidationID": "Default_Id",
        "ValidationStatus": "Success",
        "Description": "default resource",
        "TargetedResourceType": "default",
        "IpAddress": "",
        "ValidationDetail": {
            "ChecksList": {}
        },
    }

    def __init__(self, chkDict=None, logger=None, timeout=None):
        self.valStatus = SUCCESS
        self.chkDict = chkDict
        self.logger = logger
        self.cmdHandle = cmdHandler(self.logger, timeout)

    # Internal method to retrieve host IP
    def _getHostIP(self):
        cmd = 'hostname -i'
        self.logger.info("Executing \"{0}\"".format(cmd))
        (rc, cmd_out, cmd_err) = self.cmdHandle.cmd_run(cmd)
        ip = cmd_out.strip()
        return ip

    def setDetails(self, id, descripn, restype):
        self.output_dict["ValidationID"] = id
        self.output_dict["Description"] = descripn
        self.output_dict["TargetedResourceType"] = restype

    def validate(self):
        checksList = {}
        for key in self.chkDict.keys():
            entry_dict = self._validate(key)
            checksList[key] = entry_dict

        ip = self._getHostIP()
        self.output_dict["ValidationDetail"]["ChecksList"] = checksList
        self.output_dict["ValidationStatus"] = CodeToStatus_map[self.valStatus]
        self.output_dict["IpAddress"] = ip
        self.logger.info("Result is -->  {0}".
                         format(json.dumps({"ValidationResult":
                                            self.output_dict})))
        return (json.dumps({"ValidationResult": self.output_dict}))

    def getStatus(self):
        return self.valStatus

    def _validate(self, id):
        chkObj = self.chkDict[id]
        result_list = chkObj.check()
        chk_status = 0
        for entry in result_list:
            if chk_status < StatusToCode_map[entry["Status"]]:
                chk_status = StatusToCode_map[entry["Status"]]
            if self.valStatus < chk_status:
                self.valStatus = chk_status
        return result_list


def getCurrSysTime():
    utcTime = str(int(time.time()))
    return utcTime
