#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import json
import glob
from operator import itemgetter
from powervc_ttv.utils.exceptions import CommandExecutionFailed
from powervc_ttv.utils.utils import result_msg_dict
from powervc_ttv.utils import utils
from powervc_ttv.resources.fabrics import CiscoFabric
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import _
from powervc_ttv import translate_message

SUCCESS = 0
WARN = 1
CRITICAL = 2


CodeToStatus_map = {SUCCESS: "Success", WARN: "Warning", CRITICAL: "Critical"}
StatusToCode_map = {"Success": SUCCESS, "Warning": WARN, "Critical": CRITICAL}

CHECK_LOG = _("Check log for details.")

output_dict = {
    "ValidationID": "CISCO_FABRIC_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "cisco_fabric",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}
logFile = "cisco_fabric_validator.log"

# IDs for validation checks
CISCO_FABRIC_FIRMWARE = "CISCO_FABRIC_FIRMWARE"
CISCO_FABRIC_REACHABILITY = "CISCO_FABRIC_REACHABILITY"
CISCO_FABRIC_ZONING_MEMORY_SPACE = "CISCO_FABRIC_ZONING_MEMORY_SPACE"
CISCO_FABRIC_USER_ZONING_ROLE = "CISCO_FABRIC_USER_ZONING_ROLE"
CISCO_FABRIC_EXCEPTION = "CISCO_FABRIC_EXCEPTION"


class FirmwareChecker():

    def __init__(
            self, ciscofabric, expected_firmware, logger=None, timeout=60):
        self.ciscofabric = ciscofabric
        self.expectedFirmware = expected_firmware
        self.result = []

    # This method should return the result of format result_msg_dict
    def check(self):
        msg = ''
        corrAction = ''
        status = ''
        msgId = ''
        msgReplcmnt = []
        verbose = ""
        val_check_id = CISCO_FABRIC_FIRMWARE
        host = self.ciscofabric.ip

        firmwareVersion = None

        try:
            firmwareVersion = self.ciscofabric.get_firmware_version()
            if firmwareVersion:
                if utils.compareVersion(
                        firmwareVersion, self.expectedFirmware):
                    corrAction = ""
                    msgId = messageIDCatalog.CISCO_FIRMWARE_SUCCESS
                    msg = translate_message(
                        ttvCatalog.CISCO_FIRMWARE_SUCCESS,
                        desired_locale)
                    status = CodeToStatus_map[SUCCESS]
                else:
                    corrAction = translate_message(
                        ttvCatalog.CA_CISCO_FIRMWARE_FAIL.format(
                            str(self.expectedFirmware)),
                        desired_locale)
                    msgId = messageIDCatalog.CISCO_FIRMWARE_FAIL
                    msg = translate_message(
                        ttvCatalog.CISCO_FIRMWARE_FAIL,
                        desired_locale)
                    msgReplcmnt.append(self.expectedFirmware)
                    status = CodeToStatus_map[CRITICAL]
            else:
                corrAction = CHECK_LOG
                msg = translate_message(
                    ttvCatalog.CISCO_DETAILS_RETRIEVE_FAILED,
                    desired_locale)
                msgId = messageIDCatalog.CISCO_DETAILS_RETRIEVE_FAILED
                status = CodeToStatus_map[CRITICAL]
        except Exception as exp:
            logger.error(exp)
            raise exp

        chkResult = utils.checkResult(host, corrAction, [], msg, msgReplcmnt,
                                      msgId, "", status, verbose,
                                      val_check_id)
        self.result.append(chkResult)

        return (status, self.result)


class ReachabilityChecker():

    def __init__(self, ciscofabric, logger=None, timeout=60):
        self.ciscofabric = ciscofabric
        self.result = []

    # This method should return the result of format result_msg_dict
    def check(self):
        msg = ''
        corrAction = ''
        status = ''
        msgId = ''
        msgReplcmnt = []
        verbose = ''
        val_check_id = CISCO_FABRIC_REACHABILITY
        host = self.ciscofabric.ip

        reachable = False

        try:
            reachable = self.ciscofabric.is_reachable()
            if reachable:
                corrAction = ""
                msgId = messageIDCatalog.IPADDRESS_NETWORK_CONFIGURATION
                msg = translate_message(
                    ttvCatalog.RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER.
                    format(str(self.ciscofabric.ip)),
                    desired_locale)
                msgReplcmnt.append(self.ciscofabric.ip)
                status = CodeToStatus_map[SUCCESS]
                self.checkStatus = SUCCESS
            else:
                corrAction = translate_message(
                    ttvCatalog.CA_IPADDRESS_PING_FAILED.format(
                        str(self.ciscofabric.ip)),
                    desired_locale)
                msgId = messageIDCatalog.IPADDRESS_NETWORK_CONFIGURATION_ERROR
                msg = translate_message(
                    ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER.
                    format(str(self.ciscofabric.ip)),
                    desired_locale)
                msgReplcmnt.append(self.ciscofabric.ip)
                status = CodeToStatus_map[CRITICAL]
                self.checkStatus = CRITICAL
        except CommandExecutionFailed as cmd_failed_ex:
            raise cmd_failed_ex

        chkResult = utils.checkResult(host, corrAction, [], msg, msgReplcmnt,
                                      msgId, "", status, verbose,
                                      val_check_id)
        self.result.append(chkResult)

        return (status, self.result)


class ZoningMemorySpaceChecker():

    def __init__(self, ciscofabric, logger=None, timeout=60):
        self.ciscofabric = ciscofabric
        self.result = []

    # This method should return the result of format result_msg_dict
    def check(self):
        corrAction = ''
        status = ''
        msgId = ''
        msgReplcmnt = []
        verbose = ''
        val_check_id = CISCO_FABRIC_ZONING_MEMORY_SPACE
        host = self.ciscofabric.ip
        try:
            zoningMemorySpaces = self.ciscofabric.\
                get_zoning_memory_space_in_MB()
            if zoningMemorySpaces:
                actual_memory = zoningMemorySpaces.get('actual_memory')
                allocated_memory = zoningMemorySpaces.get('allocated_memory')
                if actual_memory <= allocated_memory:
                    corrAction = ""
                    msgId = messageIDCatalog.CISCO_TOTAL_ZONING_MEMORY_SUCCESS
                    msg = translate_message(
                        ttvCatalog.CISCO_TOTAL_ZONING_MEMORY_SUCCESS,
                        desired_locale)
                    status = CodeToStatus_map[SUCCESS]
                    self.checkStatus = SUCCESS
                else:
                    corrAction = translate_message(
                        ttvCatalog.CA_CISCO_TOTAL_ZONING_MEMORY_FAIL.
                        format(str(self.ciscofabric.vsan)),
                        desired_locale)
                    msgId = messageIDCatalog.CISCO_TOTAL_ZONING_MEMORY_FAIL
                    msg = translate_message(
                        ttvCatalog.CISCO_TOTAL_ZONING_MEMORY_FAIL,
                        desired_locale)
                    msgReplcmnt.append(self.ciscofabric.vsan)
                    status = CodeToStatus_map[CRITICAL]
                    self.checkStatus = CRITICAL
        except CommandExecutionFailed as cmd_failed_ex:
            raise cmd_failed_ex

        chkResult = utils.checkResult(host, corrAction, [], msg, msgReplcmnt,
                                      msgId, "", status, verbose, val_check_id)
        self.result.append(chkResult)

        return (status, self.result)


class UserZoningRoleChecker():

    def __init__(self, ciscofabric, logger=None, timeout=60):
        self.ciscofabric = ciscofabric
        self.result = []

    # This method should return the result of format result_msg_dict
    def check(self):
        corrAction = ''
        status = ''
        msgId = ''
        msgReplcmnt = []
        verbose = ''
        val_check_id = CISCO_FABRIC_USER_ZONING_ROLE
        host = self.ciscofabric.ip
        try:
            role = self.ciscofabric.get_user_role()
            if role == "network-admin":
                corrAction = ""
                msgId = messageIDCatalog.CISCO_ZONING_ROLE_SUCCESS
                msg = translate_message(
                    ttvCatalog.CISCO_ZONING_ROLE_SUCCESS,
                    desired_locale)
                status = CodeToStatus_map[SUCCESS]
                self.checkStatus = SUCCESS
            else:
                rules = self.ciscofabric.get_rules()
                rules_sorted_order = sorted(rules, key=itemgetter('rule_no'))
                rules_reverese_order = reversed(rules_sorted_order)
                flag = False
                for rule in rules_reverese_order:
                    command_type = rule['command-type'].lower()
                    feature = rule['feature'].lower()
                    if "deny" in rule['type']:
                        if command_type == "config" or command_type == "*":
                            if feature == "*" or feature == "zone":
                                break
                    if "permit" in rule['type']:
                        if command_type == "config" or command_type == "*":
                            if feature == "*" or feature == "zone":
                                flag = True
                                break
                if flag:
                    corrAction = ""
                    msgId = messageIDCatalog.CISCO_ZONING_ROLE_SUCCESS
                    msg = translate_message(
                        ttvCatalog.CISCO_ZONING_ROLE_SUCCESS,
                        desired_locale)
                    status = CodeToStatus_map[SUCCESS]
                    self.checkStatus = SUCCESS
                else:
                    corrAction = translate_message(
                        ttvCatalog.CA_CISCO_ZONING_ROLE_FAIL.
                        format(str(self.ciscofabric.vsan)),
                        desired_locale)
                    msgId = messageIDCatalog.CISCO_ZONING_ROLE_FAIL
                    msg = translate_message(
                        ttvCatalog.CISCO_ZONING_ROLE_FAIL,
                        desired_locale)
                    msgReplcmnt.append(self.ciscofabric.vsan)
                    status = CodeToStatus_map[CRITICAL]
                    self.checkStatus = CRITICAL
        except CommandExecutionFailed as cmd_failed_ex:
            raise cmd_failed_ex

        chkResult = utils.checkResult(host, corrAction, [], msg, msgReplcmnt,
                                      msgId, "", status, verbose, val_check_id)
        self.result.append(chkResult)

        return (status, self.result)


def get_fabrics():
    # fabric.conf file(s)
    env = {"FABRICS": []}
    for conf in glob.glob("/etc/cinder/fabrics.conf"):
        try:
            fabric_conf = ConfigParser.ConfigParser()
            fabric_conf.read(conf)
            fabricnames = fabric_conf.get("DEFAULT", "fc_fabric_names")
            for fabric in fabricnames.split(","):
                fabrics = {}
                if fabric_conf.has_option("DEFAULT",
                                          "fabric_type_" + fabric):
                    fc_fabric_type = fabric_conf.get("DEFAULT",
                                                     "fabric_type_" + fabric)
                    if fc_fabric_type.lower() == "cisco":
                        fabrics["fc_fabric_type"] = "CISCOFABRIC"
                    else:
                        fabrics["fc_fabric_type"] = "BROCADEFABRIC"

                    if fc_fabric_type.lower() == "cisco":
                        if fabric_conf.has_option("DEFAULT",
                                                  "fc_fabric_vsan_" + fabric):
                            fabrics["fc_fabric_vsan"] = fabric_conf.\
                                get("DEFAULT", "fc_fabric_vsan_" + fabric)
                        if fabric_conf.has_option("DEFAULT",
                                                  "fc_fabric_uniqueid_" +
                                                  fabric):
                            fabrics["fc_fabric_uniqueid"] = fabric_conf.\
                                get("DEFAULT", "fc_fabric_uniqueid_" + fabric)
                if fabric_conf.has_option("DEFAULT",
                                          "fc_fabric_address_" + fabric):
                    fabrics["hostname"] = fabric_conf.\
                        get("DEFAULT", "fc_fabric_address_" + fabric)
                    ip = fabrics["hostname"]
                    fabrics["ip"] = ip

                fabrics["fabric_name"] = fabric
                if fabric_conf.has_option("DEFAULT",
                                          "fc_fabric_user_" + fabric):
                    fabrics["fc_fabric_user"] = fabric_conf.\
                        get("DEFAULT", "fc_fabric_user_" + fabric)
                if fabric_conf.has_option("DEFAULT",
                                          "fc_fabric_password_" + fabric):
                    fabrics["fc_fabric_password"] = fabric_conf.\
                        get("DEFAULT", "fc_fabric_password_" + fabric)

                env["FABRICS"].append(fabrics)
        except Exception as e:
            raise e
    return env["FABRICS"]


def main():
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    global logger
    logger = chkLogger.getLogger()

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.CISCO_FABRIC_VALIDATIONS,
        desired_locale)

    solution_spec = "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
    validation_properties = utils.readJSONFile(solution_spec)["cisco_fabric"]
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]

    # Loop through all CISCO switches and perform all the validations
    # such as firmware, reachability etc...
    for fabric in get_fabrics():
        if fabric.get('fc_fabric_type') == "CISCOFABRIC":
            ciscoswitch = CiscoFabric(fabric.get('fabric_name'),
                                      fabric.get('ip'),
                                      fabric.get('fc_fabric_user'),
                                      fabric.get('fc_fabric_password'),
                                      vsan=fabric.get('fc_fabric_vsan'),
                                      logger=logger)
            exception_flag = False

            # Check cisco switch reachability
            try:
                reachability_checker = ReachabilityChecker(ciscoswitch)
                (status, result) = reachability_checker.check()
                chkResultList.extend(result)
            except Exception:
                exception_flag = True

            # Check cisco switch Firmware
            if "firmware_level" in validation_properties:
                try:
                    expected_firmware = validation_properties["firmware_level"]
                    firmware_checker = FirmwareChecker(ciscoswitch,
                                                       expected_firmware)
                    (status, result) = firmware_checker.check()
                    chkResultList.extend(result)
                except Exception:
                    exception_flag = True

            # Check cisco switch zoning memory
            try:
                zoning_memory_space_checker = \
                    ZoningMemorySpaceChecker(ciscoswitch)
                (status, result) = zoning_memory_space_checker.check()
                chkResultList.extend(result)
            except Exception:
                exception_flag = True

            # Check cisco switch zoning role
            if "user_zoning_role" in validation_properties:
                if validation_properties["user_zoning_role"].lower() == "on":
                    try:
                        zoning_role_checker = \
                            UserZoningRoleChecker(ciscoswitch)
                        (status, result) = zoning_role_checker.check()
                        chkResultList.extend(result)
                    except Exception:
                        exception_flag = True

            if exception_flag:
                chkResultList.append(utils.checkResult(
                    fabric.get('ip'),
                    corrAction=CHECK_LOG,
                    msg=translate_message(
                        ttvCatalog.CISCO_DETAILS_RETRIEVE_FAILED,
                        desired_locale),
                    msgId=messageIDCatalog.CISCO_DETAILS_RETRIEVE_FAILED,
                    status=CodeToStatus_map[CRITICAL],
                    val_check_id=CISCO_FABRIC_EXCEPTION))

    logger.info(_("Consolidated Results: {0}".format(
        json.dumps({"ValidationResult": output_dict}))))
    print(json.dumps({"ValidationResult": output_dict}))
    return 0


if __name__ == "__main__":
    main()
