#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import json
import glob
import os
import six
from powervc_ttv.resources import DS8KStorageResource
from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import _
from powervc_ttv import translate_message

output_dict = {
    "ValidationID": "DS8KSTORAGE_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "ds8k_storage",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}

DS8K_STORAGE_REACHABILITY = "DS8K_STORAGE_REACHABILITY"
DS8K_STORAGE_MODEL = "DS8K_STORAGE_MODEL"
DS8K_STORAGE_MCTYPE = "DS8K_STORAGE_MCTYPE"
DS8K_STORAGE_FIRMWARE = "DS8K_STORAGE_FIRMWARE"
DS8K_STORAGE_VALIDATION = "DS8K_STORAGE_VALIDATION"
NETWORK_QUALITY_CHK = "NETWORK_QUALITY_CHK"
logFile = "ds8kstorage_validator.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)
chkLogger = utils.checkerLogger(log_file)
logger = chkLogger.getLogger()


def main_func():
    rc = 0
    global desired_locale
    args = utils.parse_arguments()
    logger.debug("Checking on expected desired locale ..")
    desired_locale = args.locale if args is not None else None
    logger.debug(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.DS8K_STORAGE_VALIDATIONS,
        desired_locale)
    logger.debug("Executing %s on registered storages.."
                 % ttvCatalog.DS8K_STORAGE_VALIDATIONS)
    logger.debug("Reading information on registered storages ..")
    resource_dict = readConf(logger)

    validationProp = utils.readJSONFile(
        "/opt/ibm/powervc/ttv-validation/"
        "conf/solution_spec.json")["ds8k_storage"]

    logger.debug("Invoking validators ..")
    for resource in resource_dict:
        try:
            validateResource(validationProp, resource, logger)
        except Exception as e:
            logger.exception(_(e))

    logger.info(_("Consolidated Results: {0}".format(
        json.dumps({"ValidationResult": output_dict}))))
    print(json.dumps({"ValidationResult": output_dict}))
    return rc


class ConfigFileError(Exception):
    pass


class ReachabilityChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        logger.debug("Executing ReachabilityChecker for"
                     " address %s ..." % host)
        reachability = utils.checkReachability(host)
        if reachability == 0:
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 DS8K_STORAGE_REACHABILITY))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 DS8K_STORAGE_REACHABILITY))
        logger.debug(self.result[0]["msg"])
        return (status, self.result)


class MachineDetailChecker():

    def __init__(self, resource, expmodel, expmctype, expfirmware):
        self.resource = resource
        self.expmodel = expmodel
        self.expfirmware = expfirmware
        self.expmctype = expmctype
        self.result = []

    def check(self):
        status = ""
        rc = 1
        host = self.resource.ip
        logger.debug("Executing Model/MTM Checker on %s ..."
                     % host)
        err_message = ''
        mc_details = None
        curr_model = None
        curr_mc_type = None
        curr_firmware = None
        try:
            mc_details = self.resource.getModelFirmware()
        except Exception as e:
            err_message = six.text_type(e)
        if mc_details is not None:
            model_mc_type = mc_details["model"]
            curr_firmware = mc_details["microcode_version"]

            # split model_mc_type into model and machine type
            # Eg. For machine details 2425-961,
            # machine type is 961 and model is 2425

            try:
                list_details = model_mc_type.split("-")
                curr_model = list_details[0]
                curr_mc_type = list_details[1]
            except Exception as e:
                logger.error("Error in fetching supported "
                             "model/machine type %s" % e)

            logger.info(_("Current Model is = %s" % curr_model))
            logger.debug("Supported Models are = %s" % self.expmodel)
            logger.info(_("Current MachineType is = %s" % curr_mc_type))
            logger.debug("Supported MachineTypes are = %s" % self.expmctype)

        # Check for supported machine type  type #961, 980 etc
        if curr_mc_type is not None and \
                curr_mc_type in self.expmctype.split(","):
            rc = 0
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.DS8K_MCTYPE_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.DS8K_MCTYPE_SUCCESS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 DS8K_STORAGE_MCTYPE))
        else:
            if curr_mc_type is not None:
                camsgplaceholders = curr_mc_type
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.DS8K_MCTYPE_FAIL,
                    desired_locale)
                correctiveAction = translate_message(
                    ttvCatalog.CA_DS8K_MCTYPE_FAIL,
                    desired_locale)
                msgplaceholders = [self.expmctype]
            else:
                camsgplaceholders = []
                ttvcatalogmsgID = err_message
                correctiveAction = translate_message(
                    ttvCatalog.VALIDATION_EXCEPTION,
                    desired_locale)
                msgplaceholders = []
            messageID = messageIDCatalog.DS8K_MCTYPE_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 DS8K_STORAGE_MCTYPE))
            logger.info(_(self.result[0]["msg"]))

        if rc == 0:
            # Check for valid model type #961, 980 etc
            if curr_model is not None and \
                    curr_model in self.expmodel.split(","):
                camsgplaceholders = [host]
                correctiveAction = ""
                msgplaceholders = [host]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.DS8K_MODEL_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.DS8K_MODEL_SUCCESS
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     DS8K_STORAGE_MODEL))
            else:
                rc = 1
                if curr_model is not None:
                    camsgplaceholders = curr_model
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.DS8K_MODEL_FAIL,
                        desired_locale)
                    correctiveAction = translate_message(
                        ttvCatalog.CA_DS8K_MODEL_FAIL,
                        desired_locale)
                    msgplaceholders = [self.expmodel]
                else:
                    camsgplaceholders = []
                    ttvcatalogmsgID = err_message
                    correctiveAction = translate_message(
                        ttvCatalog.VALIDATION_EXCEPTION,
                        desired_locale)
                    msgplaceholders = []
                messageID = messageIDCatalog.DS8K_MODEL_FAIL
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     DS8K_STORAGE_MODEL))
            logger.info(_(self.result[0]["msg"]))

        # Check for firmware levels only if model & machine type is supported
        if rc == 0:
            if utils.compareVersion(curr_firmware, self.expfirmware):
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = [host]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.DS8K_FIRMWARE_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.DS8K_FIRMWARE_SUCCESS
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     DS8K_STORAGE_FIRMWARE))
            else:
                if curr_firmware is not None:
                    camsgplaceholders = [curr_firmware, self.expfirmware]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.DS8K_MODEL_FAIL,
                        desired_locale)
                    correctiveAction = translate_message(
                        ttvCatalog.CA_DS8K_FIRMWARE_FAIL,
                        desired_locale)
                    msgplaceholders = [["microcode_version"], self.expfirmware]
                else:
                    camsgplaceholders = []
                    ttvcatalogmsgID = err_message
                    correctiveAction = translate_message(
                        ttvCatalog.VALIDATION_EXCEPTION,
                        desired_locale)
                    msgplaceholders = []
                messageID = messageIDCatalog.DS8K_FIRMWARE_FAIL
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     DS8K_STORAGE_FIRMWARE))
            logger.info(_("Current Firmware: %s" % curr_firmware))
            logger.debug("Supported Firmware(s): %s" % self.expfirmware)
            logger.info(_(self.result[0]["msg"]))
        return self.result


class NetworkQualityChecker():

    def __init__(self, ip):
        self.ip = ip
        self.result = []

    def check(self):
        status = ""
        host = self.ip
        logger.debug("Executing NetworkQuality Checker on %s ..."
                     % host)
        quality = utils.checkNetworkQuality(self.ip)
        quality = int(quality)

        if quality == 100:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_BAD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality >= 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_DEGRADED,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality < 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_GOOD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        logger.debug(self.result[0]["msg"])
        return (status, self.result)


def validateResource(validationProp, resource, logger=None):
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]
    exception = False
    host = resource["hostname"]
    ds8kresourcemgr = DS8KStorageResource.DS8KStorageResource(
        resource, logger)
    checkers = []
    """
    checkers.append(
        FirmwareChecker(ds8kresourcemgr, validationProp["firmware"]))
    """
    checkers.append(MachineDetailChecker(
        ds8kresourcemgr, validationProp["model"],
        validationProp["machine_type"],
        validationProp["firmware"]))

    status = ""
    result = []
    try:
        (status, result) = ReachabilityChecker(ds8kresourcemgr).check()
        chkResultList.extend(result)
        (status, result) = NetworkQualityChecker(host).check()
        chkResultList.extend(result)
    except Exception as e:
        logger.error("An error encountered when checking "
                     "for Reachability and/or NetworkQuality.")
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        messageID = ""
        msgSubstitues = ""
        status = "Critical"
        verbose = ""
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               DS8K_STORAGE_REACHABILITY))
        logger.exception(_(e))
    if status == "Success" and not exception:

        for checker in checkers:
            status = ""
            result = []

            try:
                result = checker.check()
                chkResultList.extend(result)
            except Exception as e:
                logger.exception(_(e))
                exception = True

    if exception:
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.CA_DS8K_DETAILS_RETRIEVE_FAILED,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.DS8K_DETAILS_RETRIEVE_FAILED,
            desired_locale)
        messageID = messageIDCatalog.DS8K_DETAILS_RETRIEVE_FAILED
        msgSubstitues = ""
        status = "Critical"
        verbose = "verbose"
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               DS8K_STORAGE_VALIDATION))


def readConf(logger):
    env = {}
    env["ds8k-storage"] = []
    for conf in glob.glob("/etc/cinder/cinder-*.conf"):
        if conf == "/etc/cinder/cinder-health.conf":
            continue
        try:
            logger.debug(_("reading cinder.conf file {0}".format(conf)))
            cinder_conf = ConfigParser.ConfigParser()
            cinder_conf.read(conf)
            if cinder_conf.has_option('DEFAULT', 'host_type'):
                host_type = cinder_conf.get('DEFAULT', 'host_type')
            if host_type == 'ds8k' or host_type == 'DS8K':
                ds8k_storage = {}
                # the san_ip may in fact be a hostname, so convert to IP as ECT
                # must have an IP address
                logger.debug("Getting storage details..")
                ip = cinder_conf.get("DEFAULT", "san_ip")
                ds8k_storage["hostname"] = cinder_conf.get("DEFAULT", "san_ip")
                if not ip:
                    logger.error(_(("Name resolution failure in %(conf)s \
                    file - name is %(san_ip)s")
                                   % dict(conf=conf, san_ip=cinder_conf.
                                          get("DEFAULT", "san_ip"))))
                ds8k_storage["ip"] = ip

                ds8k_storage["userid"] = \
                    cinder_conf.get("DEFAULT", "san_login")

                if cinder_conf.has_option("DEFAULT", "san_private_key"):
                    if os.path.exists(cinder_conf.
                                      get("DEFAULT", "san_private_key")):
                        ds8k_storage["key-file"] = \
                            cinder_conf.get("DEFAULT", "san_private_key")
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    ds8k_storage["passwd"] = \
                        cinder_conf.get("DEFAULT", "san_password")
                for prop in ('san_clustername', 'san_ssh_port'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        ds8k_storage[prop] = cinder_conf.get('DEFAULT', prop)
                logger.debug("Config info on storage %s gathered successfully "
                             % ip)
                env["ds8k-storage"].append(ds8k_storage)

        except ConfigParser.NoOptionError, e:
            logger.error(_("cinder.conf file %(conf)s error: %(err_msg)s"
                           % dict(conf=conf, err_msg=e)))
        except ConfigFileError, e:
            logger.warning(_(e))

    return env["ds8k-storage"]


def main():
    rc = 0
    # Parse the command line arguments
    rc = main_func()
    exit(rc)

if __name__ == "__main__":
    main()
