#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------

from powervc_ttv.resources.EMCVNXResource import EMCVNXResource
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import utils
from powervc_ttv.utils.exceptions import ConfigFileError
from powervc_ttv import _
import ConfigParser
import glob
import json
import os
from powervc_ttv import translate_message

logFile = "emc_vnx_validator.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)

EMC_VNX_NAVISECCLI_PATH = "EMC_VNX_NAVISECCLI_PATH"
EMC_VNX_REQUIRED_FIRMWARE = "EMC_VNX_REQUIRED_FIRMWARE"
EMC_VNX_FAST_CACHE_ENABLED = "EMC_VNX_FAST_CACHE_ENABLED"
EMC_VNX_FAST_SUPPORT = "EMC_VNX_FAST_SUPPORT"
EMC_VNX_DEDUPLICATION_SUPPORT = "EMC_VNX_DEDUPLICATION_SUPPORT"
EMC_VNX_COMPRESSION_SUPPORT = "EMC_VNX_COMPRESSION_SUPPORT"
EMC_VNX_CLONING_SNAPSHOT_ATTACHMENT_SUPPORT =\
    "EMC_VNX_CLONING_SNAPSHOT_ATTACHMENT_SUPPORT"
EMC_VNX_THINPROVISIONING_SUPPORT = "EMC_VNX_THINPROVISIONING_SUPPORT"
NETWORK_QUALITY_CHK = "NETWORK_QUALITY_CHK"


class NetworkQualityChecker():

    def __init__(self, ip):
        self.ip = ip
        self.result = []

    def check(self):
        status = ""
        host = self.ip
        quality = utils.checkNetworkQuality(self.ip)
        quality = int(quality)

        if quality == 100:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_BAD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality >= 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_DEGRADED,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality < 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_GOOD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))

        return (status, self.result)


class emcVNXChecker():

    # Intakes EMC VNX ip, User id and password, keyFile and Expected VNX conf
    # for connecting and retrieving the VNX details
    def __init__(self, emcvnxip, emcvnxUser, emcvnxPasswd, keyFile,
                 expVNXConfDict, logger):
        self.emcvnxip = emcvnxip
        self.emcvnxUser = emcvnxUser
        self.emcvnxPasswd = emcvnxPasswd
        self.keyFile = keyFile
        self.expVNXConfDict = expVNXConfDict
        self.logger = logger

    def compare(self,
                actualVersion, expectedVersion,
                msgOk, msgCritical,
                corrAction, msgIDOk,
                msgIDCritical):
        """ Compares expected and actual firmware version.
        Returns the checkList with status(pass/fail)"""
        try:
            self.logger.info(_("expectedVersion {0}".format(expectedVersion)))
            self.logger.info(_("actualVersion {0}".format(actualVersion)))
            # Commenting below lines.. as this is not required.
            """
            actual = actualVersion.split(".")
            expected = expectedVersion.split(".")
            actual = [int(i) for i in actual]
            expected = [int(i) for i in expected]
            if len(actual) > len(expected):
                diff = len(actual) - len(expected)
                for i in range(0, diff):
                    expected.append(0)
            elif len(expected) > len(actual):
                diff = len(expected) - len(actual)
                for i in range(0, diff):
                    actual.append(0)"""
            result = utils.compareVersion(actualVersion, expectedVersion)
            if result:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = []
                ttvcatalogmsgID = msgOk
                messageID = msgIDOk
                msgSubstitues = ""
                verbose = ""
                status = "Success"
            else:
                camsgplaceholders = [expectedVersion]
                correctiveAction = corrAction
                msgplaceholders = []
                ttvcatalogmsgID = msgCritical
                messageID = msgIDCritical
                msgSubstitues = ""
                verbose = ""
                status = "Critical"
        except Exception as e:
            self.logger.exception(_("Error occurred while comparing" +
                                    "versions{0}".format(str(e))))
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVNX_FIRMWARE_ERROR,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_FIRMWARE_ERROR,
                desired_locale)
            messageID = messageIDCatalog.EMCVNX_FIRMWARE_ERROR
            msgSubstitues = ""
            verbose = ""
            status = "Critical"

        chkResult = utils.checkResult(self.emcvnxip,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      EMC_VNX_REQUIRED_FIRMWARE)
        return chkResult

    # Each validation check should implement this interface
    # to perform the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed
    # using __init__ method
    def check(self):

        chkResultList = []
        navisecclipath = self.expVNXConfDict["naviseccli_path"]
        self.vnxResource = EMCVNXResource(self.emcvnxip, self.emcvnxUser,
                                          self.emcvnxPasswd, self.keyFile,
                                          navisecclipath, self.logger)

        # Check for naviseccli_path
        if not self.vnxResource.checkNavisecCLIPathExists():
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_NAVSECCLIPATH_FAIL,
                desired_locale)
            msgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVNX_NAVSECCLIPATH_FAIL,
                desired_locale)
            camsgplaceholders = []
            messageID = messageIDCatalog.EMCVNX_NAVSECCLIPATH_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            chkResult = utils.checkResult(self.emcvnxip,
                                          correctiveAction,
                                          camsgplaceholders,
                                          ttvcatalogmsgID,
                                          msgplaceholders,
                                          messageID,
                                          msgSubstitues,
                                          status, verbose,
                                          EMC_VNX_NAVISECCLI_PATH)
            chkResultList.append(chkResult)
            return chkResultList
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_NAVSECCLIPATH_SUCCESS,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.EMCVNX_NAVSECCLIPATH_SUCCESS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            chkResult = utils.checkResult(self.emcvnxip,
                                          correctiveAction,
                                          camsgplaceholders,
                                          ttvcatalogmsgID,
                                          msgplaceholders,
                                          messageID,
                                          msgSubstitues,
                                          status, verbose,
                                          EMC_VNX_NAVISECCLI_PATH)
            chkResultList.append(chkResult)

        # Check for required_firmware
        actualVer = self.vnxResource.getFirmware()
        expectedVer = self.expVNXConfDict["required_firmware"]
        chkResult = self.compare(
            actualVer,
            str(expectedVer),
            translate_message(
                ttvCatalog.EMCVNX_FIRMWARE_SUCCESS,
                desired_locale),
            translate_message(
                ttvCatalog.EMCVNX_FIRMWARE_FAIL,
                desired_locale),
            translate_message(
                ttvCatalog.CA_EMCVNX_FIRMWARE_FAIL,
                desired_locale),
            messageIDCatalog.EMCVNX_FIRMWARE_SUCCESS,
            messageIDCatalog.EMCVNX_FIRMWARE_FAIL)
        chkResultList.append(chkResult)

        # Check for licenses
        enablers = self.vnxResource.getLicensesDetails()

        if '-Compression' in enablers:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_COMPRESSION_LICENSE_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.EMCVNX_COMPRESSION_LICENSE_AVAILABLE
            msgSubstitues = ""
            status = "Success"
            verbose = ""
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_COMPRESSION_LICENSE_NOT_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVNX_COMPRESSION_LICENSE_NOT_AVAILABLE,
                desired_locale)
            camsgplaceholders = []
            messageID = messageIDCatalog.\
                EMCVNX_COMPRESSION_LICENSE_NOT_AVAILABLE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        chkResult = utils.checkResult(self.emcvnxip,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      EMC_VNX_COMPRESSION_SUPPORT)
        chkResultList.append(chkResult)

        if '-FAST' in enablers:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_FAST_LICENSE_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.EMCVNX_FAST_LICENSE_AVAILABLE
            msgSubstitues = ""
            status = "Success"
            verbose = ""
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_FAST_LICENSE_NOT_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVNX_FAST_LICENSE_NOT_AVAILABLE,
                desired_locale)
            camsgplaceholders = []
            messageID = messageIDCatalog.EMCVNX_FAST_LICENSE_NOT_AVAILABLE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        chkResult = utils.checkResult(self.emcvnxip,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      EMC_VNX_FAST_SUPPORT)
        chkResultList.append(chkResult)

        if '-Deduplication' in enablers:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_DEDUPLICATION_LICENSE_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.EMCVNX_DEDUPLICATION_LICENSE_AVAILABLE
            msgSubstitues = ""
            status = "Success"
            verbose = ""
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_DEDUPLICATION_LICENSE_NOT_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVNX_DEDUPLICATION_LICENSE_NOT_AVAILABLE,
                desired_locale)
            camsgplaceholders = []
            messageID = messageIDCatalog.\
                EMCVNX_DEDUPLICATION_LICENSE_NOT_AVAILABLE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        chkResult = utils.checkResult(self.emcvnxip,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      EMC_VNX_DEDUPLICATION_SUPPORT)
        chkResultList.append(chkResult)

        if '-ThinProvisioning' in enablers:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_THINPROVISIONING_LICENSE_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.\
                EMCVNX_THINPROVISIONING_LICENSE_AVAILABLE
            msgSubstitues = ""
            status = "Success"
            verbose = ""
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_THINPROVISIONING_LICENSE_NOT_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVNX_THINPROVISIONING_LICENSE_NOT_AVAILABLE,
                desired_locale)
            camsgplaceholders = []
            messageID = messageIDCatalog.\
                EMCVNX_THINPROVISIONING_LICENSE_NOT_AVAILABLE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        chkResult = utils.checkResult(self.emcvnxip,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      EMC_VNX_THINPROVISIONING_SUPPORT)
        chkResultList.append(chkResult)

        if '-FASTCache' in enablers:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_FAST_CACHE_LICENSE_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.EMCVNX_FAST_CACHE_LICENSE_AVAILABLE
            msgSubstitues = ""
            status = "Success"
            verbose = ""
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_FAST_CACHE_LICENSE_NOT_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVNX_FAST_CACHE_LICENSE_NOT_AVAILABLE,
                desired_locale)
            camsgplaceholders = []
            messageID = messageIDCatalog.\
                EMCVNX_FAST_CACHE_LICENSE_NOT_AVAILABLE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        chkResult = utils.checkResult(self.emcvnxip,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      EMC_VNX_FAST_CACHE_ENABLED)
        chkResultList.append(chkResult)

        if '-SnapView' in enablers:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVNX_CLONING_SNAPSHOT_ATTACH_LICENSE_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.\
                EMCVNX_CLONING_SNAPSHOT_ATTACH_LICENSE_AVAILABLE
            msgSubstitues = ""
            status = "Success"
            verbose = ""
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.
                EMCVNX_CLONING_SNAPSHOT_ATTACH_LICENSE_NOT_AVAILABLE,
                desired_locale)
            msgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.
                CA_EMCVNX_CLONING_SNAPSHOT_ATTACH_LICENSE_NOT_AVAILABLE,
                desired_locale)
            camsgplaceholders = []
            messageID = messageIDCatalog.\
                EMCVNX_CLONING_SNAPSHOT_ATTACH_LICENSE_NOT_AVAILABLE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        chkResult = utils.checkResult(
            self.emcvnxip,
            correctiveAction,
            camsgplaceholders,
            ttvcatalogmsgID,
            msgplaceholders,
            messageID,
            msgSubstitues,
            status, verbose,
            EMC_VNX_CLONING_SNAPSHOT_ATTACHMENT_SUPPORT)
        chkResultList.append(chkResult)
        return chkResultList


def readVNXExpectedConfiguration():
    expVNXConfDict = {}
    solution_spec = "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
    vnx_info = utils.readJSONFile(solution_spec)["emc_vnx"]

    expVNXConfDict["naviseccli_path"] = vnx_info["naviseccli_path"]
    expVNXConfDict["required_firmware"] = vnx_info["required_firmware"]
    expVNXConfDict["cloning_snapshot_attachment_support"] =\
        vnx_info["cloning_snapshot_attachment_support"]
    expVNXConfDict["compression_support"] = vnx_info["compression_support"]
    expVNXConfDict["deduplication_support"] = vnx_info["deduplication_support"]
    expVNXConfDict["fast_cache_enabled"] = vnx_info["fast_cache_enabled"]
    expVNXConfDict["fast_support"] = vnx_info["fast_support"]
    expVNXConfDict["thinprovisioning_support"] =\
        vnx_info["thinprovisioning_support"]

    return expVNXConfDict


def readConf(logger):
    env = {}
    env["emcvnx-storage"] = []
    for conf in glob.glob("/etc/cinder/cinder-*.conf"):
        if conf == "/etc/cinder/cinder-health.conf":
            continue
        try:
            logger.info(_("reading cinder.conf file {0}".format(conf)))
            cinder_conf = ConfigParser.ConfigParser()
            cinder_conf.read(conf)
            if cinder_conf.has_option('DEFAULT', 'host_type'):
                host_type = cinder_conf.get('DEFAULT', 'host_type').lower()
            if host_type == 'emc_vnx':
                emcnvx_storage = {}
                emcnvx_storage["hostname"] = cinder_conf.get(
                    "DEFAULT", "san_ip")
                ip = cinder_conf.get("DEFAULT", "san_ip")
                if not ip:
                    logger.error(_(("Name resolution failure in %(conf)s \
                    file - name is %(san_ip)s")
                                   % dict(conf=conf, san_ip=cinder_conf.
                                          get("DEFAULT", "san_ip"))))
                emcnvx_storage["ip"] = ip

                emcnvx_storage["userid"] = cinder_conf.get(
                    "DEFAULT", "san_login")
                emcnvx_storage["passwd"] = None
                emcnvx_storage["key-file"] = None
                if cinder_conf.has_option("DEFAULT",
                                          "storage_vnx_security_file_dir"):
                    if os.path.exists(cinder_conf.
                                      get("DEFAULT",
                                          "storage_vnx_security_file_dir")):
                        emcnvx_storage["key-file"] = utils.getClearTextString(
                            cinder_conf.get("DEFAULT",
                                            "storage_vnx_security_file_dir"))
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    emcnvx_storage["passwd"] = cinder_conf.get(
                        "DEFAULT", "san_password")
                for prop in ('storage_vnx_pool_names', 'san_ssh_port',
                             'naviseccli_path', 'host_uniqueid',
                             'volume_backend_name'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        emcnvx_storage[prop] = cinder_conf.get('DEFAULT', prop)
                env["emcvnx-storage"].append(emcnvx_storage)
        except ConfigParser.NoOptionError as e:
            logger.error(_("cinder.conf file %(conf)s error: %(err_msg)s"
                           % dict(conf=conf, err_msg=e)))
        except ConfigFileError as e:
            logger.warning(_(e))
        except Exception as e:
            logger.exception(_(str(e)))
    return env


def main():

    # Variable Initialization
    rc = 0
    output_dict = {
        "ValidationID": "EMC_VNX_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": "",
        "TargetedResourceType": "emc_vnx",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]

    # Logger Initialization
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)

    output_dict["Description"] = translate_message(
        ttvCatalog.EMC_VNX_VALIDATIONS,
        desired_locale)

    # Check for Powervc Edition
    powervc_edition = utils.getPowerVCEdition()
    if(powervc_edition.lower() != "hmc"):
        logger.info(_("PowerVC Edition is not HMC. The edition is {}".
                      format(powervc_edition)))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc

    logger.info(_("PowerVC Edition is {}".format(powervc_edition)))

    # Fetch all registered VNX Storage
    env = readConf(logger)
    logger.info(_("EMC VNX Storage Configuration details: {}".format(env)))
    listOfVNX = env["emcvnx-storage"]
    if(len(listOfVNX) == 0):
        logger.info(_("No VNX Storage is registered."))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    logger.info(_("No. of VNX Storage configured is {0}".format(
                len(listOfVNX))))

    # Read VNX Expected Configuration
    expVNXConfDict = readVNXExpectedConfiguration()

    for vnx in listOfVNX:
        emcvnxip = vnx["ip"]
        emcvnxUser = vnx["userid"]
        emcvnxPasswd = vnx["passwd"]
        keyFile = vnx["key-file"]
        logger.info(_("VNX Checker getting initialized for EMC VNX withIP"
                      "-i <{0}>".format(emcvnxip)))

        # Create object for each checker
        vnxChk = emcVNXChecker(emcvnxip,
                               emcvnxUser,
                               emcvnxPasswd,
                               keyFile,
                               expVNXConfDict,
                               logger)
        chkResultList = vnxChk.check()
        resultList.extend(chkResultList)
        (status, result) = NetworkQualityChecker(
            emcvnxip).check()
        resultList.extend(result)
    logger.info(_("Final Result is:-> {0}".format(
        json.dumps({"ValidationResult": output_dict}))))
    print(json.dumps({"ValidationResult": output_dict}))
    return rc

if __name__ == "__main__":
    rc = main()
    exit(rc)
