#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------

from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
import glob
import json
from powervc_ttv.resources.HMCResource import HMCResource
from powervc_ttv import _
from powervc_ttv import translate_message

logFile = "hmc_managed_host_val.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)

HMC_MANAGED_HOST_MAX_NO_OF_CECS = "HMC_MANAGED_HOST_MAX_NO_OF_CECS"
HMC_MANAGED_HOST_STATE = "HMC_MANAGED_HOST_STATE"
HMC_MANAGED_HOST_VIOS_CAPABLE = "HMC_MANAGED_HOST_VIOS_CAPABLE"
HMC_MANAGED_HOST_MLP_CAPABLE = "HMC_MANAGED_HOST_MLP_CAPABLE"
HMC_MANAGED_HOST_ALPM_CAPABLE = "HMC_MANAGED_HOST_ALPM_CAPABLE"
HMC_MANAGED_HOST_PROCESSOR = "HMC_MANAGED_HOST_PROCESSOR"
HMC_MANAGED_HOST_VALIDATION_ERROR = "HMC_MANAGED_HOST_VALIDATION_ERROR"
HMC_MANAGED_HOST_SRR_CAPABLE = "HMC_MANAGED_HOST_SRR_CAPABLE"


UNABLE_TO_QUERY_SRR_CAPABILITY = ttvCatalog.\
    UNABLE_TO_QUERY_POWERSERVER_PROPERTIES_SRR_CAPABLE_EXCEPTION


# Class implements HMC Managed Host checker
# Uses HMCResource Module internally.
# Which in turn makes use of K2 api's to fetch
# all the HMC details in one shot,
# which then is used for validating the required Attributes
# mentioned in Solution_spec.json
class hmcHostChecker():

    # Intakes HMC ip, user id and password
    # for connecting and retrieving the HMC details

    def __init__(self, hmcip, hmcUser, hmcPasswd,
                 expHMCHostValues, hostsUUID, timeout, logger):
        self.hmcip = hmcip
        self.hmcUser = hmcUser
        self.hmcPasswd = hmcPasswd
        self.expHMCHostValues = expHMCHostValues
        self.hostsUUID = hostsUUID
        self.timeout = timeout
        self.logger = logger

    def strValueCompare(self, hostIP, attributeTocheck,
                        expectedVal, actualVal,
                        msgOk, msgCritical,
                        corrAction, msgIDOk,
                        msgIDCritical, validatorID):
        """ Compares expected and actual String Values.
        Returns the checkList with status info"""
        self.logger.info(_("Checking HMC Hosts" + attributeTocheck + " Val"))
        if expectedVal.lower() == actualVal.lower():
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [actualVal]
            ttvcatalogmsgID = msgOk
            messageID = msgIDOk
            msgSubstitues = ""
            verbose = ""
            status = "Success"
        else:
            camsgplaceholders = []
            correctiveAction = corrAction
            msgplaceholders = [actualVal]
            ttvcatalogmsgID = msgCritical
            messageID = msgIDCritical
            msgSubstitues = ""
            verbose = ""
            status = "Critical"

        chkResult = utils.checkResult(hostIP,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      validatorID)
        return chkResult

    # Each validation check should implement this interface
    # to perform the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed
    # using __init__ method
    def check(self):
        chkResultList = []

        self.hmcResource =\
            HMCResource(self.hmcip, self.hmcUser,
                        self.hmcPasswd, self.logger, self.timeout,
                        self.hostsUUID)
        (rc, cmdOutput, cmdErr, hmcManagedResourceList) =\
            self.hmcResource.getHMCManagedResourcesDetails()

        # Maximum Number of hosts Validation.
        currentRegHosts = len(self.hostsUUID)
        expSupportedHosts = self.expHMCHostValues["hmc_max_number_of_servers"]
        if (currentRegHosts > expSupportedHosts):
            host = self.hmcip
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERSERVER_MAX_SUPPORTED_LIMIT_EXCEEDED,
                desired_locale)
            msgplaceholders = [currentRegHosts, expSupportedHosts]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERSERVER_MAX_SUPPORTED_LIMIT_EXCEEDED,
                desired_locale)
            messageID = messageIDCatalog.\
                POWERSERVER_RESOURCES_CONFIGURATION_INCORRECT
            msgSubstitues = ""
            verbose = ""
            status = "Critical"
            chkResult = utils.checkResult(host,
                                          correctiveAction,
                                          camsgplaceholders,
                                          ttvcatalogmsgID,
                                          msgplaceholders,
                                          messageID,
                                          msgSubstitues,
                                          status, verbose,
                                          HMC_MANAGED_HOST_MAX_NO_OF_CECS)
            chkResultList.append(chkResult)
        if rc == 0:
            for entry in hmcManagedResourceList:
                hostIP = entry["SystemName"]
                # State Check
                actualState = entry["State"]
                expState = self.expHMCHostValues["state"]
                chkResult =\
                    self.strValueCompare(
                        hostIP,
                        'state',
                        expState,
                        actualState,
                        translate_message(
                            ttvCatalog.POWERSERVER_OPERATING_STATE_MATCH,
                            desired_locale),
                        translate_message(
                            ttvCatalog.POWERSERVER_OPERATING_STATE_MISMATCH,
                            desired_locale),
                        translate_message(
                            ttvCatalog.CA_POWERSERVER_STATE_MISMATCH,
                            desired_locale),
                        messageIDCatalog.POWERSERVER_CONFIGURATION,
                        messageIDCatalog.
                        POWERSERVER_INCORRECT_CONFIGURATION,
                        HMC_MANAGED_HOST_STATE)
                chkResultList.append(chkResult)

                # VirtualIOServerCapable Check
                actualVal = entry["VirtualIOServerCapable"]
                expVal = self.expHMCHostValues["VirtualIOServerCapable"]
                chkResult =\
                    self.strValueCompare(
                        hostIP,
                        'VirtualIOServerCapable',
                        expVal,
                        actualVal,
                        translate_message(
                            ttvCatalog.POWERSERVER_IS_VIOS_CAPABLE,
                            desired_locale),
                        translate_message(
                            ttvCatalog.POWERSERVER_IS_NOT_VIOS_CAPABLE,
                            desired_locale),
                        translate_message(
                            ttvCatalog.CA_POWERSERVER_VIOS_CAPABLE_MISMATCH,
                            desired_locale),
                        messageIDCatalog.
                        POWERSERVER_CONFIGURATION,
                        messageIDCatalog.
                        POWERSERVER_INCORRECT_CONFIGURATION,
                        HMC_MANAGED_HOST_VIOS_CAPABLE)
                chkResultList.append(chkResult)

                # MicroLogicalPartitionCapable Check
                actualVal = entry["MicroLogicalPartitionCapable"]
                expVal = self.expHMCHostValues["MicroLogicalPartitionCapable"]
                chkResult =\
                    self.strValueCompare(
                        hostIP,
                        'MicroLogicalPartitionCapable',
                        expVal,
                        actualVal,
                        translate_message(
                            ttvCatalog.POWERSERVER_IS_MICRO_PARTITION_CAPABLE,
                            desired_locale),
                        translate_message(
                            ttvCatalog.
                            POWERSERVER_IS_NOT_MICRO_PARTITION_CAPABLE,
                            desired_locale),
                        translate_message(
                            ttvCatalog.
                            CA_POWERSERVER_MICRO_PARTITION_CAPABLE_MISMATCH,
                            desired_locale),
                        messageIDCatalog.
                        POWERSERVER_CONFIGURATION,
                        messageIDCatalog.
                        POWERSERVER_INCORRECT_CONFIGURATION,
                        HMC_MANAGED_HOST_MLP_CAPABLE)
                chkResultList.append(chkResult)

                # ActiveLogicalPartitionMobilityCapable Check
                actualState = entry["ActiveLogicalPartitionMobilityCapable"]
                expState = self.expHMCHostValues[
                    "ActiveLogicalPartitionMobilityCapable"]
                chkResult =\
                    self.strValueCompare(
                        hostIP,
                        'ActiveLogicalPartitionMobilityCapable',
                        expVal,
                        actualVal,
                        translate_message(
                            ttvCatalog.POWERSERVER_IS_ACTIVE_LPM_CAPABLE,
                            desired_locale),
                        translate_message(
                            ttvCatalog.POWERSERVER_IS_NOT_ACTIVE_LPM_CAPABLE,
                            desired_locale),
                        translate_message(
                            ttvCatalog.CA_POWERSERVER_LPM_MISMATCH,
                            desired_locale),
                        messageIDCatalog.
                        POWERSERVER_CONFIGURATION,
                        messageIDCatalog.
                        POWERSERVER_INCORRECT_CONFIGURATION,
                        HMC_MANAGED_HOST_ALPM_CAPABLE)
                chkResultList.append(chkResult)

                # HOST SRR capability check
                host_srr_capability = entry[
                    "PowerVMLogicalPartitionSimplifiedRemoteRestartCapable"]
                expectedVal = self.expHMCHostValues["HostSRRCapable"]
                if host_srr_capability:
                    if expectedVal.lower() == host_srr_capability.lower():
                        camsgplaceholders = []
                        correctiveAction = ""
                        msgplaceholders = [host_srr_capability]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.POWERSERVER_IS_SRR_CAPABLE,
                            desired_locale)
                        messageID = messageIDCatalog.POWERSERVER_CONFIGURATION
                        msgSubstitues = ""
                        verbose = ""
                        status = "Success"
                    else:
                        camsgplaceholders = []
                        correctiveAction = translate_message(
                            ttvCatalog.CA_POWERSERVER_SRR_CAPABLE_MISMATCH,
                            desired_locale)
                        msgplaceholders = [host_srr_capability]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.POWERSERVER_IS_NOT_SRR_CAPABLE,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            POWERSERVER_INCORRECT_CONFIGURATION
                        msgSubstitues = ""
                        verbose = ""
                        status = "Warning"
                else:
                    camsgplaceholders = []
                    correctiveAction = ""
                    msgplaceholders = [host_srr_capability]
                    ttvcatalogmsgID = translate_message(
                        UNABLE_TO_QUERY_SRR_CAPABILITY,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        POWERSERVER_INCORRECT_CONFIGURATION
                    msgSubstitues = ""
                    verbose = ""
                    status = "Warning"

                result = utils.checkResult(hostIP, correctiveAction,
                                           camsgplaceholders,
                                           ttvcatalogmsgID,
                                           msgplaceholders,
                                           messageID,
                                           msgSubstitues,
                                           status, verbose,
                                           "HOST_SRR_CAPABILITY")
                chkResultList.append(result)

                # Proc Types
                currProcTypes = entry[
                    "SupportedPartitionProcessorCompatibilityModes"]
                currProcTypes.split(" ")
                allowedProcTypes = self.expHMCHostValues["hmc_proc_type"]
                msg = ''
                for entry in allowedProcTypes:
                    msg = msg + ' ' + entry
                matchFound = False
                for expProc in allowedProcTypes:
                    if expProc in currProcTypes:
                        matchFound = True
                        break
                if matchFound:
                    camsgplaceholders = []
                    correctiveAction = ""
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.POWERSERVER_PROC_TYPE_MATCH,
                        desired_locale)
                    messageID = messageIDCatalog.POWERSERVER_CONFIGURATION
                    msgSubstitues = ""
                    verbose = ""
                    status = "Success"
                else:
                    camsgplaceholders = [msg]
                    correctiveAction = translate_message(
                        ttvCatalog.CA_POWERSERVER_PROC_TYPE_MISMATCH,
                        desired_locale)
                    msgplaceholders = [msg]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.POWERSERVER_PROC_TYPE_MISMATCH,
                        desired_locale)
                    messageID =\
                        messageIDCatalog.POWERSERVER_INCORRECT_CONFIGURATION
                    msgSubstitues = ""
                    verbose = ""
                    status = "Critical"

                chkResult = utils.checkResult(hostIP,
                                              correctiveAction,
                                              camsgplaceholders,
                                              ttvcatalogmsgID,
                                              msgplaceholders,
                                              messageID,
                                              msgSubstitues,
                                              status, verbose,
                                              HMC_MANAGED_HOST_PROCESSOR)
                chkResultList.append(chkResult)

        else:
            correctiveAction = ""
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VALIDATION_EXCEPTION,
                desired_locale)
            msgplaceholders = []
            messageID = messageIDCatalog.HMC_K2_CONFIGURATION_ERROR
            msgSubstitues = ""
            verbose = ""
            self.logger.error(_(cmdErr.strip()))
            status = "Critical"
            chkResult = utils.checkResult(self.hmcip,
                                          correctiveAction,
                                          camsgplaceholders,
                                          ttvcatalogmsgID,
                                          msgplaceholders,
                                          messageID,
                                          msgSubstitues,
                                          status, verbose,
                                          HMC_MANAGED_HOST_VALIDATION_ERROR)
            chkResultList.append(chkResult)

        return chkResultList


def readHMCMangedHostExpectedValues(logger):
    """ read expected values from solution_spec """
    retVal = 0
    expHMCHostValues = {}
    procType = []
    solution_spec = "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
    hmc_host_info = utils.readJSONFile(solution_spec)["hmc_manged_host"]

    maxNoOfServers = hmc_host_info["hmc_max_number_of_servers"]
    state = hmc_host_info["state"]
    VIOSCapable = hmc_host_info["VirtualIOServerCapable"]
    microLparCapable = hmc_host_info["MicroLogicalPartitionCapable"]
    activeLParCapable = hmc_host_info["ActiveLogicalPartitionMobilityCapable"]
    host_srr_capable = \
        hmc_host_info["PowerVMLogicalPartitionSimplifiedRemoteRestartCapable"]
    proc1 = hmc_host_info["hmc_proc_type1"]
    proc2 = hmc_host_info["hmc_proc_type2"]
    proc3 = hmc_host_info["hmc_proc_type3"]

    try:
        expMaxNoOfServers = int(maxNoOfServers)
        expHMCHostValues["hmc_max_number_of_servers"] = expMaxNoOfServers
    except ValueError:
        logger.error(_("Error: Max_no_of_servers in" +
                       "solution_spec file is not convertible to int"))
        retVal = 1

    expHMCHostValues["state"] = state
    expHMCHostValues["VirtualIOServerCapable"] = VIOSCapable
    expHMCHostValues["MicroLogicalPartitionCapable"] = microLparCapable
    expHMCHostValues["ActiveLogicalPartitionMobilityCapable"] =\
        activeLParCapable
    expHMCHostValues["HostSRRCapable"] = host_srr_capable

    procType.append(proc1)
    procType.append(proc2)
    procType.append(proc3)
    expHMCHostValues["hmc_proc_type"] = procType

    return (retVal, expHMCHostValues)


def readRegisteredHMCHosts():
    """ helper method to load HMC hosts information"""
    hmchosts = []
    if len(hmchosts) == 0:
        config_file_path = "/etc/nova"
        conffiles = glob.glob("{0}/nova-*.conf".format(config_file_path))

        for conffile in conffiles:
            with open(conffile, "r") as f:
                lines = f.read()
            uuid = getValue('powervm_mgr_uuid', lines)
            if uuid is not None:
                hmchosts.append(uuid)
    return hmchosts


def getValue(key, buffer):
    """ helper method to find the value of key in buffer"""
    match = None
    for line in buffer.splitlines():
        if key in line:
            match = line
            break
    if match is not None:
        return match[match.find("=") + 2:].strip()
    else:
        return None


def filterHost(hmcManagedALLHosts, hostsUUID):
    """ helper method to find actual hosts registered into PowerVC """
    hostsRegisteredIntoPowerVC = []
    for entry in hostsUUID:
        if entry in hmcManagedALLHosts:
            hostsRegisteredIntoPowerVC.append(entry)
    return hostsRegisteredIntoPowerVC


def prpareMappingBetweenHostsAndHMC(listOfHMCs, hostsUUID, logger):
    """ helper method to find mapping between Hosts and HMC """
    if (len(listOfHMCs) > 0):
        HMCCecsMappingDict = {}
        for entry in listOfHMCs:
            hmcip = entry["ip"]
            hmcUser = entry["userid"]
            hmcPasswd = entry["passwd"]
            hmcResource = HMCResource(hmcip, hmcUser, hmcPasswd, logger, 120)
            (hmcManagedALLHosts) = hmcResource.getHMCManagedAllHosts()
            HMCCecsMappingDict[hmcip] = filterHost(hmcManagedALLHosts,
                                                   hostsUUID)

        return HMCCecsMappingDict


def addFailureResult(output_dict):
    correctiveAction = ""
    camsgplaceholders = []
    ttvcatalogmsgID = translate_message(
        ttvCatalog.HMC_CONFIGURATION_EXCEPTION,
        desired_locale)
    msgplaceholders = []
    messageID = messageIDCatalog.HMC_CONFIGURATION_ERROR
    msgSubstitues = ""
    verbose = ""
    status = "Critical"
    chkResult = utils.checkResult("",
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  HMC_MANAGED_HOST_VALIDATION_ERROR)
    output_dict["ValidationDetail"]["ChecksList"].append(chkResult)
    return output_dict


def main():
    # Variable Initialization
    rc = 0
    output_dict = {
        "ValidationID": "HMC_MANAGED_HOST_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": "",
        "TargetedResourceType": "hmc",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]

    # Logger Initialization
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.HMC_MANAGED_HOST_VALIDATIONS,
        desired_locale)

    # Check for Powervc Edition
    powervc_edition = utils.getPowerVCEdition()
    if(powervc_edition.lower() != "hmc"):
        logger.info(_("PowerVC Edition is not HMC. The edition is {}".
                      format(powervc_edition)))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc

    logger.info(_("PowerVC Edition is {}".format(powervc_edition)))

    # Read the registered HMC Hosts through /etc/nova/nova-*
    hostsUUID = readRegisteredHMCHosts()
    if len(hostsUUID) == 0:
        logger.info(_("No HMC hosts are registered."))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc

    # Read HMC Managed Hosts expected Values
    (retVal, expHMCHostValues) = readHMCMangedHostExpectedValues(logger)
    logger.info(_("Expected values from config files.{0},".format(
                expHMCHostValues)))

    if retVal == 1:
        return (retVal)

    # Get HMC Information from DB Query and form listofHMCs.
    listOfHMCs = []
    try:
        env = utils.getAllHMCInfo()
        listOfHMCs = env["HMC"]
    except Exception as e:
        logger.error(_("Validation failed because of Exception "
                       "While retrieving info about HMC."))
        logger.exception(_(e))
        rc = 1
        output_dict = addFailureResult(output_dict)
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    logger.info(_("No. of HMCs configured in current environment is {0}".
                  format(len(listOfHMCs))))

    # Get the Mapping of HMC and Managed Host
    HMCCecsMappingDict = prpareMappingBetweenHostsAndHMC(listOfHMCs,
                                                         hostsUUID,
                                                         logger)

    # Iterate through listOfHMCs and
    # put the final validation results into validationResults[]
    if (len(listOfHMCs) > 0):
        for entry in listOfHMCs:
            hmcip = entry["ip"]
            hmcUser = entry["userid"]
            hmcPasswd = entry["passwd"]
            cecIds = HMCCecsMappingDict[hmcip]
            if len(cecIds) == 0:
                logger.error(_("Error : " +
                               "Not able to retrieve Hosts managed by" +
                               hmcip))
                continue
            logger.info(_("Host Checker getting initialized for HMC -i <{0}>"
                          .format(hmcip)))

            # Create object for each checker
            hmcHostChk = hmcHostChecker(hmcip,
                                        hmcUser,
                                        hmcPasswd,
                                        expHMCHostValues,
                                        cecIds,
                                        120,
                                        logger)

            chkResultList = hmcHostChk.check()
            resultList.extend(chkResultList)
        logger.info(_("Final Result is:-> {0}".format(
            json.dumps({"ValidationResult": output_dict}))))
        print(json.dumps({"ValidationResult": output_dict}))
    else:
        logger.info(_("HMC is not configured on Management Server."))
        print(json.dumps({"ValidationResult": output_dict}))
    exit(rc)

if __name__ == "__main__":
    main()
