#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------

from powervc_ttv.utils import utils
import json
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv.resources.HMCResource import HMCResource
from powervc_ttv import _
from powervc_ttv import translate_message

logFile = "hmc_version_val.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)

HMC_VERSION = "HMC_VERSION"
HMC_VERSION_VALIDATION_ERROR = "HMC_VERSION_VALIDATION_ERROR"
NETWORK_QUALITY_CHK = "NETWORK_QUALITY_CHK"


class NetworkQualityChecker():

    def __init__(self, ip):
        self.ip = ip
        self.result = []

    def check(self):
        status = ""
        host = self.ip
        quality = utils.checkNetworkQuality(self.ip)
        quality = int(quality)
        if quality == -1:
            return (status, None)
        if quality == 100:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_BAD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality >= 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_DEGRADED,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality < 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_GOOD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))

        return (status, self.result)


# Class implements HMC checker
# Makes use of K2 api's to fetch all the HMC details in one shot,
# which then is used for validating the expected version
class hmcChecker():
    # Intakes HMC ip, user id and password
    # for connecting and retrieving the HMC details

    def __init__(self, hmcip, hmcUser, hmcPasswd, expVersion, expRelease,
                 expMaintenance, timeout, logger):
        self.hmcip = hmcip
        self.hmcUser = hmcUser
        self.hmcPasswd = hmcPasswd
        self.expVersion = expVersion
        self.expRelease = expRelease
        self.expMaintenance = expMaintenance
        self.timeout = timeout
        self.logger = logger

    def versionCheck(self, actualVersion, actualRelease, actualMaintenance):
        """ Compares expected and actual HMC version and
        returns the checkList with status pass/fail"""
        expVer = self.expVersion + "." + self.expRelease + "." +\
            self.expMaintenance
        actVer = actualVersion + "." + actualRelease + "." +\
            actualMaintenance
        is_version_supported = utils.compareVersion(actVer, expVer)
        if is_version_supported:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.HMC_VERSION_SUCCESS,
                desired_locale)
            msgplaceholders = []
            correctiveAction = ""
            camsgplaceholders = []
            messageID = messageIDCatalog.HMC_CONFIGURATION
            msgSubstitues = ""
            status = "Success"
            verbose = ""
        else:
            ttvcatalogmsgID = translate_message(
                ttvCatalog.HMC_VERSION_FAILED,
                desired_locale)
            msgplaceholders = [actualVersion, actualRelease, actualMaintenance]
            correctiveAction = translate_message(
                ttvCatalog.CA_HMC_VERSION_FAILED,
                desired_locale)
            camsgplaceholders = [str(self.expVersion), str(self.expRelease),
                                 str(self.expMaintenance)]
            messageID = messageIDCatalog.HMC_CONFIGURATION_ERROR
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        chkResult = utils.checkResult(self.hmcip,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      HMC_VERSION)
        return chkResult

    # Each validation check should implement this interface
    # to perform the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed
    # using __init__ method
    def check(self):
        chkResult = {}
        # Create an object for HMCResource
        # And call the method getHMCVersionDetails()
        self.hmcResource = HMCResource(self.hmcip, self.hmcUser,
                                       self.hmcPasswd, self.logger,
                                       self.timeout)
        (rc, cmdOutput, cmdErr, hmcResource) =\
            self.hmcResource.getHMCVersionDetails()

        if rc == 0:
            # TODO Check for Null and Empty Values from K2 API..
            actualVersion = hmcResource['version']
            actualRelease = hmcResource['release']
            actualMaintenance = hmcResource['maintenance']
            chkResult = self.versionCheck(actualVersion, actualRelease,
                                          actualMaintenance)

        else:
            correctiveAction = translate_message(
                ttvCatalog.CA_HMC_CONFIGURATION,
                desired_locale)
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.HMC_VERSION_QUERY_EXCEPTION,
                desired_locale)
            msgplaceholders = [self.hmcip]
            messageID = messageIDCatalog.HMC_K2_CONFIGURATION_ERROR
            msgSubstitues = ""
            verbose = ""
            self.logger.error(_(cmdErr.strip()))
            status = "Critical"
            chkResult = utils.checkResult(self.hmcip,
                                          correctiveAction,
                                          camsgplaceholders,
                                          ttvcatalogmsgID,
                                          msgplaceholders,
                                          messageID,
                                          msgSubstitues,
                                          status, verbose,
                                          HMC_VERSION_VALIDATION_ERROR)
        return chkResult


def readHMCExpectedConfigValues():
    solution_spec = "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
    hmc_exp_info = utils.readJSONFile(solution_spec)["hmc"]
    expVer = hmc_exp_info["min_version_level"]
    expRel = hmc_exp_info["min_release_level"]
    expMaint = hmc_exp_info["min_maintenance_level"]
    return(expVer, expRel, expMaint)


def addFailureResult(output_dict):
    correctiveAction = ""
    camsgplaceholders = []
    ttvcatalogmsgID = translate_message(
        ttvCatalog.HMC_CONFIGURATION_EXCEPTION,
        desired_locale)
    msgplaceholders = []
    messageID = messageIDCatalog.HMC_CONFIGURATION_ERROR
    msgSubstitues = ""
    verbose = ""
    status = "Critical"
    chkResult = utils.checkResult("",
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  HMC_VERSION_VALIDATION_ERROR)
    output_dict["ValidationDetail"]["ChecksList"].append(chkResult)
    return output_dict


def main():

    # Variable Initialization
    rc = 0
    output_dict = {
        "ValidationID": "HMC_VERSION_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": "",
        "TargetedResourceType": "hmc",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]

    # Logger Initialization
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()

    args = utils.parse_arguments()
    global desired_locale
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.HMC_VERSION_VALIDATIONS,
        desired_locale)

    # Check for Powervc Edition
    powervc_edition = utils.getPowerVCEdition()
    if(powervc_edition.lower() != "hmc"):
        logger.info(_("PowerVC Edition is not HMC. The edition is {}".
                      format(powervc_edition)))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc

    logger.info(_("PowerVC Edition is {}".format(powervc_edition)))

    # Read the expected values from Solution_spec.json
    (expVer, expRel, expMaint) = readHMCExpectedConfigValues()
    logger.info(_("Expected values from config files. exp version: {0},"
                  "exp release: {1}, exp maint: {2}".format(
                      expVer, expRel, expMaint)))

    # Get HMC Information from DB Query and form listofHMCs.
    listOfHMCs = []
    try:
        env = utils.getAllHMCInfo()
        listOfHMCs = env["HMC"]
    except Exception as e:
        logger.error(_("Validation failed because of Exception "
                       "While retrieving info about HMC."))
        logger.exception(_(e))
        rc = 1
        output_dict = addFailureResult(output_dict)
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    logger.info(_("No. of HMCs configured in current environment is {0}".
                  format(len(listOfHMCs))))

    if (len(listOfHMCs) > 0):
        for entry in listOfHMCs:
            hmcip = entry["ip"]
            hmcUser = entry["userid"]
            hmcPasswd = entry["passwd"]
            logger.info(_("HMC Info: i<{0}> -u<{1}> -v<{2}> -r<{3}> -m<{4}>"
                          .format(hmcip, hmcUser, expVer, expRel, expMaint)))
            # Create object for each checker
            hmcChk = hmcChecker(hmcip,
                                hmcUser,
                                hmcPasswd,
                                expVer,
                                expRel,
                                expMaint,
                                120,
                                logger)

            chkResultDict = hmcChk.check()
            resultList.append(chkResultDict)

            (status, result) = NetworkQualityChecker(hmcip).check()
            if result is not None:
                resultList.extend(result)

        logger.info(_("Final Result is:-> {0}".format(
            json.dumps({"ValidationResult": output_dict}))))
        print(json.dumps({"ValidationResult": output_dict}))
    else:
        logger.info(_("HMC is not configured on Management Server."))
        print(json.dumps({"ValidationResult": output_dict}))
    exit(rc)

if __name__ == "__main__":
    main()
