#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------
from powervc_ttv.utils import utils
from base_validator import cmdHandler
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
import json
from powervc_ttv import _
from powervc_ttv import translate_message
import glob
import ConfigParser


mgt_svr_res_fs_spcae_not_enough =\
    messageIDCatalog.MANAGEMENT_SERVER_RESOURCE_FILESYSTEM_SPACE_NOT_ENOUGH

MANAGEMENT_SERVER_PROCESSOR_REQUIREMENT = ("MANAGEMENT_SERVER_"
                                           "PROCESSOR_REQUIREMENT")
MANAGEMENT_SERVER_MEMORY_REQUIREMENT = ("MANAGEMENT_SERVER_MEMORY_"
                                        "REQUIREMENT")
MANAGEMENT_SERVER_SWAP_SPACE_REQUIREMENT = ("MANAGEMENT_SERVER_SWAP_"
                                            "SPACE_REQUIREMENT")
MANAGEMENT_SERVER_FILE_SYSTEM_SPACE_REQUIREMENT = ("MANAGEMENT_SERVER_"
                                                   "FILE_SYSTEM_SPACE_"
                                                   "REQUIREMENT")
EXECUTOR_THREAD_POOL_SIZE_REQUIREMENT =\
    "EXECUTOR_THREAD_POOL_SIZE_REQUIREMENT"
EXECUTOR_THREAD_POOL_SIZE_DEFAULT_REQUIREMENT =\
    "EXECUTOR_THREAD_POOL_SIZE_DEFAULT_REQUIREMENT"

'''Get current status of resources
'''


class resourceManager():

    def __init__(self, logger, timeout):
        self.logger = logger
        self.cmdHandle = cmdHandler(self.logger, timeout)

    def getCpu(self):
        cmd = "lscpu | grep '^CPU(s):'"
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def getMem(self):
        cmd = "awk '$3 == \"kB\"{$2 = $2 / 1024 ** 2; $3 = \"GB\"; } 1' " +\
            "/proc/meminfo | column -t | grep MemTotal"
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def getDisk(self):
        cmd = "df --total -PkTml /var "
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def getSwap(self):
        cmd = " free -m | grep Swap | awk '{print $2}' "
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def _getHostIP(self):
        cmd = 'hostname -i'
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmd_out, cmd_err) = self.cmdHandle.cmd_run(cmd)
        ip = cmd_out.strip()
        return ip

    def get_arch_type(self):
        cmd = "arch"
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def get_capped_mode(self):
        cmd = "cat /proc/ppc64/lparcfg | grep capped"
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def get_shared_processor_mode(self):
        cmd = "cat /proc/ppc64/lparcfg | grep shared_processor_mode"
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def get_partition_entitled_capacity(self):
        cmd = "cat /proc/ppc64/lparcfg | grep partition_entitled_capacity"
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

    def get_partition_active_processors(self):
        cmd = "cat /proc/ppc64/lparcfg | grep partition_active_processors"
        self.logger.debug("Executing \"{0}\"".format(cmd))
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        return (rc, cmdOutput, cmdErr)

# Class implements system CPU count checker


'''Check for minimum processor requirement
'''


class cpuChecker():

    def __init__(self, logger, expCpu, timeout=120):
        self.logger = logger
        self.expCpuCount = int(expCpu)
        self.cmdHandle = cmdHandler(self.logger, timeout)

    def _validate_ppc_cpu(self, cpu_details):
        # Validation should pass if any one of the following matches.
        # 1. An uncapped shared processor virtual machine with 1.0 processor
        #    unit of entitled processing capacity and 2 virtual processors.
        # 2. A capped shared processor virtual machine with 2.0 or more
        #    processor units of entitled capacity and 2 or more
        #    virtual processors.
        # 3. A virtual machine with 2 or more dedicated processors.

        # Actual processor details
        shared_processor_mode = None
        mode = None
        ent = None

        if cpu_details.get("shared_processor_mode") == "1":
            shared_processor_mode = "shared"
        else:
            shared_processor_mode = "dedicated"
        if cpu_details.get("capped"):
            mode = "capped" if cpu_details.get("capped") == "1" else \
                "uncapped"
        if cpu_details.get("partition_entitled_capacity"):
            ent = float(cpu_details.get("partition_entitled_capacity"))
        processors = cpu_details.get("partition_active_processors")
        vCPUsOrCores = float(processors)

        # Expected processor details
        exp_ent_uncapped = 1.0
        exp_ent_capped = 2.0
        exp_vcpus_or_cores = 2

        # Flags
        ent_flag = False
        vcpu_flag = False
        success_flag = False

        if shared_processor_mode == "shared":
            if mode is None:
                # Assuming mode is None for VMs running on KVM.
                ent_flag = True
                # Assuming no need to check entitled capacity
                # if VM is running on KVM.
            elif mode == "uncapped":
                if ent is not None and ent >= exp_ent_uncapped:
                    ent_flag = True
            elif mode == "capped":
                if ent is not None and ent >= exp_ent_capped:
                    ent_flag = True

            if exp_vcpus_or_cores:
                if vCPUsOrCores >= exp_vcpus_or_cores:
                    vcpu_flag = True
        elif shared_processor_mode == "dedicated":
            if vCPUsOrCores >= exp_vcpus_or_cores:
                success_flag = True

        return ((ent_flag and vcpu_flag) or success_flag)

    # Each validation check should implement this interface to perform
    # the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed using
    # __init__ method
    # This method should return the dict created by _checkResult
    def check(self):
        correctiveAction = ""
        camsgplaceholders = []
        ttvcatalogmsgID = ""
        msgplaceholders = []
        messageID = ""
        msgSubstitues = ""
        status = ""
        verbose = ""
        val_check_id = MANAGEMENT_SERVER_PROCESSOR_REQUIREMENT
        resmangr = resourceManager(self.logger, 120)
        is_power_architecture = False
        (rc, cmdOutput, cmdErr) = resmangr.get_arch_type()
        if rc == 0:
            if "ppc" in cmdOutput.lower():
                is_power_architecture = True
        if not is_power_architecture:
            return None

        (rc1, cmdOutput1, cmdErr1) = resmangr.get_capped_mode()
        (rc2, cmdOutput2, cmdErr2) = resmangr.get_shared_processor_mode()
        (rc3, cmdOutput3, cmdErr3) = resmangr.get_partition_entitled_capacity()
        (rc4, cmdOutput4, cmdErr4) = resmangr.get_partition_active_processors()

        cpu_details = {}
        if cmdOutput1 is not None and cmdOutput1 != "":
            capped = cmdOutput1.split('=')[1].strip()
            cpu_details['capped'] = capped
        if cmdOutput2 is not None and cmdOutput2 != "":
            shared_processor_mode = cmdOutput2.split('=')[1].strip()
            cpu_details['shared_processor_mode'] = shared_processor_mode
        if cmdOutput3 is not None and cmdOutput3 != "":
            partition_entitled_capacity = cmdOutput3.split('=')[1].strip()
            cpu_details['partition_entitled_capacity'] = \
                partition_entitled_capacity
        if cmdOutput4 is not None and cmdOutput4 != "":
            partition_active_processors = cmdOutput4.split('=')[1].strip()
            cpu_details['partition_active_processors'] = \
                partition_active_processors
        self.logger.debug("Processor details: %s" % cpu_details)

        if self._validate_ppc_cpu(cpu_details):
            self.logger.info(_("All CPU requirements are met"))
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = []
            ttvcatalogmsgID =\
                translate_message(
                    ttvCatalog.POWERVC_SERVER_MINIMUM_CPU_CHECK_SUCCESS,
                    desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_MINIMUM_CPU_SPACE
            msgSubstitues = ""
            verbose = ""
            status = "Success"
        else:
            self.logger.info(_("CPU requirements not met"))
            camsgplaceholders = []
            correctiveAction =\
                translate_message(
                    ttvCatalog.CA_POWERVC_SERVER_MINIMUM_CPU_CHECK,
                    desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID =\
                translate_message(ttvCatalog.POWERVC_MIN_CPU_CHECK_FAILURE,
                                  desired_locale)
            messageID =\
                messageIDCatalog.\
                POWERVC_MIN_CPU_CHECK_FAILURE
            msgSubstitues = ""
            verbose = ""
            status = "Critical"
        chkResult = utils.checkResult(utils.get_host_name(),
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose, val_check_id)
        return chkResult


'''Check for minimum Swap space requirement
'''


class swapChecker():

    def __init__(self, logger, expSwap):
        self.logger = logger
        self.expSwap = int(expSwap)

    def check(self):
        flag = False
        correctiveAction = ""
        camsgplaceholders = []
        ttvcatalogmsgID = ""
        msgplaceholders = []
        messageID = ""
        msgSubstitues = ""
        status = ""
        verbose = ""
        val_check_id = MANAGEMENT_SERVER_SWAP_SPACE_REQUIREMENT
        # Retrieving Actual memory and compare against actuals.
        resmangr = resourceManager(self.logger, 120)
        (rc, cmdOutput, cmdErr) = resmangr.getSwap()
        if rc == 0:
            memSize = cmdOutput.split()[0]
            if int(memSize) >= self.expSwap:
                flag = True

            if flag is True:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.POWERVC_SERVER_MINIMUM_SWAP_SPACE_CHECK_SUCCESS,
                    desired_locale)
                messageID =\
                    messageIDCatalog.\
                    MANAGEMENT_SERVER_RESOURCE_MINIMUM_SWAP_SPACE
                msgSubstitues = ""
                verbose = ""
                status = "Success"
            else:
                camsgplaceholders = [str(self.expSwap)]
                correctiveAction =\
                    translate_message(
                        ttvCatalog.CA_POWERVC_SERVER_MINIMUM_SWAP_SPACE_CHECK,
                        desired_locale)
                msgplaceholders = [str(memSize), str(self.expSwap)]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.POWERVC_SERVER_MINIMUM_SWAP_SPACE_CHECK_FAILURE,
                    desired_locale)
                messageID =\
                    messageIDCatalog.\
                    MANAGEMENT_SERVER_RESOURCE_MINIMUM_SWAP_SPACE_NOT_ENOUGH
                msgSubstitues = ""
                verbose = ""
                status = "Warning"
        else:
            correctiveAction =\
                translate_message(
                    ttvCatalog.
                    CA_POWERVC_SERVER_MINIMUM_SWAP_SPACE_CHECK_EXCEPTION,
                    desired_locale)
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERVC_SERVER_MINIMUM_SWAP_SPACE_CHECK_EXCEPTION,
                desired_locale)
            msgplaceholders = []
            messageID = \
                messageIDCatalog.\
                POWERVC_SERVER_MIN_SWAP_SPACE_CHECK_FAILED
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.logger.error(cmdErr.strip())
            status = "Critical"

        chkResult = utils.checkResult(utils.get_host_name(),
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose, val_check_id)
        return chkResult


# Class implements system memory checker
'''Check for Minumum memory requirement
'''


class memChecker():

    def __init__(self, logger, expMem):
        self.logger = logger
        self.expMemSize = int(expMem)

    # Each validation check should implement this interface to
    # perform the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed using
    # __init__ method
    # This method should return the dict created by _checkResult
    def check(self):
        flag = False
        correctiveAction = ""
        camsgplaceholders = []
        ttvcatalogmsgID = ""
        msgplaceholders = []
        messageID = ""
        msgSubstitues = ""
        status = ""
        verbose = ""
        val_check_id = MANAGEMENT_SERVER_MEMORY_REQUIREMENT
        # Retrieving Actual memory and compare against actuals.
        resmangr = resourceManager(self.logger, 120)
        (rc, cmdOutput, cmdErr) = resmangr.getMem()
        if rc == 0:
            memSize = cmdOutput.split()[1]
            if float(memSize.strip()) >= self.expMemSize:
                flag = True

            if flag is True:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.POWERVC_SERVER_MINIMUM_MEMORY_CHECK_SUCCESS,
                    desired_locale)
                messageID =\
                    messageIDCatalog.MANAGEMENT_SERVER_RESOURCE_MINIMUM_MEMORY
                msgSubstitues = ""
                verbose = ""
                status = "Success"
            else:
                camsgplaceholders = [str(self.expMemSize)]
                correctiveAction = translate_message(
                    ttvCatalog.CA_POWERVC_SERVER_MINIMUM_MEMORY_CHECK,
                    desired_locale)
                msgplaceholders = [str(memSize), str(self.expMemSize)]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.POWERVC_SERVER_MINIMUM_MEMORY_CHECK_FAILURE,
                    desired_locale)
                messageID =\
                    messageIDCatalog.\
                    MANAGEMENT_SERVER_RESOURCE_MINIMUM_MEMORY_NOT_ENOUGH
                msgSubstitues = ""
                verbose = ""
                status = "Warning"
        else:
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERVC_SERVER_MINIMUM_MEMORY_CHECK_EXCEPTION,
                desired_locale)
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERVC_SERVER_MINIMUM_MEMORY_CHECK_EXCEPTION,
                desired_locale)
            msgplaceholders = []
            messageID = messageIDCatalog.POWERVC_SERVER_MIN_MEMORY_CHECK_FAILED
            msgSubstitues = ""
            verbose = ""
            self.logger.error(cmdErr.strip())
            status = "Critical"

        chkResult = utils.checkResult(utils.get_host_name(),
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose, val_check_id)
        return chkResult


'''Check for minimum file system space requirement
'''


class diskChecker():

    def __init__(self, logger, expTotalSpace,
                 expRootSpace, expOptSpace, expTmpSpace):
        self.expTotalSpace = float(expTotalSpace)
        self.expRootSpace = float(expRootSpace)
        self.expOptSpace = float(expOptSpace)
        self.expTmpSpace = float(expTmpSpace)
        self.logger = logger

    def check(self):
        correctiveAction = ""
        camsgplaceholders = []
        ttvcatalogmsgID = ""
        msgplaceholders = []
        messageID = ""
        msgSubstitues = ""
        status = ""
        verbose = ""
        val_check_id = MANAGEMENT_SERVER_FILE_SYSTEM_SPACE_REQUIREMENT
        fileSystem_avalableSpace_dict = {}
        resmangr = resourceManager(self.logger, 120)
        (rc, cmdOutput, cmdErr) = resmangr.getDisk()
        if rc == 0:
            cmdOutputLines = cmdOutput.splitlines()
            for line in cmdOutputLines:
                # print line
                # result = (re.split(r'\s{1,}', line.strip()))
                result = line.strip().split()
                count = result.__len__()
                if count < 5:
                    continue
                if count == 6:
                    if result[5] == '/'\
                            or result[5] == '/tmp'\
                            or result[5] == '/opt':
                        availableSizeByte = float(result[3])
                        fileSystem_avalableSpace_dict[result[5]] =\
                            availableSizeByte
                else:
                    if result[6] == '/'\
                            or result[6] == '/tmp'\
                            or result[6] == '/opt':
                        availableSizeByte = float(result[4])
                        fileSystem_avalableSpace_dict[result[6]] =\
                            availableSizeByte

            if fileSystem_avalableSpace_dict is not None and\
                    len(fileSystem_avalableSpace_dict) > 0:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = []
                msgSubstitues = ""
                verbose = ""

                # Only one file system is available
                if len(fileSystem_avalableSpace_dict) == 1:
                    if self.expTotalSpace >\
                            float(fileSystem_avalableSpace_dict['/']):
                        correctiveAction = translate_message(
                            ttvCatalog.CA_FILESYSTEM_SIZE_CHECK_EXPAND,
                            desired_locale)
                        camsgplaceholders = [str(self.expTotalSpace)]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.FILESYSTEM_SIZE_CHECK_FAILURE,
                            desired_locale)
                        msgplaceholders =\
                            [fileSystem_avalableSpace_dict['/'],
                             self.expTotalSpace]
                        messageID = mgt_svr_res_fs_spcae_not_enough
                        msgSubstitues = ""
                        status = "Critical"
                        verbose = ""
                        self.logger.error(cmdErr.strip())
                    else:
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.FILESYSTEM_SIZE_CHECK_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            MANAGEMENT_SERVER_RESOURCE_FILESYSTEM_SPACE
                        status = "Success"
                # More file systems found
                elif len(fileSystem_avalableSpace_dict) > 1:
                    newFilureDescription = ""
                    if '/' in fileSystem_avalableSpace_dict:
                        if self.expRootSpace >\
                                float(fileSystem_avalableSpace_dict['/']):
                            newFilureDescription = "/ FS"

                    if '/opt' in fileSystem_avalableSpace_dict:
                        if self.expOptSpace >\
                                float(fileSystem_avalableSpace_dict['/opt']):
                            newFilureDescription = newFilureDescription\
                                + " /opt FS"

                    if '/tmp' in fileSystem_avalableSpace_dict:
                        if self.expTmpSpace >\
                                float(fileSystem_avalableSpace_dict['/tmp']):
                            newFilureDescription = newFilureDescription\
                                + " /tmp FS"

                    if newFilureDescription:
                        """
                        msgReplcmnt.append(newFilureDescription)
                        msgReplcmnt.append(str(self.expRootSpace))
                        msgReplcmnt.append(str(self.expOptSpace))
                        msgReplcmnt.append(str(self.expTmpSpace))
                        msg1 = "Free disk space is not sufficient under {0}.".\
                            format(newFilureDescription)
                        msg2 = " Expected / FS {0}, /opt FS {1}, /tmp FS {2}".\
                            format(str(self.expRootSpace),
                                   str(self.expOptSpace),
                                   str(self.expTmpSpace))
                        """
                        correctiveAction = translate_message(
                            ttvCatalog.
                            CA_LOCAL_HOST_FILESYSTEM_SIZE_CHECK_EXPAND.
                            desired_locale)
                        camsgplaceholders = [str(self.expRootSpace)]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.FILESYSTEM_SIZE_CHECK_FAILURE,
                            desired_locale)
                        msgplaceholders =\
                            [fileSystem_avalableSpace_dict['/'],
                             self.expTotalSpace]
                        messageID = mgt_svr_res_fs_spcae_not_enough
                        msgSubstitues = ""
                        status = "Critical"
                        verbose = ""
                        self.logger.error(cmdErr.strip())
            else:
                return None
        else:
            correctiveAction = translate_message(
                ttvCatalog.CA_FILESYSTEM_SIZE_CHECK_EXCEPTION,
                desired_locale)
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.FILESYSTEM_SIZE_CHECK_EXCEPTION,
                desired_locale)
            msgplaceholders = []
            messageID =\
                messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_FILESYSTEM_SPACE_NOT_ENOUGH
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.logger.error(cmdErr.strip())
            status = "Critical"

        chkResult = utils.checkResult(utils.get_host_name(),
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose, val_check_id)
        return chkResult


'''Check for executor_thread_pool_size requirement
'''


class threadPoolChecker():

    def __init__(self, logger):
        self.logger = logger

    def check(self):
        correctiveAction = ""
        camsgplaceholders = []
        ttvcatalogmsgID = ""
        msgplaceholders = []
        messageID = ""
        msgSubstitues = ""
        status = ""
        verbose = ""
        cinder_size = []
        nova_size = []
        val_check_id = EXECUTOR_THREAD_POOL_SIZE_REQUIREMENT
        config = ConfigParser.ConfigParser()
        # Get all cinder conf files and a list of the rpc_conn_pool_size vals
        for conf in glob.glob('/etc/cinder/cinder-*.conf'):
            if conf == '/etc/cinder/cinder-health.conf':
                continue
            config.readfp(open(conf))
            cinder_size.append(config.get('DEFAULT',
                                          'executor_thread_pool_size'))
        # Get all nova conf files and a list of the rpc_conn_pool_size vals
        for conf in glob.glob('/etc/nova/nova-*.conf'):
            if conf == '/etc/nova/nova-health.conf':
                continue
            config.readfp(open(conf))
            nova_size.append(config.get('DEFAULT',
                                        'executor_thread_pool_size'))
        # We need to check and see if there are any valid files first
        if cinder_size and nova_size:
            # The lowest cinder value must not be smaller than any nova value
            if min(cinder_size) >= max(nova_size):
                self.logger.info(_("All executor_thread_pool_size values are "
                                   "valid"))
                ttvcatalogmsgID =\
                    translate_message(ttvCatalog.
                                      CINDER_THREAD_POOL_SIZE_CHECK_SUCCESS,
                                      desired_locale)
                messageID = messageIDCatalog.CINDER_THREAD_POOL_CHECK_SUCCESS
                status = "Success"
            else:
                self.logger.info(_("One or more executor_thread_pool_size "
                                   "values may not be valid"))
                correctiveAction =\
                    translate_message(ttvCatalog.
                                      CA_CINDER_THREAD_POOL_SIZE_CHECK_FAILURE,
                                      desired_locale)
                ttvcatalogmsgID =\
                    translate_message(ttvCatalog.
                                      CINDER_THREAD_POOL_SIZE_CHECK_FAILURE,
                                      desired_locale)
                messageID = messageIDCatalog.CINDER_THREAD_POOL_CHECK_FAILURE
                status = "Warning"
        else:
            self.logger.info(_("All executor_thread_pool_size values are "
                               "valid"))
            ttvcatalogmsgID =\
                translate_message(ttvCatalog.
                                  CINDER_THREAD_POOL_SIZE_CHECK_SUCCESS,
                                  desired_locale)
            messageID = messageIDCatalog.CINDER_THREAD_POOL_CHECK_SUCCESS

            status = "Success"

        chkResult = utils.checkResult(utils.get_host_name(),
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      val_check_id)
        return chkResult


"""Check for executor_thread_pool_size default
"""


class threadPoolDefaultChecker():

    def __init__(self, logger):
        self.logger = logger

    def check(self):
        correctiveAction = ""
        camsgplaceholders = []
        ttvcatalogmsgID = ""
        msgplaceholders = []
        messageID = ""
        msgSubstitues = ""
        status = ""
        verbose = ""
        changed = False
        cinder_size = []
        nova_size = []
        val_check_id = EXECUTOR_THREAD_POOL_SIZE_DEFAULT_REQUIREMENT
        config = ConfigParser.ConfigParser()
        # Get cinder conf files and list of the executor_thread_pool_size vals
        for conf in glob.glob('/etc/cinder/cinder-*.conf'):
            if conf == '/etc/cinder/cinder-health.conf':
                continue
            config.readfp(open(conf))
            cinder_size.append(config.get('DEFAULT',
                                          'executor_thread_pool_size'))
        # Get nova conf files and list of the executor_thread_pool_size vals
        for conf in glob.glob('/etc/nova/nova-*.conf'):
            if conf == '/etc/nova/nova-health.conf':
                continue
            config.readfp(open(conf))
            nova_size.append(config.get('DEFAULT',
                                        'executor_thread_pool_size'))

        # Check values against default values
        for val in cinder_size:
            if int(val) != 256:
                changed = True
                break
        if not changed:
            for val in nova_size:
                if int(val) != 128:
                    changed = True
                    break

        # If a value has changed from the default value, warn
        if changed:
            self.logger.info(_("At least one cinder or nova configuration "
                               "property value for executor_thread_pool_size "
                               "has been changed from its default value"))
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(ttvCatalog.
                                                CINDER_THREAD_POOL_CHANGED,
                                                desired_locale)
            messageID = messageIDCatalog.CINDER_THREAD_POOL_VALUE_CHANGED
            msgSubstitues = ""
            verbose = ""
            status = "Warning"
        else:
            self.logger.info(_("All cinder and nova configuration property "
                               "values for executor_thread_pool_size are set "
                               "to their default values"))
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(ttvCatalog.
                                                CINDER_THREAD_POOL_UNCHANGED,
                                                desired_locale)
            messageID = messageIDCatalog.CINDER_THREAD_POOL_VALUE_UNCHANGED
            msgSubstitues = ""
            verbose = ""
            status = "Success"

        chkResult = utils.checkResult(utils.get_host_name(),
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      val_check_id)
        return chkResult


def main_func(expMem, expCpu, expTotFreeSpc, expRootFreeSpc,
              expOptFreeSpc, expTmpFreeSpc, expSwap, logger):
    rc = 0
    output_dict = {
        "ValidationID": "MANAGEMENT_SERVER_RESOURCES_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": translate_message(
                        ttvCatalog.MANAGEMENT_SERVER_RESOURCES_VALIDATIONS,
                        desired_locale),
        "TargetedResourceType": "Management Server",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]
#     logFile = "management_server_resources_val.log"
#     home_dir = "/var/log/ttv-validation/validators"
#     log_file = "{0}/{1}".format(home_dir.strip(), logFile)
#     chkLogger = utils.checkerLogger(log_file)
#     logger = chkLogger.getLogger()

    try:
        expMemSize = int(expMem)
    except ValueError:
        logger.error("Error: Memory size is not an integer")
        rc = 1
    try:
        expCpuCount = int(expCpu)
    except ValueError:
        logger.error("Error: CPU count is not an integer")
        rc = 1

    if rc == 1:
        return 1

    logger.info("User input: -m <{0}> -p <{1}>".format(str(expMemSize),
                                                       str(expCpuCount)))
    logger.info("User input: -m <{0}> -p <{1}> -a <{2}> -r <{3}> "
                "-o <{4}> -t <{5}>".
                format(str(expMemSize), str(expCpuCount),
                       str(expTotFreeSpc),
                       str(expRootFreeSpc),
                       str(expOptFreeSpc), str(expTmpFreeSpc)))

    # Create object for each checker
    swapChk = swapChecker(logger, expSwap)
    memChk = memChecker(logger, expMemSize)
    cpuChk = cpuChecker(logger, expCpuCount)
    diskChk = diskChecker(logger, expTotFreeSpc, expRootFreeSpc, expOptFreeSpc,
                          expTmpFreeSpc)
    tpChk = threadPoolChecker(logger)
    tpdChk = threadPoolDefaultChecker(logger)
    chkResultList = []
    chkResult = swapChk.check()
    chkResultList.append(chkResult)

    chkResult = memChk.check()
    chkResultList.append(chkResult)

    chkResult = cpuChk.check()
    if chkResult is not None:
        chkResultList.append(chkResult)

    chkResult = diskChk.check()
    if chkResult is not None:
        chkResultList.append(chkResult)

    chkResult = tpChk.check()
    chkResultList.append(chkResult)

    chkResult = tpdChk.check()
    chkResultList.append(chkResult)

    resultList.extend(chkResultList)
    logger.info("Final Result is:-> {0}".format(
        json.dumps({"ValidationResult": output_dict})))
    print(json.dumps({"ValidationResult": output_dict}))

    return rc


'''Reads minimum requirements from solution_spec.json
'''


def readManagemtServerConf():

    solution_spec = "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
    mngt_server_info = utils.readJSONFile(solution_spec)["management_server"]

    expCPUCount = mngt_server_info["cpucount"]
    expMemorysize = mngt_server_info["memorysize"]
    expTotalfs = mngt_server_info["totalfs"]
    expRootfs = mngt_server_info["rootfs"]
    expOptfs = mngt_server_info["optfs"]
    expTmpfs = mngt_server_info["tmpfs"]
    expSwap = mngt_server_info["swapsize"]

    return (expCPUCount, expMemorysize, expTotalfs, expRootfs, expOptfs,
            expTmpfs, expSwap)


def main():
    rc = 0
    (expCPUCount, expMemorysize, expTotalfs, expRootfs, expOptfs,
        expTmpfs, expSwap) = readManagemtServerConf()

    logFile = "management_server_resources_val.log"
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()

    args = utils.parse_arguments()
    global desired_locale
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)

    rc = main_func(expMemorysize, expCPUCount, expTotalfs, expRootfs, expOptfs,
                   expTmpfs, expSwap, logger)
    exit(rc)

if __name__ == "__main__":
    main()
