#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------

from powervc_ttv.utils import utils
import json
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv.resources.NOVALINKResource import NOVALINKResource
from powervc_ttv import _
from powervc_ttv import translate_message


logFile = "novalink_registration_val.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)

NOVALINK_PARTITION_REACHABILITY = "NOVALINK_PARTITION_REACHABILITY"
NOVALINK_PARTITION_CREDENTIAL = "NOVALINK_PARTITION_CREDENTIAL"
NOVALINK_REQUIRED_PROCESSORS = "NOVALINK_REQUIRED_PROCESSORS"
NOVALINK_REQUIRED_PROC_UNITS = "NOVALINK_REQUIRED_PROC_UNITS"
NOVALINK_REQUIRED_AVL_MEMORY = "NOVALINK_REQUIRED_AVL_MEMORY"
NOVALINK_REQUIRED_FIRMWARE = "NOVALINK_REQUIRED_FIRMWARE"
NOVALINK_VALIDATION_ERROR = "NOVALINK_VALIDATION_ERROR"
NOVALINK_COMPUTE_USER_PERMISSION = "NOVALINK_COMPUTE_USER_PERMISSION"
NOVALINK_NEUTRON_USER_PERMISSION = "NOVALINK_NEUTRON_USER_PERMISSION"
NOVALINK_VIOS_STATE = "NOVALINK_VIOS_STATE"
NOVALINK_VIOS_RMC_STATE = "NOVALINK_VIOS_RMC_STATE"
NOVALINK_SEA = "NOVALINK_SEA"
NOVALINK_MGMT_VSWITCH = "NOVALINK_MGMT_VSWITCH"


class CredentialChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        camsgplaceholders = []
        correctiveAction = ""
        msgSubstitues = ""
        verbose = ""
        ip = self.resource.ip
        host_disp_name = self.resource.host_disp_name
        is_credential_valid = self.resource.checkCredential()
        if is_credential_valid:
            msgplaceholders = [ip]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_VALID,
                desired_locale)
            messageID = messageIDCatalog.NOVALINK_PARTITION_CREDENTIAL_PASS
            status = "Success"
        else:
            camsgplaceholders = [ip]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [ip]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.NOVALINK_PARTITION_CREDENTIAL_FAIL
            status = "Critical"
        chk_result = utils.checkResult(
            host_disp_name,
            correctiveAction,
            camsgplaceholders,
            ttvcatalogmsgID,
            msgplaceholders,
            messageID,
            msgSubstitues,
            status, verbose,
            NOVALINK_PARTITION_CREDENTIAL)
        self.result.append(chk_result)
        return (is_credential_valid, self.result)


class ReachabilityChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []
        self.is_pingable = False

    def check(self):
        camsgplaceholders = []
        correctiveAction = ""
        msgSubstitues = ""
        verbose = ""
        ip = self.resource.ip
        host_disp_name = self.resource.host_disp_name
        reachability = utils.checkReachability(ip)
        if reachability == 0:
            self.is_pingable = True
            msgplaceholders = [ip]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = messageIDCatalog.NOVALINK_PARTITION_REACHABLE
            status = "Success"
        else:
            camsgplaceholders = [ip]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            msgplaceholders = [ip]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = messageIDCatalog.NOVALINK_PARTITION_NOT_REACHABLE
            status = "Critical"
        chk_result = utils.checkResult(
            host_disp_name,
            correctiveAction,
            camsgplaceholders,
            ttvcatalogmsgID,
            msgplaceholders,
            messageID,
            msgSubstitues,
            status, verbose,
            NOVALINK_PARTITION_REACHABILITY)
        self.result.append(chk_result)
        return (self.is_pingable, self.result)


class CPUChecker():

    def __init__(self, resource, novalink_exp_config):
        self.resource = resource
        self.novalink_exp_config = novalink_exp_config

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name
        actual_virtual_procs = None
        actual_proc_units = None
        exp_virtual_proc = self.novalink_exp_config[
            "required_virtual_processors"]
        exp_proc_units = self.novalink_exp_config[
            "required_proc_units"]
        '''validation for required no.of virtual processors'''
        returnCode, stdout, stderr = self.resource.get_actual_CPU_details()
        if returnCode == 0:
            stdoutput = [x for x in stdout if x]
            single_val_list = stdoutput[0].split(",")
            try:
                for entry in single_val_list:
                    val = entry.split("=")
                    if val[0].lower().strip() == "desired_virtual":
                        actual_virtual_procs = val[1]
                        break
            except Exception as exp:
                logger.exception(exp)
                logger.error("Exception occurred while parsing" +
                             "virtual processors.")
            try:
                for entry in single_val_list:
                    val = entry.split("=")
                    if val[0].lower().strip() == "desired_units":
                        actual_proc_units = val[1]
                        break
            except Exception as exp:
                logger.exception(exp)
                logger.error("Exception occurred while parsing" +
                             "proc units.")
            logger.info(_("Nova Link partition virtual processors:<{0}>"
                          .format(actual_virtual_procs)))
            logger.info(_("Nova Link partition proc units:<{0}>"
                          .format(actual_proc_units)))
            if actual_virtual_procs is not None and len(
                    actual_virtual_procs) > 0:
                camsgplaceholders = []
                correctiveAction = ''
                msgplaceholders = []
                ttvcatalogmsgID = ""
                messageID = ""
                msgSubstitues = ""
                status = ""
                verbose = ""
                if int(actual_virtual_procs) >= int(exp_virtual_proc):
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.
                        NOVALINK_PARTITION_MINIMUM_PROCESSOR_CHECK_SUCCESS,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        NOVALINK_PARTITION_MINIMUM_PROCESSOR_CHECK_SUCCESS
                    status = "Success"
                else:
                    camsgplaceholders = []
                    correctiveAction = translate_message(
                        ttvCatalog.
                        CA_NOVALINK_PARTITION_MINIMUM_PROCESSOR_CHECK_FAILURE,
                        desired_locale)
                    msgplaceholders = [int(actual_virtual_procs),
                                       int(exp_virtual_proc)]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.
                        NOVALINK_PARTITION_MINIMUM_PROCESSOR_CHECK_FAILURE,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        NOVALINK_PARTITION_MINIMUM_PROCESSOR_CHECK_FAILURE
                    status = "Critical"

                check_result = utils.checkResult(
                    host_disp_name,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    NOVALINK_REQUIRED_PROCESSORS)
                chkResultList.append(check_result)
            else:
                check_result = add_attribute_fetch_error(host_disp_name,
                                                         "virt_proc")
                chkResultList.append(check_result)

            if actual_proc_units is not None and len(actual_proc_units) > 0:
                camsgplaceholders = []
                correctiveAction = ''
                msgplaceholders = []
                ttvcatalogmsgID = ""
                messageID = ""
                msgSubstitues = ""
                status = ""
                verbose = ""
                if utils.compareVersion(actual_proc_units, exp_proc_units):
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.
                        NOVALINK_PARTITION_MINIMUM_PROC_UNIT_CHECK_SUCCESS,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        NOVALINK_PARTITION_MINIMUM_PROC_UNIT_CHECK_SUCCESS
                    status = "Success"
                else:
                    camsgplaceholders = []
                    correctiveAction = translate_message(
                        ttvCatalog.
                        CA_NOVALINK_PARTITION_MINIMUM_PROC_UNIT_CHECK_FAILURE,
                        desired_locale)
                    msgplaceholders = [actual_proc_units, exp_proc_units]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.
                        NOVALINK_PARTITION_MINIMUM_PROC_UNIT_CHECK_FAILURE,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        NOVALINK_PARTITION_MINIMUM_PROC_UNIT_CHECK_FAILURE
                    status = "Critical"

                check_result = utils.checkResult(
                    host_disp_name,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    NOVALINK_REQUIRED_PROC_UNITS)
                chkResultList.append(check_result)
            else:
                check_result = add_attribute_fetch_error(host_disp_name,
                                                         "proc_units")
                chkResultList.append(check_result)
        else:
            check_result = add_failure_result(host_disp_name, "CPU")
            chkResultList.append(check_result)
        return chkResultList


class MemoryChecker():

    def __init__(self, resource, novalink_exp_config):
        self.resource = resource
        self.novalink_exp_config = novalink_exp_config
        self.result = []

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name
        actual_mem_avl = None
        exp_avl_mem = self.novalink_exp_config[
            "required_available_memory"]
        # check against actual available memory
        returnCode, stdout, stderr =\
            self.resource.get_actual_available_memory()
        if returnCode == 0:
            stdoutput = [x for x in stdout if x]
            try:
                single_val_list = stdoutput[0].split(",")
                for entry in single_val_list:
                    val = entry.split("=")
                    if val[0].lower().strip() == "desired":
                        actual_mem_avl = val[1]
                        break
            except Exception as exp:
                logger.exception(exp)
                logger.error("Exception occurred while parsing" +
                             "available memory.")
            logger.info(_("Nova Link partition available memory:<{0}>"
                          .format(actual_mem_avl)))
            if actual_mem_avl is not None and len(actual_mem_avl) > 0:
                camsgplaceholders = []
                correctiveAction = ''
                msgplaceholders = []
                ttvcatalogmsgID = ""
                messageID = ""
                msgSubstitues = ""
                status = ""
                verbose = ""
                if int(actual_mem_avl) >= int(exp_avl_mem):
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.
                        NOVALINK_PARTITION_MIN_MEMORY_CHECK_SUCCESS,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        NOVALINK_PARTITION_MIN_MEMORY_CHECK_SUCCESS
                    status = "Success"
                else:
                    camsgplaceholders = [int(exp_avl_mem)]
                    correctiveAction = translate_message(
                        ttvCatalog.
                        CA_NOVALINK_PARTITION_MIN_MEMORY_CHECK_FAILURE,
                        desired_locale)
                    msgplaceholders = [int(actual_mem_avl),
                                       int(exp_avl_mem)]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.
                        NOVALINK_PARTITION_MIN_MEMORY_CHECK_FAILURE,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        NOVALINK_PARTITION_MIN_MEMORY_CHECK_FAILURE
                    status = "Critical"

                check_result = utils.checkResult(
                    host_disp_name,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    NOVALINK_REQUIRED_AVL_MEMORY)
                chkResultList.append(check_result)
            else:
                check_result = add_attribute_fetch_error(host_disp_name,
                                                         "avl_memory")
                chkResultList.append(check_result)
        else:
            check_result = add_failure_result(host_disp_name, "Memory")
            chkResultList.append(check_result)
        return chkResultList


class NovalinkPackagesChecker():

    def __init__(self, resource, req_pkg_cmd_list):
        self.resource = resource
        self.req_pkg_cmd_list = req_pkg_cmd_list

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name

        '''validations for packages installed on novalink host'''
        pkg_status_dict = self.resource.check_required_packages(
            self.req_pkg_cmd_list)
        logger.debug(pkg_status_dict)
        for pkg in pkg_status_dict:
            camsgplaceholders = []
            correctiveAction = ''
            msgSubstitues = ""
            verbose = ""
            val_chk_id = pkg.upper()
            if pkg_status_dict[pkg] == "INSTALLED":
                msgplaceholders = [pkg]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_PACKAGE_STATUS_PASS,
                    desired_locale)
                messageID = messageIDCatalog.NOVALINK_PACKAGE_STATUS_PASS
                status = "Success"
            elif pkg_status_dict[pkg] == "NOT_INSTALLED":
                correctiveAction = translate_message(
                    ttvCatalog.CA_NOVALINK_PACKAGE_STATUS_FAIL,
                    desired_locale)
                camsgplaceholders = [pkg]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_PACKAGE_STATUS_FAIL,
                    desired_locale)
                msgplaceholders = [pkg]
                messageID = messageIDCatalog.NOVALINK_PACKAGE_STATUS_FAIL
                if pkg.lower().strip() == "pvm-cli":
                    status = "Warning"
                else:
                    status = "Critical"
            elif pkg_status_dict[pkg] == "UNABLE_TO_DETERMINE_PKG_STATUS":
                correctiveAction = translate_message(
                    ttvCatalog.VALIDATION_EXCEPTION,
                    desired_locale)
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_PACKAGE_STATUS_EXCEPTION,
                    desired_locale)
                msgplaceholders = [pkg]
                messageID = messageIDCatalog.NOVALINK_PACKAGE_STATUS_EXCEPTION
                status = "Critical"

            check_result = utils.checkResult(host_disp_name,
                                             correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status,
                                             verbose,
                                             val_chk_id)
            chkResultList.append(check_result)
        return chkResultList


class NovalinkServicesChecker():

    def __init__(self, resource, req_service_cmd_list):
        self.resource = resource
        self.req_service_cmd_list = req_service_cmd_list

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name

        '''validations for services running on novalink host'''
        service_status_dict = self.resource.check_services_status(
            self.req_service_cmd_list)
        logger.debug(service_status_dict)
        for service in service_status_dict:
            camsgplaceholders = []
            correctiveAction = ''
            msgSubstitues = ""
            verbose = ""
            val_chk_id = service.upper()
            if "running" in service_status_dict[service]:
                msgplaceholders = [service, service_status_dict[service]]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_SERVICE_STATUS_PASS,
                    desired_locale)
                messageID = messageIDCatalog.NOVALINK_SERVICE_STATUS_PASS
                status = "Success"
            elif (("dead" in service_status_dict[service]) or
                  ("failed" in service_status_dict[service])):
                correctiveAction = translate_message(
                    ttvCatalog.CA_NOVALINK_SERVICE_STATUS_FAIL,
                    desired_locale)
                camsgplaceholders = [service, service]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_SERVICE_STATUS_FAIL,
                    desired_locale)
                msgplaceholders = [service, service_status_dict[service]]
                messageID = messageIDCatalog.NOVALINK_SERVICE_STATUS_FAIL
                status = "Critical"
            else:
                correctiveAction = translate_message(
                    ttvCatalog.VALIDATION_EXCEPTION,
                    desired_locale)
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_SERVICE_STATUS_EXCEPTION,
                    desired_locale)
                msgplaceholders = [service]
                messageID = messageIDCatalog.NOVALINK_SERVICE_STATUS_EXCEPTION
                status = "Critical"

            check_result = utils.checkResult(host_disp_name,
                                             correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status,
                                             verbose,
                                             val_chk_id)
            chkResultList.append(check_result)
        return chkResultList


class NovalinkUserPermissionChecker():

    def __init__(self, resource, novalink_exp_config):
        self.resource = resource
        self.novalink_exp_config = novalink_exp_config

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name.decode("utf-8")
        novalink_reg_user = self.resource.user
        exp_novalink_user_grp = self.novalink_exp_config[
            "novalink_user_group"]

        '''validations for services running on novalink host'''
        is_user_privileged = self.resource.is_user_privileged(
            exp_novalink_user_grp)
        if is_user_privileged:
            logger.debug("novalink user {0} has execute permission".format(
                novalink_reg_user))
        else:
            logger.debug("novalink user {0} does not have execute " +
                         "permission".format(novalink_reg_user))
        camsgplaceholders = []
        correctiveAction = ''
        msgSubstitues = ""
        verbose = ""
        if is_user_privileged:
            msgplaceholders = [novalink_reg_user, host_disp_name]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NOVALINK_USER_PERMISSION_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.NOVALINK_USER_PERMISSION_SUCCESS
            status = "Success"
        else:
            correctiveAction = translate_message(
                ttvCatalog.CA_NOVALINK_USER_PERMISSION_FAILURE,
                desired_locale)
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NOVALINK_USER_PERMISSION_FAILURE,
                desired_locale)
            msgplaceholders = [novalink_reg_user, host_disp_name]
            messageID = messageIDCatalog.NOVALINK_USER_PERMISSION_FAILURE
            status = "Warning"

        check_result = utils.checkResult(host_disp_name,
                                         correctiveAction,
                                         camsgplaceholders,
                                         ttvcatalogmsgID,
                                         msgplaceholders,
                                         messageID,
                                         msgSubstitues,
                                         status,
                                         verbose,
                                         NOVALINK_COMPUTE_USER_PERMISSION)
        chkResultList.append(check_result)
        return (is_user_privileged, chkResultList)


class NovalinkNeutronUserPermissionChecker():

    def __init__(self, resource, novalink_exp_config):
        self.resource = resource
        self.novalink_exp_config = novalink_exp_config

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name.decode("utf-8")
        novalink_neutron_user = self.resource.neutron_user
        exp_novalink_user_grp = self.novalink_exp_config[
            "novalink_user_group"]

        '''validations for neutron user privilege on novalink host'''
        is_user_privileged = self.resource.is_neutron_user_privileged(
            exp_novalink_user_grp)
        if is_user_privileged:
            logger.debug("novalink user {0} has execute permission".format(
                novalink_neutron_user))
        else:
            logger.debug("novalink user {0} does not have execute" +
                         "permission".format(novalink_neutron_user))
        camsgplaceholders = []
        correctiveAction = ''
        msgSubstitues = ""
        verbose = ""
        if is_user_privileged:
            msgplaceholders = [novalink_neutron_user, host_disp_name]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NOVALINK_USER_PERMISSION_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.NOVALINK_USER_PERMISSION_SUCCESS
            status = "Success"
        else:
            correctiveAction = translate_message(
                ttvCatalog.CA_NOVALINK_USER_PERMISSION_FAILURE,
                desired_locale)
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NOVALINK_USER_PERMISSION_FAILURE,
                desired_locale)
            msgplaceholders = [novalink_neutron_user, host_disp_name]
            messageID = messageIDCatalog.NOVALINK_USER_PERMISSION_FAILURE
            status = "Warning"

        check_result = utils.checkResult(host_disp_name,
                                         correctiveAction,
                                         camsgplaceholders,
                                         ttvcatalogmsgID,
                                         msgplaceholders,
                                         messageID,
                                         msgSubstitues,
                                         status,
                                         verbose,
                                         NOVALINK_NEUTRON_USER_PERMISSION)
        chkResultList.append(check_result)
        return (is_user_privileged, chkResultList)


class SEAChecker():

    def __init__(self, resource, novalink_exp_config):
        self.resource = resource
        self.novalink_exp_config = novalink_exp_config

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name
        do_SEAs_exist = False
        '''validation to check if SEA exists'''
        returnCode, stdout, stderr = self.resource.get_SEA_list()
        if returnCode == 0:
            try:
                if len(stdout) > 0:
                    do_SEAs_exist = True
            except Exception as exp:
                logger.exception(exp)
                logger.error("Exception occurred while parsing " +
                             "SEAs.")
                check_result = add_attribute_fetch_error(host_disp_name,
                                                         "SEAs")
                chkResultList.append(check_result)
            camsgplaceholders = []
            correctiveAction = ''
            msgplaceholders = []
            ttvcatalogmsgID = ""
            messageID = ""
            msgSubstitues = ""
            status = ""
            verbose = ""
            if do_SEAs_exist:
                logger.info(_("Nova Link partition has atleast one SEA"))
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.
                    NOVALINK_PARTITION_SEA_EXISTENCE_CHECK_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.\
                    NOVALINK_PARTITION_SEA_EXISTENCE_CHECK_SUCCESS
                status = "Success"
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog.
                    CA_NOVALINK_PARTITION_SEA_EXISTENCE_CHECK_FAILURE,
                    desired_locale)
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.
                    NOVALINK_PARTITION_SEA_EXISTENCE_CHECK_FAILURE,
                    desired_locale)
                messageID = messageIDCatalog.\
                    NOVALINK_PARTITION_SEA_EXISTENCE_CHECK_FAILURE
                status = "Critical"

            check_result = utils.checkResult(
                host_disp_name,
                correctiveAction,
                camsgplaceholders,
                ttvcatalogmsgID,
                msgplaceholders,
                messageID,
                msgSubstitues,
                status,
                verbose,
                NOVALINK_SEA)
            chkResultList.append(check_result)
        else:
            check_result = add_failure_result(host_disp_name, "SEA")
            chkResultList.append(check_result)
        return chkResultList


class MgmtVswitchChecker():

    def __init__(self, resource, novalink_exp_config):
        self.resource = resource
        self.novalink_exp_config = novalink_exp_config

    def check(self):
        chkResultList = []
        host_disp_name = self.resource.host_disp_name
        is_MGMTSWITCH_existing = False
        '''validation to check if MGMTSWITCH exists'''
        returnCode, stdout, stderr = self.resource.get_MGMTSWITCH()
        if returnCode == 0:
            try:
                for entry in stdout:
                    val = entry.split("=")
                    if val[1].strip() == "MGMTSWITCH":
                        is_MGMTSWITCH_existing = True
                        break
            except Exception as exp:
                logger.exception(exp)
                logger.error("Exception occurred while parsing " +
                             "virtual switches.")
                check_result = add_attribute_fetch_error(host_disp_name,
                                                         "MGMTSWITCH")
                chkResultList.append(check_result)
            camsgplaceholders = []
            correctiveAction = ''
            msgplaceholders = []
            ttvcatalogmsgID = ""
            messageID = ""
            msgSubstitues = ""
            status = ""
            verbose = ""
            if is_MGMTSWITCH_existing:
                logger.info(_("Nova Link partition has a MGMTSWITCH"))
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.
                    NOVALINK_PARTITION_MGMT_SWITCH_CHECK_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.\
                    NOVALINK_PARTITION_MGMT_SWITCH_CHECK_SUCCESS
                status = "Success"
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog.
                    CA_NOVALINK_PARTITION_MGMT_SWITCH_CHECK_FAILURE,
                    desired_locale)
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.
                    NOVALINK_PARTITION_MGMT_SWITCH_CHECK_FAILURE,
                    desired_locale)
                messageID = messageIDCatalog.\
                    NOVALINK_PARTITION_MGMT_SWITCH_CHECK_FAILURE
                status = "Warning"

            check_result = utils.checkResult(
                host_disp_name,
                correctiveAction,
                camsgplaceholders,
                ttvcatalogmsgID,
                msgplaceholders,
                messageID,
                msgSubstitues,
                status,
                verbose,
                NOVALINK_MGMT_VSWITCH)
            chkResultList.append(check_result)
        else:
            check_result = add_failure_result(host_disp_name, "MGMTSWITCH")
            chkResultList.append(check_result)
        return chkResultList


class VIOSRMCChecker():

    def __init__(self, resource, novalink_exp_config):
        self.resource = resource
        self.novalink_exp_config = novalink_exp_config

    def check(self):
        chkResultList = []
        camsgplaceholders = []
        correctiveAction = ''
        msgplaceholders = []
        msgSubstitues = ""
        verbose = ""

        host_disp_name = self.resource.host_disp_name
        exp_vios_state = self.novalink_exp_config["vios_state"]
        exp_vios_rmc_state = self.novalink_exp_config["vios_rmc_state"]

        '''validations for vios state and RMC State'''
        (is_error, vios_list) = self.resource.get_vios_rmc_state()
        if is_error:
            logger.debug("Error occurred while fetching vios details.")
            correctiveAction = translate_message(
                ttvCatalog.NOVALINK_VIOS_STATE_EXCEPTION,
                desired_locale)
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NOVALINK_VIOS_STATE_EXCEPTION,
                desired_locale)
            messageID = messageIDCatalog.NOVALINK_VIOS_EXCEPTION
            status = "Critical"

            check_result = utils.checkResult(host_disp_name,
                                             correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status,
                                             verbose,
                                             NOVALINK_VIOS_STATE)
            chkResultList.append(check_result)
            return chkResultList

        logger.debug("VIOS details: {0}".format(vios_list))
        if(len(vios_list) == 0):
            logger.debug("No Vios is configured.")

        for entry in vios_list:
            id = entry["id"]
            name = entry["name"]
            uuid = entry["uuid"]
            state = entry["state"]
            rmc_state = entry["rmc_state"]
            vios_name = host_disp_name + ":" + name

            if state.lower().strip() == exp_vios_state.lower().strip():
                msgplaceholders = [name, state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_VIOS_STATE_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.NOVALINK_VIOS_STATE_SUCCESS
                status = "Success"
            else:
                correctiveAction = translate_message(
                    ttvCatalog.CA_NOVALINK_VIOS_STATE_FAILURE,
                    desired_locale)
                camsgplaceholders = [name, exp_vios_state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_VIOS_STATE_FAILURE,
                    desired_locale)
                msgplaceholders = [name, state, exp_vios_state]
                messageID = messageIDCatalog.NOVALINK_VIOS_STATE_FAILURE
                status = "Critical"

            check_result = utils.checkResult(vios_name,
                                             correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status,
                                             verbose,
                                             NOVALINK_VIOS_STATE)
            chkResultList.append(check_result)

            if rmc_state.lower().strip() == exp_vios_rmc_state.lower().strip():
                msgplaceholders = [name, rmc_state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_VIOS_RMC_STATE_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.NOVALINK_VIOS_RMC_STATE_SUCCESS
                status = "Success"
            else:
                correctiveAction = translate_message(
                    ttvCatalog.CA_NOVALINK_VIOS_RMC_STATE_FAILURE,
                    desired_locale)
                camsgplaceholders = [name, exp_vios_rmc_state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.NOVALINK_VIOS_RMC_STATE_FAILURE,
                    desired_locale)
                msgplaceholders = [name, rmc_state, exp_vios_rmc_state]
                messageID = messageIDCatalog.NOVALINK_VIOS_RMC_STATE_FAILURE
                status = "Critical"

            check_result = utils.checkResult(vios_name,
                                             correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status,
                                             verbose,
                                             NOVALINK_VIOS_RMC_STATE)
            chkResultList.append(check_result)
        return chkResultList


def read_exp_conf_values():
    exp_novalink_values = {}

    solution_spec = "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
    novalink_reg_exp_info = utils.readJSONFile(solution_spec)[
        "novalink_registration"]

    exp_novalink_values["required_virtual_processors"] =\
        novalink_reg_exp_info["required_virtual_processors"]
    exp_novalink_values["required_proc_units"] =\
        novalink_reg_exp_info["required_proc_units"]
    exp_novalink_values["required_available_memory"] =\
        novalink_reg_exp_info["required_available_memory"]
    exp_novalink_values["packages_installed"] =\
        novalink_reg_exp_info["packages_installed"]
    exp_novalink_values["services"] =\
        novalink_reg_exp_info["services"]
    exp_novalink_values["novalink_user_group"] =\
        novalink_reg_exp_info["novalink_user_group"]
    exp_novalink_values["vios_state"] =\
        novalink_reg_exp_info["vios_state"]
    exp_novalink_values["vios_rmc_state"] =\
        novalink_reg_exp_info["vios_rmc_state"]

    return exp_novalink_values


def add_attribute_fetch_error(host, attr):
    correctiveAction = translate_message(
        ttvCatalog.VALIDATION_EXCEPTION,
        desired_locale)
    camsgplaceholders = []
    ttvcatalogmsgID = translate_message(
        ttvCatalog.NOVALINK_PARTITION_ATTRIBUTE_FETCH_ERROR,
        desired_locale)
    msgplaceholders = [attr]
    messageID = messageIDCatalog.NOVALINK_PARTITION_CONFIGURATION_ERROR
    msgSubstitues = ""
    verbose = ""
    status = "Critical"
    chkResult = utils.checkResult(host,
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  NOVALINK_VALIDATION_ERROR)
    return chkResult


def add_failure_result(host, msg):
    correctiveAction = translate_message(
        ttvCatalog.VALIDATION_EXCEPTION,
        desired_locale)
    camsgplaceholders = []
    ttvcatalogmsgID = translate_message(
        ttvCatalog.NOVALINK_PARTITION_CONFIGURATION_ERROR,
        desired_locale)
    msgplaceholders = [msg]
    messageID = messageIDCatalog.NOVALINK_PARTITION_CONFIGURATION_ERROR
    msgSubstitues = ""
    verbose = ""
    status = "Critical"
    chkResult = utils.checkResult(host,
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  NOVALINK_VALIDATION_ERROR)
    return chkResult


def prepare_pkg_cmd_list(novalink_exp_config):
    req_pkg_cmd_list = []
    pkg_list = novalink_exp_config["packages_installed"].split(",")
    for pkg in pkg_list:
        pkg_tuple = (pkg.strip(), "dpkg -s " + pkg.strip())
        req_pkg_cmd_list.append(pkg_tuple)
    return req_pkg_cmd_list


def prepare_service_cmd_list(novalink_exp_config):
    req_service_cmd_list = []
    service_list = novalink_exp_config["services"].split(",")
    for service in service_list:
        serice_tuple = (service.strip(), "systemctl status " + service.strip())
        req_service_cmd_list.append(serice_tuple)
    return req_service_cmd_list


def main():

    # Variable Initialization
    rc = 0
    output_dict = {
        "ValidationID": "NOVA_LINK_HOST_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": "",
        "TargetedResourceType": "NovaLink",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]

    # Logger Initialization
    chkLogger = utils.checkerLogger(log_file)
    global logger
    logger = chkLogger.getLogger()

    args = utils.parse_arguments()
    global desired_locale
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.NOVA_LINK_HOST_VALIDATIONS,
        desired_locale)

    # Fetch all nova link partition details
    env = utils.read_nova_conf(logger)
    list_of_novalink_partitions = env["novalink-partitions"]
    if(len(list_of_novalink_partitions) == 0):
        logger.info(_("Nova Link partition is not available."))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    logger.info(_("No. of Nova Link Partition configured is {0}".format(
                len(list_of_novalink_partitions))))

    # Read Nova Link expected Configuration
    novalink_exp_config = read_exp_conf_values()

    req_pkg_cmd_list = prepare_pkg_cmd_list(novalink_exp_config)

    req_service_cmd_list = prepare_service_cmd_list(novalink_exp_config)

    for novalink in list_of_novalink_partitions:
        logger.info(_("Nova Link validations getting initialized for"
                      " -i <{0}>".format(novalink["powervm_host_ip"])))

        nova_resource_obj = NOVALINKResource(novalink, logger)

        (is_pingable, chkResultList) =\
            ReachabilityChecker(nova_resource_obj).check()
        resultList.extend(chkResultList)

        if is_pingable:
            (is_cred_ok, chkResultList) =\
                CredentialChecker(nova_resource_obj).check()
            resultList.extend(chkResultList)

            if is_cred_ok:
                chkResultList = NovalinkPackagesChecker(
                    nova_resource_obj, req_pkg_cmd_list).check()
                resultList.extend(chkResultList)

                chkResultList = NovalinkServicesChecker(
                    nova_resource_obj, req_service_cmd_list).check()
                resultList.extend(chkResultList)

                (is_user_privileged, chkResultList) =\
                    NovalinkUserPermissionChecker(
                        nova_resource_obj, novalink_exp_config).check()
                resultList.extend(chkResultList)

                (is_neutron_user_privileged, chkResultList) =\
                    NovalinkNeutronUserPermissionChecker(
                        nova_resource_obj, novalink_exp_config).check()
                resultList.extend(chkResultList)

                if is_user_privileged:
                    chkResultList = CPUChecker(
                        nova_resource_obj, novalink_exp_config).check()
                    resultList.extend(chkResultList)

                    chkResultList = MemoryChecker(
                        nova_resource_obj, novalink_exp_config).check()
                    resultList.extend(chkResultList)

                    chkResultList = VIOSRMCChecker(
                        nova_resource_obj, novalink_exp_config).check()
                    resultList.extend(chkResultList)

                if is_neutron_user_privileged:
                    chkResultList = SEAChecker(
                        nova_resource_obj, novalink_exp_config).check()
                    resultList.extend(chkResultList)

                    chkResultList = MgmtVswitchChecker(
                        nova_resource_obj, novalink_exp_config).check()
                    resultList.extend(chkResultList)

    logger.info(_("Final Result is:-> {0}".format(
        json.dumps({"ValidationResult": output_dict}))))
    print(json.dumps({"ValidationResult": output_dict}))
    return rc

if __name__ == "__main__":
    rc = main()
    exit(rc)
