#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import ConfigParser
import glob
import json
import optparse
import sys
import time
from powervc_ttv.utils import utils
from powervc_ttv.resources.BrocadeFabricResource import BrocadeFabricResource\
    as brocadefabric_details
from powervc_ttv.resources.fabrics import CiscoFabric as ciscofabric_details
from powervc_ttv.resources.SVCStorageResource import SVCStorageResource\
    as svc_details
from powervc_ttv.resources.XIVStorageResource import XIVStorageResource\
    as xiv_details
from powervc_ttv.resources.EMCVMAXStorageResource import\
    EMCVMAXStorageResource as emc_vmax_details
from powervc_ttv.resources.EMCVNXResource import\
    EMCVNXResource as emc_vnx_details


LOG_FILENAME = 'scg_npiv_validation.log'
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), LOG_FILENAME)
chkLogger = utils.checkerLogger(log_file)
LOG = chkLogger.getLogger()

DEFAULT_SECTION = "DEFAULT"

scg_npiv_validations_dict = {}
scg_npiv_validations_list = []


'''
Read input conf file
'''


def readInputConfigFile(file):
    conf = glob.glob(file)
    validationProp = ConfigParser.ConfigParser()
    validationProp.read(conf)
    LOG.debug("********READING ALL STORAGE INFO FROM INPUT CONF FILE********")
    storage_types_list = validationProp.get("DEFAULT", "Storages").split(",")
    all_storage_info = {}
    for type in storage_types_list:
        storage_type_info_list = []
        count = int(validationProp.get("DEFAULT", type + "_count"))
        for num in range(1, count + 1):
            storage_info = {}
            storage_info["ip"] = validationProp.get(
                type + "_" + str(num), "ip")
            storage_info["user"] = validationProp.get(
                type + "_" + str(num), "user")
            storage_info["pwd"] = validationProp.get(
                type + "_" + str(num), "password")
            storage_info["key"] = validationProp.get(
                type + "_" + str(num), "key")
            storage_info["pool"] = validationProp.get(
                type + "_" + str(num), "pool")
            storage_info["port"] = validationProp.get(
                type + "_" + str(num), "port")
            storage_info["NavSecCLIPath"] = validationProp.get(
                type + "_" + str(num), "NavSecCLIPath") if validationProp.\
                has_option(type + "_" + str(num), "NavSecCLIPath") else None

            storage_type_info_list.append(storage_info)

        all_storage_info[type] = storage_type_info_list
    LOG.debug(
        "********DONE READING ALL STORAGE INFO FROM INPUT CONF FILE********")

    LOG.debug("********READING ALL FABRIC INFO FROM INPUT CONF FILE********")
    fabric_types_list = validationProp.get("DEFAULT", "fabrics").split(",")
    all_fabric_info = {}
    for type in fabric_types_list:
        fabric_type_info_list = []
        count = int(validationProp.get("DEFAULT", type + "_count"))
        for num in range(1, count + 1):
            fabric_info = {}
            fabric_info["ip"] = validationProp.get(
                type + "_" + str(num), "ip")
            fabric_info["user"] = validationProp.get(
                type + "_" + str(num), "user")
            fabric_info["pwd"] = validationProp.get(
                type + "_" + str(num), "password")
            fabric_info["key"] = validationProp.get(
                type + "_" + str(num), "key")
            fabric_info["pool"] = validationProp.get(
                type + "_" + str(num), "pool")
            fabric_info["port"] = validationProp.get(
                type + "_" + str(num), "port")
            fabric_info["vsan"] = validationProp.get(
                type + "_" + str(num), "vsan")
            fabric_info["fabric_name"] = validationProp.get(
                type + "_" + str(num), "fabric_name")
            fabric_type_info_list.append(fabric_info)

        all_fabric_info[type] = fabric_type_info_list
    LOG.debug(
        "********DONE READING ALL FABRIC INFO FROM INPUT CONF FILE********")

    LOG.debug("********READING ALL VIOS INFO FROM INPUT CONF FILE********")
    all_vios_info_list = []
    vios_count = int(validationProp.get("DEFAULT", "vios_count"))
    for num in range(1, vios_count + 1):
        vios_info = {}
        vios_info["Name"] = validationProp.get("vios_" + str(num), "Name")
        vios_info["Host"] = validationProp.get("vios_" + str(num), "Host")
        vios_info["SCGs"] = validationProp.get("vios_" + str(num), "SCGs")
        vios_info["WWPNs"] = validationProp.get("vios_" + str(num), "WWPNs")
        vios_info["fabrics"] = validationProp.get(
            "vios_" + str(num), "fabrics")
        all_vios_info_list.append(vios_info)
    LOG.debug(
        "********DONE READING ALL VIOS INFO FROM INPUT CONF FILE********")

    return (all_vios_info_list, all_storage_info, all_fabric_info)

'''
Validate the host FC physical connectivity with storage
'''


def validateSCGNPIVConnectivity(all_vios_info_list, all_storage_info,
                                all_fabric_info):
    '''(all_vios_info_list, all_storage_info,
     all_fabric_info) = readInputConfigFile(file)'''
    storage_fc_ports_dict = {}

    LOG.debug("********RETRIEVING ALL STORAGE SYSTEM FC PORT INFO********")
    for svc in all_storage_info["svc-storage"]:
        svcresource = svc_details(svc, LOG)
        LOG.debug("reading Storage FC ports for storage %s" % svc["hostname"])
        storageFCPorts = svcresource.getSystemFCPorts()
        storage_fc_ports_dict[svc["hostname"]] = {}
        storage_fc_ports_dict[svc["hostname"]]["type"] = 'SVC'
        storage_fc_ports_dict[svc["hostname"]]["wwpns"] = storageFCPorts

    for xiv in all_storage_info["xiv-storage"]:
        xivresource = xiv_details(xiv, LOG)
        LOG.debug("reading Storage FC ports for storage %s" % xiv["hostname"])
        storageFCPorts = xivresource.getSystemFCPorts()
        storage_fc_ports_dict[xiv["hostname"]] = {}
        storage_fc_ports_dict[xiv["hostname"]]["type"] = 'XIV'
        storage_fc_ports_dict[xiv["hostname"]]["wwpns"] = storageFCPorts

    for vnx in all_storage_info["emcvnx-storage"]:
        emcvnxip = vnx["hostname"]
        emcvnxUser = vnx["userid"]
        emcvnxPasswd = vnx["passwd"]
        keyFile = vnx["key-file"]
        navisecclipath = vnx['naviseccli_path']

        vnxresource = emc_vnx_details(emcvnxip, emcvnxUser,
                                      emcvnxPasswd, keyFile,
                                      navisecclipath, LOG)
        LOG.debug("reading Storage FC ports for storage %s" % vnx["hostname"])
        storageFCPorts = vnxresource.getSystemFCPorts()
        storage_fc_ports_dict[vnx["hostname"]] = {}
        storage_fc_ports_dict[vnx["hostname"]]["type"] = 'EMCVNX'
        storage_fc_ports_dict[vnx["hostname"]]["wwpns"] = storageFCPorts

    for vmax in all_storage_info["emcvmax-storage"]:
        LOG.debug("reading Storage FC ports for storage %s" % vmax["hostname"])
        vmaxresource = emc_vmax_details(vmax, LOG)
        storageFCPorts = vmaxresource.getSystemFCPorts()
        storage_fc_ports_dict[vmax["hostname"]] = {}
        storage_fc_ports_dict[vmax["hostname"]]["type"] = 'EMCVMAX'
        storage_fc_ports_dict[vmax["hostname"]]["wwpns"] = storageFCPorts
    LOG.debug(
        "********DONE RETRIEVING ALL STORAGE SYSTEM FC PORT INFO********")
    # LOG.debug(storage_fc_ports_dict)

    LOG.debug(
        "********RETRIEVING ALL"
        " LOGGED IN HOST FC PORT INFO ON EACH SWITCH********")
    fabric_dict_by_ip = {}
    fabric_dict_by_name = {}
    for fabric in all_fabric_info["FABRICS"]:
        if fabric.get("fc_fabric_type").lower() == "ciscofabric":
            LOG.debug("reading logged in host ports on switch %s" %
                      fabric['hostname'])
            ciscofabric = ciscofabric_details(fabric)
            lsfabric_dict = ciscofabric.get_loggedin_host_ports(fabric)
            fabric["type"] = "CISCOFABRIC"
            fabric_name = fabric['fabric_name'].upper()
            fabric_dict_by_name[fabric_name] = {}
            fabric_dict_by_name[fabric_name]["type"] = 'CISCOFABRIC'
            fabric_dict_by_name[fabric_name]["wwpns"] = lsfabric_dict
            fabric_dict_by_name[fabric_name]["ip"] = fabric["hostname"]
            fabric_dict_by_ip[fabric["hostname"]] = {}
            fabric_dict_by_ip[fabric["hostname"]]["type"] = 'CISCOFABRIC'
            fabric_dict_by_ip[fabric["hostname"]]["wwpns"] = lsfabric_dict
            fabric_dict_by_ip[fabric["hostname"]]["fabric_name"] = fabric_name

        elif fabric.get("fc_fabric_type").lower() == "brocadefabric":
            LOG.debug("reading logged in host ports on switch %s" %
                      fabric["hostname"])
            brocadefabric = brocadefabric_details(fabric, LOG)
            lsfabric_dict = brocadefabric.get_loggedin_host_ports()
            fabric["type"] = "BROCADEFABRIC"
            fabric_name = fabric['fabric_name'].upper()
            fabric_dict_by_name[fabric_name] = {}
            fabric_dict_by_name[fabric_name]["type"] = 'BROCADEFABRIC'
            fabric_dict_by_name[fabric_name]["wwpns"] = lsfabric_dict
            fabric_dict_by_name[fabric_name]["ip"] = fabric["hostname"]
            fabric_dict_by_ip[fabric["hostname"]] = {}
            fabric_dict_by_ip[fabric["hostname"]]["type"] = 'BROCADEFABRIC'
            fabric_dict_by_ip[fabric["hostname"]]["wwpns"] = lsfabric_dict
            fabric_dict_by_ip[fabric["hostname"]]["fabric_name"] = fabric_name

    LOG.debug(
        "********DONE RETRIEVING ALL LOGGED"
        " IN HOST FC PORT INFO ON EACH SWITCH********")
    # LOG.debug("Attached port info for all fabrics fabric_dict_by_ip")
    # LOG.debug(fabric_dict_by_ip)

    # LOG.debug("Attached port info for all fabrics fabric_dict_by_name")
    # LOG.debug(fabric_dict_by_name)

    LOG.debug(
        "********CHECKING THE CONNECTIVITY"
        " BETWEEN EACH STORAGE AND SWITCH ********")
    # LOG.debug("storage_fc_ports_dict")
    # LOG.debug(storage_fc_ports_dict)
    # LOG.debug("fabric_dict_by_ip")
    # LOG.debug(fabric_dict_by_ip)

    storage_connected_dict = {}

    for storage in storage_fc_ports_dict.keys():
        storage_connected_dict[storage] = {}
        connected_switch_list = []
        not_connected_switch_list = []
        storage_connected_dict[storage][
            "type"] = storage_fc_ports_dict.get(storage).get('type')
        storage_connected_dict[storage][
            "connected_switch_list"] = connected_switch_list
        storage_connected_dict[storage][
            "not_connected_switch_list"] = not_connected_switch_list
        storare_wwpn_set = storage_fc_ports_dict.get(storage).get('wwpns')

        ##############################################################
        for fabric in fabric_dict_by_ip.keys():
            fabric_wwpn_set = fabric_dict_by_ip.get(fabric).get("wwpns")
            matching_wwpns = list(
                set(fabric_wwpn_set).intersection(set(storare_wwpn_set)))
            if len(matching_wwpns) > 0:
                connected_switch_list.append(fabric)
            else:
                not_connected_switch_list.append(fabric)

        ##############################################################

    LOG.debug(
        "********DONE CHECKING THE CONNECTIVITY"
        " BETWEEN EACH STORAGE AND SWITCH ********")
    LOG.debug(storage_connected_dict)

    LOG.debug(
        "********CHECKING THE CONNECTIVITY"
        " BETWEEN EACH VIOS AND SWITCH ********")
    # LOG.debug("all_vios_info_list")
    # LOG.debug(all_vios_info_list)
    # LOG.debug("fabric_dict_by_name")
    # LOG.debug(fabric_dict_by_name)
    # LOG.debug("fabric_dict_by_ip")
    # LOG.debug(fabric_dict_by_ip)

    all_vios_wwpn_dict = {}
    for host in all_vios_info_list:
        vios = host.get("host") + ":" + host.get("name")
        all_vios_wwpn_dict[vios] = host.get("wwpns")

    all_vios_scg_dict = {}
    for host in all_vios_info_list:
        vios = host.get("host") + ":" + host.get("name")
        all_vios_scg_dict[vios] = host.get("scgs")["scg_name_list"]

    vios_connected_dict = {}
    vios_wwpns_connected_switch_dict = {}

    for host in all_vios_info_list:
        wwpns = host.get("wwpns")
        fabrics = host.get("fabrics")

        vios = host.get("host") + ":" + host.get("name")
        vios_connected_dict[vios] = {}
        vios_wwpns_connected_switch_dict[vios] = {}
        connected_switch_list = []
        not_connected_switch_list = []
        vios_connected_dict[vios][
            "connected_switch_list"] = connected_switch_list
        vios_connected_dict[vios][
            "not_connected_switch_list"] = not_connected_switch_list

        for i in range(len(wwpns)):
            wwpn = wwpns[i].upper()
            if fabric_dict_by_name.get(fabrics[i]):
                LOG.debug("corresponding fabric fabrics[i]")
                LOG.debug(fabrics[i])

                fabric_wwpn_set = fabric_dict_by_name.get(
                    fabrics[i]).get("wwpns")
                LOG.debug(
                    'corresponding fabric_wwpn_set in vios to switch check')
                # LOG.debug(fabric_wwpn_set)

                fabric = fabric_dict_by_name.get(fabrics[i]).get("ip")
                if wwpn in fabric_wwpn_set:
                    # vios_connected_switch_and_wwpns_list.add(wwpn)
                    # print "vios_connected_switch_and_wwpns_list",
                    # vios_connected_switch_and_wwpns_list
                    if vios_wwpns_connected_switch_dict.get(vios).get(fabric):
                        vios_wwpns_connected_switch_dict.get(
                            vios).get(fabric).add(wwpn)
                    else:
                        set_wwpn = set()
                        set_wwpn.add(wwpn)
                        vios_wwpns_connected_switch_dict.get(
                            vios)[fabric] = set_wwpn
                        # vios_connected_switch_and_wwpns_list
                    if fabric not in connected_switch_list:
                        connected_switch_list.append(fabric)
                        if fabric in not_connected_switch_list:
                            not_connected_switch_list.remove(fabric)
                    LOG.debug(
                        'VIOS ' + vios + ' wwpn ' + wwpn +
                        ' has connected to fabric ' + fabric)
                else:
                    if fabric not in not_connected_switch_list and\
                            fabric not in connected_switch_list:
                        not_connected_switch_list.append(fabric)
                    LOG.debug(
                        'VIOS ' + vios + ' wwpn ' + wwpn +
                        ' has not connected to fabric ' + fabric)

    LOG.debug(
        "********DONE CHECKING THE CONNECTIVITY"
        " BETWEEN EACH VIOS AND SWITCH ********")
    # LOG.debug("vios_connected_dict")
    # LOG.debug(vios_connected_dict)
    # LOG.debug("vios_wwpns_connected_switch_dict")
    # LOG.debug(vios_wwpns_connected_switch_dict)

    for vios in vios_connected_dict.keys():
        vios_connected_switch_list = vios_connected_dict.get(
            vios).get("connected_switch_list")
        if len(vios_connected_switch_list) > 0:
            for storage in storage_connected_dict.keys():
                storage_connected_switch_list = storage_connected_dict.get(
                    storage).get("connected_switch_list")
                type = storage_connected_dict.get(storage).get("type")
                if len(storage_connected_switch_list) > 0:
                    # print "storage_connected_switch_list",
                    # storage_connected_switch_list
                    matching_switches = list(
                        set(vios_connected_switch_list).
                        intersection(set(storage_connected_switch_list)))
                    LOG.debug(
                        "matching_switches between vios " + vios +
                        " and storage " + storage)
                    # LOG.debug(matching_switches)
                    LOG.debug(
                        "storage_connected_switch_list storage " + storage)
                    # LOG.debug(storage_connected_switch_list)
                    LOG.debug("vios_connected_switch_list vios " + vios)
                    LOG.debug(vios_connected_switch_list)
                    # print "matching_switches", matching_switches
                    if len(matching_switches) == 0:
                        LOG.debug("vios " + vios + " is not connected to " +
                                  type + "STORAGE " + storage +
                                  " via fabrics " +
                                  ",".join(vios_connected_switch_list))
                        scg_npiv_validations_dict[
                            vios + "!!!" + type + "STORAGE " + storage +
                            "!!!" + "NOTCONNECTED"] = ",".\
                            join(vios_connected_switch_list) + "!!!" + \
                            ",".join(all_vios_scg_dict.get(vios))

#                        LOG.debug( "vios " + vios + " is connectedto" + type +
#                         "STORAGE " + storage + " via fabrics " +
#                         ",".join(vios_connected_switch_list))
#                        scg_npiv_validations_dict[
#                            vios + "!!!" + type + "STORAGE " + storage
#                            + "!!!" + "CONNECTED"] = ",".\
#                            join(vios_connected_switch_list) + "!!!" + \
#                            ",".join(all_vios_scg_dict.get(vios))
                    elif len(matching_switches) > 0:
                        wwpns_list = []
                        for switch in matching_switches:
                            wwpns_list.append(
                                switch + "--" + ",".
                                join(list(vios_wwpns_connected_switch_dict.
                                          get(vios).get(switch))))
                        LOG.debug("vios " + vios + " is connected to " + type +
                                  "STORAGE " + storage +
                                  " via fabrics with WWPNs" +
                                  ",".join(wwpns_list))
                        scg_npiv_validations_dict[
                            vios + "!!!" + type + "STORAGE " + storage +
                            "!!!" + "CONNECTED"] = ",".join(wwpns_list) + \
                            "!!!" + \
                            ",".join(all_vios_scg_dict.get(vios)) + "!!!" + \
                            ",".\
                            join(matching_switches)
                        non_matching_switches = list(
                            set(vios_connected_switch_list) -
                            set(matching_switches))
                        LOG.debug(
                            "non_matching_switches between vios " +
                            vios + "and storage " + storage)
                        # LOG.debug(non_matching_switches)
                        if len(non_matching_switches) > 0:
                            LOG.debug("vios " + vios +
                                      " is not connected to " +
                                      type + "STORAGE " + storage +
                                      " via fabrics " +
                                      ",".join(non_matching_switches))
                            scg_npiv_validations_dict[
                                vios + "!!!" + type + "STORAGE " + storage +
                                "!!!" + "NOTCONNECTED"] = ",".\
                                join(non_matching_switches) + "!!!" + \
                                ",".join(all_vios_scg_dict.get(vios))
                else:
                    LOG.debug("vios " + vios + " is not connected to " +
                              type + "STORAGE " + storage +
                              " via fabrics " +
                              ",".join(vios_connected_switch_list))
                    scg_npiv_validations_dict[
                        vios + "!!!" + type + "STORAGE " + storage +
                        "!!!" + "NOTCONNECTED"] = ",".\
                        join(vios_connected_switch_list) + "!!!" + \
                        ",".join(all_vios_scg_dict.get(vios))
        else:
            # print "vios " + vios + " is not connected to any fabrics
            # registered with powervc"
            scg_npiv_validations_dict[
                vios + "!!!NOTCONNECTED"] = "None" + "!!!" +\
                ",".join(all_vios_scg_dict.get(vios))

#    for storage in storage_connected_dict.keys():
#        storage_connected_switch_list =  storage_connected_dict.\
# get(storage).get("connected_switch_list")
#        type = storage_connected_dict.get(storage).get("type")
#        if len(storage_connected_switch_list) == 0:
# print type + "STORAGE " + storage+"!!!NOTCONNECTED"
#            scg_npiv_validations_dict[type + "STORAGE " +\
# storage+"!!!NOTCONNECTED"] = "None"

    scg_npiv_validations_list.append(scg_npiv_validations_dict)
    scg_npiv_validations_dict_json = json.dumps(
        scg_npiv_validations_dict,
        sort_keys=True,
        indent=4)
    # LOG.debug("final result for scg_npiv_validations")
    # LOG.debug(scg_npiv_validations_dict_json)
    return scg_npiv_validations_dict_json


'''
main method for this program
'''


def my_main():
    parser = optparse.OptionParser()
    parser.add_option(
        '-f',
        '--propfile',
        dest='propfile',
        help='properties file',
        default=None)

    help_msg = "scg_npiv_validation.py -f propfile"

    (options, args) = parser.parse_args()

    if options.propfile is None:
        print "HELP : " + help_msg
        sys.exit(2)

    LOG.debug("********************************************************")
    LOG.debug("*************SCG NPIV Connectivity Validation****************")
    LOG.debug("********************************************************")
    propfile = options.propfile

    start_time = time.time()
    try:
        validateSCGNPIVConnectivity(propfile)
        stop_time = time.time()
        LOG.debug('total time taken for scg validation:' +
                  str(round(stop_time - start_time, 1)))
        sys.exit(0)
    except Exception as e:
        LOG.exception("Error while performing scg npiv validation:" + str(e))
        sys.exit(1)

if __name__ == '__main__':
    my_main()
