#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import ConfigParser
import glob
import json
import optparse
import sys
import time
from powervc_ttv.utils import utils
from powervc_ttv.resources.SVCStorageResource import\
    SVCStorageResource as svc_details
from powervc_ttv.resources.XIVStorageResource import\
    XIVStorageResource as xiv_details
from powervc_ttv.resources.EMCVMAXStorageResource import\
    EMCVMAXStorageResource as emc_vmax_details
from powervc_ttv.resources.EMCVNXResource import\
    EMCVNXResource as emc_vnx_details

LOG_FILENAME = 'scg_vscsi_validation.log'
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), LOG_FILENAME)
chkLogger = utils.checkerLogger(log_file)
LOG = chkLogger.getLogger()

DEFAULT_SECTION = "DEFAULT"

scg_validations_dict = {}
scg_validations_list = []


'''
Read input conf file
'''


def readInputConfigFile(file):
    conf = glob.glob(file)
    validationProp = ConfigParser.ConfigParser()
    validationProp.read(conf)
    '''read storage information'''
    storage_types_list = validationProp.get("DEFAULT", "Storages").split(",")
    all_storage_info = {}
    for type in storage_types_list:
        storage_type_info_list = []
        count = int(validationProp.get("DEFAULT", type + "_count"))
        for num in range(1, count + 1):
            storage_info = {}
            storage_info["ip"] = validationProp.get(
                type + "_" + str(num), "ip")
            storage_info["user"] = validationProp.get(
                type + "_" + str(num), "user")
            storage_info["pwd"] = validationProp.get(
                type + "_" + str(num), "password")
            storage_info["key"] = validationProp.get(
                type + "_" + str(num), "key")
            storage_info["pool"] = validationProp.get(
                type + "_" + str(num), "pool")
            storage_info["port"] = validationProp.get(
                type + "_" + str(num), "port")
            storage_info["NavSecCLIPath"] = validationProp.get(
                type + "_" + str(num), "NavSecCLIPath") if validationProp.\
                has_option(type + "_" + str(num), "NavSecCLIPath") else None

            storage_type_info_list.append(storage_info)

        all_storage_info[type] = storage_type_info_list

    '''read vios information'''
    all_vios_info_list = []
    vios_count = int(validationProp.get("DEFAULT", "vios_count"))
    for num in range(1, vios_count + 1):
        vios_info = {}
        vios_info["name"] = validationProp.get("vios_" + str(num), "name")
        vios_info["host"] = validationProp.get("vios_" + str(num), "host")
        vios_info["scgs"] = validationProp.get("vios_" + str(num), "scgs")
        vios_info["WWPNs"] = validationProp.get("vios_" + str(num), "WWPNs")
        all_vios_info_list.append(vios_info)
    return (all_vios_info_list, all_storage_info)

'''
Validate the host FC physical connectivity with storage
'''


def validateSCGVSCSIConnectivity(all_vios_info_list, all_storage_info):
    # (all_vios_info_list, all_storage_info) = readInputConfigFile(file)
    for svc in all_storage_info["svc-storage"]:
        LOG.debug("reading host wwwpns in emc vnx ip: %s" % svc["hostname"])
        svcresource = svc_details(svc, LOG)
        lsfabric_dict = svcresource.getlsfabric()
        LOG.debug("fc_host_connectivity_dict from VNX storage %s" %
                  svc["hostname"])
        LOG.debug(lsfabric_dict)
        validateWithSVC(
            svc["hostname"], all_vios_info_list, lsfabric_dict,
            scg_validations_dict)
    for xiv in all_storage_info["xiv-storage"]:
        LOG.debug("reading host wwwpns in emc vnx ip: %s" % xiv["hostname"])
        xivresource = xiv_details(xiv, LOG)
        fc_host_connectivity_dict = xivresource.getConnectivityList()
        LOG.debug("fc_host_connectivity_dict from XIV storage %s" %
                  xiv["hostname"])
        LOG.debug(fc_host_connectivity_dict)
        validateWithXIV(xiv["hostname"], all_vios_info_list,
                        fc_host_connectivity_dict, scg_validations_dict)
    for vnx in all_storage_info["emcvnx-storage"]:
        LOG.debug("reading host wwwpns in emc vnx ip: %s" % vnx["hostname"])
        emcvnxip = vnx["hostname"]
        emcvnxUser = vnx["userid"]
        emcvnxPasswd = vnx["passwd"]
        keyFile = vnx["key-file"]
        navisecclipath = vnx['naviseccli_path']

        vnxresource = emc_vnx_details(emcvnxip, emcvnxUser,
                                      emcvnxPasswd, keyFile,
                                      navisecclipath, LOG)
        fc_host_connectivity_dict = vnxresource.getConnectivityList()
        LOG.debug("fc_host_connectivity_dict from VNX storage %s" %
                  vnx["hostname"])
        LOG.debug(fc_host_connectivity_dict)
        validateWithEMCVNX(
            vnx["hostname"], all_vios_info_list, fc_host_connectivity_dict,
            scg_validations_dict)
    for vmax in all_storage_info["emcvmax-storage"]:
        LOG.debug("reading host wwwpns in vmax ip: %s" % vmax["hostname"])
        vmaxresource = emc_vmax_details(vmax, LOG)
        fc_host_connectivity_dict = vmaxresource.getConnectivityList()
        LOG.debug("fc_host_connectivity_dict from VMAX storage %s" %
                  vmax["hostname"])
        LOG.debug(fc_host_connectivity_dict)
        validateWithEMCVMAX(
            vmax["hostname"], all_vios_info_list, fc_host_connectivity_dict,
            scg_validations_dict)

    scg_validations_list.append(scg_validations_dict)
    scg_validations_dict_json = json.dumps(
        scg_validations_dict,
        sort_keys=True,
        indent=4)

    return scg_validations_dict_json


'''
Validate the host FC physical connectivity with XIV storage
'''


def validateWithXIV(
        svcip, all_vios_info_list, connectivity_dict, scg_validations_dict):

    for vios in all_vios_info_list:
        success_zoned_and_hostobj_wwpns = 0
        warning_zoned_but_no_hostobj_wwpns = 0
        warning_notzoned_wwpns = 0
        zoned_but_no_hostobj_wwpns_list = []
        warning_notzoned_wwpns_list = []
        success_zoned_and_hostobj_wwpns_list = []

        vios_wwpns = vios["wwpns"]
        vios_total_wwpns_count = len(vios_wwpns)
        host_wwpns = []
        host_wwpn_dict = connectivity_dict.get("host_wwpn_dict")
        for host in host_wwpn_dict.keys():
            for x in host_wwpn_dict.get(host):
                host_wwpns.append(x)
        LOG.debug('host_wwpns')
        LOG.debug(host_wwpns)
        LOG.debug(
            "Match each WWPN of vios with logged in host wwpns from storage")
        for wwpn in vios_wwpns:
            if wwpn in connectivity_dict["wwpn_set"]:
                if wwpn in host_wwpns:
                    success_zoned_and_hostobj_wwpns += 1
                    success_zoned_and_hostobj_wwpns_list.append(wwpn)
                else:
                    warning_zoned_but_no_hostobj_wwpns += 1
                    zoned_but_no_hostobj_wwpns_list.append(wwpn)
            else:
                warning_notzoned_wwpns = warning_notzoned_wwpns + 1
                warning_notzoned_wwpns_list.append(wwpn)

        if vios_total_wwpns_count == success_zoned_and_hostobj_wwpns:

            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!XIV!" +
                svcip + "!!!ALL_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif success_zoned_and_hostobj_wwpns > 0:
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!XIV!" +
                svcip + "!!!SOME_ZONED"] = \
                ",".join(vios["scgs"]["scg_name_list"]) +\
                "!!!" + \
                ",".join(success_zoned_and_hostobj_wwpns_list)

        if vios_total_wwpns_count == warning_notzoned_wwpns:
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!XIV!" + svcip +
                "!!!NONE_ZONED"] = ",".join(vios["scgs"]["scg_name_list"])

        elif warning_notzoned_wwpns > 0:
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!XIV!" + svcip +
                                 "!!!NOT_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"]) + \
                "!!!" + ",".join(warning_notzoned_wwpns_list)

        if vios_total_wwpns_count == warning_zoned_but_no_hostobj_wwpns:
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!XIV!" +
                svcip + "!!!ALL_ZONED_NO_HOST_OBJECT"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif warning_zoned_but_no_hostobj_wwpns > 0:
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!XIV!" + svcip +
                                 "!!!ZONED_NO_HOST_OBJECT"] = \
                ",".join(vios["scgs"]["scg_name_list"]) + "!!!" + \
                ",".join(zoned_but_no_hostobj_wwpns_list)


'''
Validate the host FC physical connectivity with EMC VNX storage
'''


def validateWithEMCVNX(
        emcvnxip, all_vios_info_list, connectivity_dict, scg_validations_dict):

    for vios in all_vios_info_list:
        success_zoned_and_hostobj_wwpns = 0
        warning_zoned_but_no_hostobj_wwpns = 0
        warning_notzoned_wwpns = 0
        zoned_but_no_hostobj_wwpns_list = set()
        warning_notzoned_wwpns_list = set()
        success_zoned_and_hostobj_wwpns_list = set()

        vios_wwpns = vios["wwpns"]
        vios_total_wwpns_count = len(vios_wwpns)
        host_wwpn_dict = connectivity_dict.get("host_wwpn_dict")
        LOG.debug(
            "Match WWPNs of vios with logged in host wwpns from VNX storage")
        for wwpn in vios_wwpns:
            for emcvnxhostwwpn in host_wwpn_dict.keys():
                if wwpn in host_wwpn_dict.get(emcvnxhostwwpn):
                    if emcvnxhostwwpn !=\
                            'empty_unknown_zoned_unconfigured_ttv99999':
                        success_zoned_and_hostobj_wwpns += 1
                        success_zoned_and_hostobj_wwpns_list.add(wwpn)
                    else:
                        warning_zoned_but_no_hostobj_wwpns += 1
                        zoned_but_no_hostobj_wwpns_list.add(wwpn)
                else:
                    warning_notzoned_wwpns = warning_notzoned_wwpns + 1
                    warning_notzoned_wwpns_list.add(wwpn)

        LOG.debug(
            "Check the zoned WWPNs and non-zoned"
            " WWPNs and generating results accordingly")
        if vios_total_wwpns_count == success_zoned_and_hostobj_wwpns:
            LOG.debug("ALL_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCVNX!" + emcvnxip + "!!!ALL_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCVNX!" +
                emcvnxip + "!!!ALL_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif success_zoned_and_hostobj_wwpns > 0:
            LOG.debug("SOME_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCVNX!" + emcvnxip + "!!!SOME_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]) + "!!!" +
                      ",".join(success_zoned_and_hostobj_wwpns_list))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCVNX!" +
                emcvnxip + "!!!SOME_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"]) +\
                "!!!" + \
                ",".join(success_zoned_and_hostobj_wwpns_list)

        if vios_total_wwpns_count == warning_notzoned_wwpns:
            LOG.debug("NONE_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCVNX!" + emcvnxip + "!!!NONE_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCVNX!" + emcvnxip +
                "!!!NONE_ZONED"] = ",".join(vios["scgs"]["scg_name_list"])

        elif warning_notzoned_wwpns > 0:
            LOG.debug("NOT_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCVNX!" + emcvnxip +
                      "!!!NOT_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]) + "!!!" +
                      ",".join(warning_notzoned_wwpns_list))
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!EMCVNX!" + emcvnxip +
                                 "!!!NOT_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"]) + \
                "!!!" + ",".join(warning_notzoned_wwpns_list)

        if vios_total_wwpns_count == warning_zoned_but_no_hostobj_wwpns:
            LOG.debug("ALL_ZONED_NO_HOST_OBJECT" + vios["host"] + ":" + vios[
                      "name"] + "!!!EMCVNX!" + emcvnxip +
                      "!!!ALL_ZONED_NO_HOST_OBJECT" +
                      ",".join(vios["scgs"]["scg_name_list"]))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCVNX!" +
                emcvnxip + "!!!ALL_ZONED_NO_HOST_OBJECT"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif warning_zoned_but_no_hostobj_wwpns > 0:
            LOG.debug("ZONED_NO_HOST_OBJECT" + vios["host"] + ":" +
                      vios["name"] + "!!!EMCVNX!" + emcvnxip +
                      "!!!ZONED_NO_HOST_OBJECT" +
                      ",".join(vios["scgs"]["scg_name_list"]) +
                      "!!!" + ",".join(zoned_but_no_hostobj_wwpns_list))
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!EMCVNX!" + emcvnxip +
                                 "!!!ZONED_NO_HOST_OBJECT"] = \
                ",".join(vios["scgs"]["scg_name_list"]) + "!!!" + \
                ",".join(zoned_but_no_hostobj_wwpns_list)


'''
Validate the host FC physical connectivity with EMC VMAX storage
'''


def validateWithEMCVMAX(
        emcvmaxip, all_vios_info_list, connectivity_dict,
        scg_validations_dict):

    for vios in all_vios_info_list:
        success_zoned_and_hostobj_wwpns = 0
        warning_zoned_but_no_hostobj_wwpns = 0
        warning_notzoned_wwpns = 0
        zoned_but_no_hostobj_wwpns_list = set()
        warning_notzoned_wwpns_list = set()
        success_zoned_and_hostobj_wwpns_list = set()

        vios_wwpns = vios["wwpns"]
        vios_total_wwpns_count = len(vios_wwpns)
        host_wwpn_dict = connectivity_dict.get("host_wwpn_dict")
        LOG.debug(
            "Match WWPNs of vios with logged in host wwpns from VMAX storage")
        for wwpn in vios_wwpns:
            for emcvmaxhostwwpn in host_wwpn_dict.keys():
                if wwpn in host_wwpn_dict.get(emcvmaxhostwwpn):
                    if emcvmaxhostwwpn !=\
                            'empty_unknown_zoned_unconfigured_ttv99999':
                        success_zoned_and_hostobj_wwpns += 1
                        success_zoned_and_hostobj_wwpns_list.add(wwpn)
                    else:
                        warning_zoned_but_no_hostobj_wwpns += 1
                        zoned_but_no_hostobj_wwpns_list.add(wwpn)
                else:
                    warning_notzoned_wwpns = warning_notzoned_wwpns + 1
                    warning_notzoned_wwpns_list.add(wwpn)

        LOG.debug(
            "Check the zoned WWPNs and non-zoned"
            " WWPNs and generating results accordingly")
        if vios_total_wwpns_count == success_zoned_and_hostobj_wwpns:
            LOG.debug("ALL_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCVMAX!" + emcvmaxip + "!!!ALL_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCVMAX!" +
                emcvmaxip + "!!!ALL_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif success_zoned_and_hostobj_wwpns > 0:
            LOG.debug("SOME_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCVMAX!" + emcvmaxip + "!!!SOME_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]) + "!!!" +
                      ",".join(success_zoned_and_hostobj_wwpns_list))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCvmax!" +
                emcvmaxip + "!!!SOME_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"]) + "!!!" + \
                ",".join(success_zoned_and_hostobj_wwpns_list)

        if vios_total_wwpns_count == warning_notzoned_wwpns:
            LOG.debug("NONE_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCvmax!" + emcvmaxip + "!!!NONE_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCVMAX!" + emcvmaxip +
                "!!!NONE_ZONED"] = ",".join(vios["scgs"]["scg_name_list"])

        elif warning_notzoned_wwpns > 0:
            LOG.debug("NOT_ZONED" + vios["host"] + ":" + vios["name"] +
                      "!!!EMCVMAX!" + emcvmaxip +
                      "!!!NOT_ZONED" +
                      ",".join(vios["scgs"]["scg_name_list"]) + "!!!" +
                      ",".join(warning_notzoned_wwpns_list))
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!EMCVMAX!" + emcvmaxip +
                                 "!!!NOT_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"]) + \
                "!!!" + ",".join(warning_notzoned_wwpns_list)

        if vios_total_wwpns_count == warning_zoned_but_no_hostobj_wwpns:
            LOG.debug("ALL_ZONED_NO_HOST_OBJECT" + vios["host"] + ":" + vios[
                      "name"] + "!!!EMCVMAX!" + emcvmaxip +
                      "!!!ALL_ZONED_NO_HOST_OBJECT" +
                      ",".join(vios["scgs"]["scg_name_list"]))
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!EMCVMAX!" +
                emcvmaxip + "!!!ALL_ZONED_NO_HOST_OBJECT"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif warning_zoned_but_no_hostobj_wwpns > 0:
            LOG.debug("ZONED_NO_HOST_OBJECT" + vios["host"] + ":" +
                      vios["name"] + "!!!EMCVMAX!" + emcvmaxip +
                      "!!!ZONED_NO_HOST_OBJECT" +
                      ",".join(vios["scgs"]["scg_name_list"]) +
                      "!!!" + ",".join(zoned_but_no_hostobj_wwpns_list))
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!EMCVMAX!" + emcvmaxip +
                                 "!!!ZONED_NO_HOST_OBJECT"] = \
                ",".join(vios["scgs"]["scg_name_list"]) + "!!!" + \
                ",".join(zoned_but_no_hostobj_wwpns_list)

'''
Validate the host FC physical connectivity with SVC storage
'''


def validateWithSVC(
        svcip, all_vios_info_list, lsfabric_dict, scg_validations_dict):

    for vios in all_vios_info_list:
        success_zoned_and_hostobj_wwpns = 0
        warning_zoned_but_no_hostobj_wwpns = 0
        warning_notzoned_wwpns = 0
        zoned_but_no_hostobj_wwpns_list = []
        warning_notzoned_wwpns_list = []
        success_zoned_and_hostobj_wwpns_list = []

        vios_wwpns = vios["wwpns"]
        vios_total_wwpns_count = len(vios_wwpns)
        LOG.debug(
            "Match WWPN of vios with logged in host wwpns from SVC storage")
        for wwpn in vios_wwpns:
            if wwpn in lsfabric_dict["wwpn_dict"].keys():
                if lsfabric_dict["wwpn_dict"].get(wwpn).get("host_name") != \
                        "empty_unknown_zoned_unconfigured_ttv99999":
                    success_zoned_and_hostobj_wwpns += 1
                    success_zoned_and_hostobj_wwpns_list.append(wwpn)
                else:
                    warning_zoned_but_no_hostobj_wwpns += 1
                    zoned_but_no_hostobj_wwpns_list.append(wwpn)
            else:
                warning_notzoned_wwpns = warning_notzoned_wwpns + 1
                warning_notzoned_wwpns_list.append(wwpn)

        if vios_total_wwpns_count == success_zoned_and_hostobj_wwpns:
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!SVC!" +
                svcip + "!!!ALL_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif success_zoned_and_hostobj_wwpns > 0:
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!SVC!" +
                svcip + "!!!SOME_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"]) + "!!!" + \
                ",".join(success_zoned_and_hostobj_wwpns_list)

        if vios_total_wwpns_count == warning_notzoned_wwpns:
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!SVC!" + svcip +
                "!!!NONE_ZONED"] = ",".join(vios["scgs"]["scg_name_list"])

        elif warning_notzoned_wwpns > 0:
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!SVC!" + svcip +
                                 "!!!NOT_ZONED"] =\
                ",".join(vios["scgs"]["scg_name_list"]) + \
                "!!!" + ",".join(warning_notzoned_wwpns_list)

        if vios_total_wwpns_count == warning_zoned_but_no_hostobj_wwpns:
            scg_validations_dict[vios[
                "host"] + ":" + vios["name"] + "!!!SVC!" +
                svcip + "!!!ALL_ZONED_NO_HOST_OBJECT"] =\
                ",".join(vios["scgs"]["scg_name_list"])

        elif warning_zoned_but_no_hostobj_wwpns > 0:
            scg_validations_dict[vios["host"] + ":" + vios["name"] +
                                 "!!!SVC!" + svcip +
                                 "!!!ZONED_NO_HOST_OBJECT"] = \
                ",".join(vios["scgs"]["scg_name_list"]) + "!!!" + \
                ",".join(zoned_but_no_hostobj_wwpns_list)

'''
main method for this program
'''


def my_main():
    parser = optparse.OptionParser()
    parser.add_option(
        '-f',
        '--propfile',
        dest='propfile',
        help='properties file',
        default=None)

    help_msg = "scg_validation.py -f propfile"

    (options, args) = parser.parse_args()

    if options.propfile is None:
        print "HELP : " + help_msg
        sys.exit(2)

    LOG.debug("********************************************************")
    LOG.debug("*************SCG Connectivity Validation****************")
    LOG.debug("********************************************************")
    propfile = options.propfile

    start_time = time.time()
    try:
        validateSCGVSCSIConnectivity(propfile)

        stop_time = time.time()
        LOG.debug('total time taken for scg validation:' +
                  str(round(stop_time - start_time, 1)))
        sys.exit(0)
    except Exception as e:
        LOG.exception("Error while performing scg validation:" + str(e))
        sys.exit(1)

if __name__ == '__main__':
    my_main()
