#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------

from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv.resources.SSPResource import SSPResource
import json
from powervc_ttv import _
from powervc_ttv import translate_message

logFile = "ssp_validator.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)

CSA_MSG_ID = "MSG080"
DSLU_MSG_ID = "MSG180"
CLU_MSG_ID = "MSG150"
CSLU_MSG_ID = "MSG110"

SSP_CLUSTER_ACCESSIBLE = "SSP_CLUSTER_ACCESSIBLE"
SSP_DELETE_LOGICAL_UNIT = "SSP_DELETE_LOGICAL_UNIT"
SSP_CLONE_LOGICAL_UNIT = "SSP_CLONE_LOGICAL_UNIT"
SSP_CREATE_LOGICAL_UNIT = "SSP_CREATE_LOGICAL_UNIT"
SSP_VALIDATION_ERROR = "SSP_VALIDATION_ERROR"


# Class implements SSP checker
# Uses SSPResource Module internally.
# Which in turn makes use of K2 api's to fetch SSP details,
# which in turn is used for validating against the expected Attributes
# mentioned in Solution_spec.json
class sspConfigChecker():

    # Intakes HMC ip, user id and password
    # for connecting and retrieving the HMC details
    def __init__(self, hmcip, hmcUser, hmcPasswd,
                 clusterSSPAccessStatus, delSSPLunsStatus, cloneLunsStatus,
                 createSSPLunsStatus, timeout, logger):
        self.hmcip = hmcip
        self.hmcUser = hmcUser
        self.hmcPasswd = hmcPasswd
        self.clusterSSPAccessStatus = clusterSSPAccessStatus
        self.delSSPLunsStatus = delSSPLunsStatus
        self.cloneLunsStatus = cloneLunsStatus
        self.createSSPLunsStatus = createSSPLunsStatus
        self.timeout = timeout
        self.logger = logger

    def strValueCompare(self, attributeTocheck,
                        expectedVal, actualVal,
                        msgOk, msgCritical,
                        corrAction, msgIDOk,
                        msgIDCritical, storageHost,
                        validatorID):
        """ Compares expected and actual String Values.
        Returns the checkList with status info"""
        self.logger.info(_("Checking SSP" + attributeTocheck + " Value"))
        if expectedVal.lower() == actualVal.lower():
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [storageHost]
            ttvcatalogmsgID = msgOk
            messageID = msgIDOk
            msgSubstitues = ""
            verbose = ""
            status = "Success"
        else:
            camsgplaceholders = []
            correctiveAction = corrAction
            msgplaceholders = [storageHost]
            ttvcatalogmsgID = msgCritical
            messageID = msgIDCritical
            msgSubstitues = ""
            verbose = ""
            status = "Critical"

        chkResult = utils.checkResult(storageHost,
                                      correctiveAction,
                                      camsgplaceholders,
                                      ttvcatalogmsgID,
                                      msgplaceholders,
                                      messageID,
                                      msgSubstitues,
                                      status, verbose,
                                      validatorID)
        return chkResult

    # Each validation check should implement this interface
    # to perform the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed
    # using __init__ method
    def check(self):

        chkResultList = []
        self.sspResource = SSPResource(self.hmcip, self.hmcUser,
                                       self.hmcPasswd, self.logger,
                                       self.timeout)
        (rc, cmdOutput, cmdErr, sspResourceList) =\
            self.sspResource.getSSPDetails()

        if rc == 0:
            if (sspResourceList is not None and len(sspResourceList) > 0):
                for entry in sspResourceList:
                    storageHost = entry["storage_host"]
                    message_id = entry["message"]
                    if message_id is not None and message_id == CSA_MSG_ID:
                        self.logger.info(_("=====Perform SSP validation for" +
                                           "configure cluster"))
                        actualStatus = entry["status"]
                        chkResult = self.strValueCompare(
                            'cluster_ssp_accessible',
                            self.clusterSSPAccessStatus,
                            actualStatus,
                            translate_message(
                                ttvCatalog.CLUSTER_SSP_CONFIGURE_SUCCESS,
                                desired_locale),
                            translate_message(
                                ttvCatalog.CLUSTER_SSP_CONFIGURE_FAIL,
                                desired_locale),
                            translate_message(
                                ttvCatalog.CA_CLUSTER_SSP_CONFIGURE,
                                desired_locale),
                            messageIDCatalog.HMC_CLUSTER_SSP_CONFIGURATION,
                            messageIDCatalog.
                            HMC_CLUSTER_SSP_CONFIGURATION_ERROR,
                            storageHost,
                            SSP_CLUSTER_ACCESSIBLE)
                        chkResultList.append(chkResult)
                    elif message_id is not None and message_id == CSLU_MSG_ID:
                        self.logger.info(_("=====Perform SSP validation for " +
                                           "create LogicalUnits"))
                        actualStatus = entry["status"]
                        chkResult = self.strValueCompare(
                            'create_ssp_luns',
                            self.createSSPLunsStatus,
                            actualStatus,
                            translate_message(
                                ttvCatalog.SSP_CREATE_LUNS_SUCCESS,
                                desired_locale),
                            translate_message(
                                ttvCatalog.SSP_CREATE_LUNS_FAIL,
                                desired_locale),
                            translate_message(
                                ttvCatalog.CA_SSP_CREATE_LUNS,
                                desired_locale),
                            messageIDCatalog.
                            HMC_CLUSTER_SSP_CONFIGURATION,
                            messageIDCatalog.
                            HMC_CLUSTER_SSP_CONFIGURATION_ERROR,
                            storageHost,
                            SSP_CREATE_LOGICAL_UNIT)
                        chkResultList.append(chkResult)
                    elif message_id is not None and message_id == CLU_MSG_ID:
                        self.logger.info(_("=====Perform SSP validation for" +
                                           "Clone LogicalUnits"))
                        actualStatus = entry["status"]
                        chkResult = self.strValueCompare(
                            'clone_luns',
                            self.cloneLunsStatus,
                            actualStatus,
                            translate_message(
                                ttvCatalog.SSP_CLONE_LUNS_SUCCESS,
                                desired_locale),
                            translate_message(
                                ttvCatalog.SSP_CLONE_LUNS_FAIL,
                                desired_locale),
                            translate_message(
                                ttvCatalog.CA_SSP_CLONE_LUNS,
                                desired_locale),
                            messageIDCatalog.
                            HMC_CLUSTER_SSP_CONFIGURATION,
                            messageIDCatalog.
                            HMC_CLUSTER_SSP_CONFIGURATION_ERROR,
                            storageHost,
                            SSP_CLONE_LOGICAL_UNIT)
                        chkResultList.append(chkResult)
                    elif message_id is not None and message_id == DSLU_MSG_ID:
                        self.logger.info(_("=====Perform SSP validation for " +
                                           "delete LogicalUnits"))
                        actualStatus = entry["status"]
                        chkResult = self.strValueCompare(
                            'del_ssp_luns',
                            self.delSSPLunsStatus,
                            actualStatus,
                            translate_message(
                                ttvCatalog.SSP_DELETE_LUNS_SUCCESS,
                                desired_locale),
                            translate_message(
                                ttvCatalog.SSP_DELETE_LUNS_FAIL,
                                desired_locale),
                            translate_message(
                                ttvCatalog.CA_SSP_DELETE_LUNS,
                                desired_locale),
                            messageIDCatalog.
                            HMC_CLUSTER_SSP_CONFIGURATION,
                            messageIDCatalog.
                            HMC_CLUSTER_SSP_CONFIGURATION_ERROR,
                            storageHost,
                            SSP_DELETE_LOGICAL_UNIT)
                        chkResultList.append(chkResult)
            else:
                self.logger.info(_("sspResourceList is empty."))
                self.logger.info(_("HMC {0} is not configured with SSP.".
                                   format(self.hmcip)))
        else:
            correctiveAction = ""
            camsgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VALIDATION_EXCEPTION,
                desired_locale)
            msgplaceholders = []
            messageID = messageIDCatalog.HMC_CLUSTER_SSP_CONFIGURATION_ERROR
            msgSubstitues = ""
            verbose = ""
            self.logger.error(_(cmdErr.strip()))
            status = "Critical"
            chkResult = utils.checkResult(self.hmcip,
                                          correctiveAction,
                                          camsgplaceholders,
                                          ttvcatalogmsgID,
                                          msgplaceholders,
                                          messageID,
                                          msgSubstitues,
                                          status, verbose,
                                          SSP_VALIDATION_ERROR)
            chkResultList.append(chkResult)
        return chkResultList


def readSSPExpectedConfiguration():
    """ read SSP Expected configuration details from solution spec"""
    solution_spec = "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
    ssp_info = utils.readJSONFile(solution_spec)["ssp"]

    clusterSSPAccessStatus = ssp_info["cluster_ssp_accessible"]
    delSSPLunsStatus = ssp_info["del_ssp_luns"]
    cloneLunsStatus = ssp_info["clone_luns"]
    createSSPLunsStatus = ssp_info["create_ssp_luns"]

    return (clusterSSPAccessStatus, delSSPLunsStatus, cloneLunsStatus,
            createSSPLunsStatus)


def addFailureResult(output_dict, catalogmsgID, msgD):
    correctiveAction = ""
    camsgplaceholders = []
    ttvcatalogmsgID = catalogmsgID
    msgplaceholders = []
    messageID = msgD
    msgSubstitues = ""
    verbose = ""
    status = "Critical"
    chkResult = utils.checkResult("",
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  SSP_VALIDATION_ERROR)
    output_dict["ValidationDetail"]["ChecksList"].append(chkResult)
    return output_dict


def fetchHMCInfo(uuid, listOfHMCs):
    """ Helper method to fetch HMC IP, UserID, Password"""
    hmcip = ""
    hmcUser = ""
    hmcPasswd = ""
    for entry in listOfHMCs:
        if entry["uuid"] == uuid:
            hmcip = entry["ip"]
            hmcUser = entry["userid"]
            hmcPasswd = entry["passwd"]
            break
    return(hmcip, hmcUser, hmcPasswd)


def main():

    # Variable Initialization
    rc = 0
    output_dict = {
        "ValidationID": "SSP_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": "",
        "TargetedResourceType": "ssp",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]

    # Logger Initialization
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()

    args = utils.parse_arguments()
    global desired_locale
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.SSP_VALIDATIONS,
        desired_locale)

    # Check for Powervc Edition
    powervc_edition = utils.getPowerVCEdition()
    if(powervc_edition.lower() != "hmc"):
        logger.info(_("PowerVC Edition is not HMC. The edition is {}".
                      format(powervc_edition)))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc

    logger.info(_("PowerVC Edition is {}".format(powervc_edition)))

    # get SSP Information from DB Query and form hmcUUID List
    hmcUUIDList = []
    try:
        env = utils.getAllSSPInfo()
        listOfSSPs = env["SSP"]
        if(len(listOfSSPs) == 0):
            logger.info(("No SSP is registered."))
            print(json.dumps({"ValidationResult": output_dict}))
            return rc
        else:
            for entry in listOfSSPs:
                hmcUUID = entry["HMC_UUID"]
                hmcUUIDList.append(hmcUUID)
    except Exception as e:
        logger.error(_("Exception While retrieving info about SSP."))
        logger.exception(_(e))
        rc = 1
        output_dict = addFailureResult(
            output_dict,
            translate_message(ttvCatalog.VALIDATION_EXCEPTION, desired_locale),
            messageIDCatalog.HMC_CLUSTER_SSP_CONFIGURATION_ERROR)
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    logger.info(_("No. of SSP configured in current environment is {0}".format(
                len(hmcUUIDList))))
    logger.info(_("UUID's of connected HMCs are {0}".format(hmcUUIDList)))

    # Read SSP Expected Configuration
    (clusterSSPAccessStatus, delSSPLunsStatus, cloneLunsStatus,
     createSSPLunsStatus) = readSSPExpectedConfiguration()
    logger.debug(_("Expected SSP Config values from solution spec are."))
    logger.debug(_("clusterSSPAccessStatus <{0}>".format(
        clusterSSPAccessStatus)))
    logger.debug(_("delSSPLunsStatus <{0}>".format(delSSPLunsStatus)))
    logger.debug(_("cloneLunsStatus <{0}>".format(cloneLunsStatus)))
    logger.debug(_("createSSPLunsStatus <{0}>".format(createSSPLunsStatus)))

    # Get HMC Information from DB Query and form listofHMCs.
    listOfHMCs = []
    try:
        env = utils.getAllHMCInfo()
        listOfHMCs = env["HMC"]
    except Exception as e:
        logger.error(_("Exception While retrieving info about HMC."))
        logger.exception(_(e))
        rc = 1
        output_dict = addFailureResult(
            output_dict,
            ttvCatalog.HMC_CONFIGURATION_EXCEPTION,
            messageIDCatalog.HMC_CONFIGURATION_ERROR)
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    logger.info(_("No. of HMCs configured in current environment is {0}".
                  format(len(listOfHMCs))))

    if (len(hmcUUIDList) > 0):
        for uuid in hmcUUIDList:
            (hmcip, hmcUser, hmcPasswd) = fetchHMCInfo(uuid, listOfHMCs)
            if(hmcip is None or len(hmcip) == 0):
                logger.info(_("hmcip is either None or Empty"))
                continue
            logger.info(_("SSP Checker getting initialized for HMC -i <{0}>"
                          .format(hmcip)))

            # Create object for each checker
            sspChk = sspConfigChecker(hmcip,
                                      hmcUser,
                                      hmcPasswd,
                                      clusterSSPAccessStatus,
                                      delSSPLunsStatus,
                                      cloneLunsStatus,
                                      createSSPLunsStatus,
                                      120,
                                      logger)
            chkResultList = sspChk.check()
            resultList.extend(chkResultList)
        logger.info(_("Final Result is:-> {0}".format(
            json.dumps({"ValidationResult": output_dict}))))
        print(json.dumps({"ValidationResult": output_dict}))
    else:
        logger.info(_("HMC is not configured on Management Server."))
    exit(rc)

if __name__ == "__main__":
    main()
