#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import os
import json
import glob
from powervc_ttv.resources import SVCStorageResource
from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import translate_message

output_dict = {
    "ValidationID": "SVCSTORAGE_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "svc_storage",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}

logger = None
# Log file name.
logFile = "svcstorage_validator.log"
NETWORK_QUALITY_CHK = "NETWORK_QUALITY_CHK"


def main_func():
    rc = 0
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    global logger
    logger = chkLogger.getLogger()
    if rc == 1:
        return 1

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.SVC_STORAGE_VALIDATIONS,
        desired_locale)

    resource_dict = readConf(logger)
    svc = "svc_storage"
    '''read reference params from solution_spec for validations'''
    validationProp = utils.readJSONFile(
        "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json")[svc]

    for resource in resource_dict:
        try:
            validateResource(validationProp, resource, logger)
        except Exception as e:
            logger.exception(e)
    print(json.dumps({"ValidationResult": output_dict}))
    return rc

'''
Firmware check validation class
'''


class FirmwareChecker():

    def __init__(self, resource, expfirmware):
        self.resource = resource
        self.expfirmware = expfirmware
        self.result = []

    def check(self):
        status = ""
        host = self.resource.svcip
        firmware = self.resource.getFirmware()
        if utils.compareVersion(firmware, self.expfirmware):
            '''firmware version matches'''
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.STORAGE_FIRMWARE_MATCH,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'SVC_STORAGE_FIRMWARE'))
        else:
            '''firmware version does not match'''
            camsgplaceholders = [firmware, self.expfirmware]
            correctiveAction = translate_message(
                ttvCatalog.STORAGE_FIRMWARE_MISMATCH,
                desired_locale)
            msgplaceholders = [firmware, self.expfirmware]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.STORAGE_FIRMWARE_MISMATCH,
                desired_locale)
            messageID = messageIDCatalog.STORAGE_FIRMWARE_INCORRECT
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'SVC_STORAGE_FIRMWARE'))
        return (status, self.result)


class NetworkQualityChecker():

    def __init__(self, ip):
        self.ip = ip
        self.result = []

    def check(self):
        status = ""
        host = self.ip
        quality = utils.checkNetworkQuality(self.ip)
        quality = int(quality)

        if quality == 100:
            '''network quality is bad'''
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_BAD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality >= 5:
            '''network quality is degraded'''
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_DEGRADED,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality < 5:
            '''network quality is good'''
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_GOOD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))

        return (status, self.result)


class CredentialChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.svcip
        credential = self.resource.checkCredential()
        if credential == 0:
            '''credentials are correct'''
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_VALID,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'SVC_STROAGE_CREDENTIAL'))
        else:
            '''credentials are not correct'''
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'SVC_STROAGE_CREDENTIAL'))

        return (status, self.result)


class ReachabilityChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.svcip
        reachability = utils.checkReachability(host)
        if reachability == 0:
            '''resource is reachable'''
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'SVC_STORAGE_REACHABILITY'))
        else:
            '''resource is not reachable'''
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'SVC_STORAGE_REACHABILITY'))
        return (status, self.result)


def validateResource(validationProp, resource, logger=None):
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]
    exception = False
    host = resource["hostname"]
    svcresourcemgr = SVCStorageResource.SVCStorageResource(resource, logger)
    checkers = []
    '''add all checkers to checker list and execute them in loop'''
    # checkers.append(ReachabilityChecker(svcresourcemgr))
    # checkers.append(CredentialChecker(svcresourcemgr))
    checkers.append(
        FirmwareChecker(svcresourcemgr, validationProp["firmware"]))
    checkers.append(NetworkQualityChecker(resource["hostname"]))
    status = ""
    result = []
    try:
        (status, result) = ReachabilityChecker(svcresourcemgr).check()
        chkResultList.extend(result)
    except Exception as e:
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        messageID = messageIDCatalog.\
            MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
        msgSubstitues = ""
        status = "Critical"
        verbose = ""
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               'SVC_STORAGE_REACHABILITY'))
        logger.exception(e)
    if status == "Success" and not exception:
        try:
            status = ""
            result = []
            (status, result) = CredentialChecker(svcresourcemgr).check()
            chkResultList.extend(result)
        except Exception as e:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            chkResultList.append(utils.checkResult(host,
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   'SVC_STROAGE_CREDENTIAL'))
            logger.exception(e)
        if status == "Success":
            for checker in checkers:
                status = ""
                result = []
                try:
                    (status, result) = checker.check()
                    chkResultList.extend(result)
                except Exception as e:
                    logger.exception(e)
                    exception = True

    if exception:
        correctiveAction = "an exception occurred while performing some or"\
            " all validations of this resource. check the log for more details"
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.CA_FABRIC_FETCH_ERROR,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.FABRIC_FETCH_ERROR,
            desired_locale)
        messageID = messageIDCatalog.\
            MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
        msgSubstitues = ""
        status = "Critical"
        verbose = "verbose"
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               'SVC_STORAGE_VALIDATION'))


def readConf(logger):
    env = {}
    env["svc-storage"] = []
    '''read all config of svc from cinder conf files'''
    for conf in glob.glob("/etc/cinder/cinder-*.conf"):
        if conf == "/etc/cinder/cinder-health.conf":
            continue
        try:
            cinder_conf = ConfigParser.SafeConfigParser()
            cinder_conf.read(conf)
            svc_storage = {}
            if cinder_conf.has_option('DEFAULT', 'host_type'):
                host_type = cinder_conf.get('DEFAULT', 'host_type')
            if host_type == 'svc' or host_type == 'SVC':
                svc_storage["hostname"] = cinder_conf.get("DEFAULT", "san_ip")
                svc_storage["userid"] = cinder_conf.get("DEFAULT", "san_login")
                svc_storage["key-file"] = None
                if cinder_conf.has_option("DEFAULT", "san_private_key"):
                    key_file = cinder_conf.get("DEFAULT", "san_private_key")
                    if os.path.exists(key_file):
                        svc_storage["key-file"] = key_file
                else:
                    svc_storage["key-file"] = None
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    svc_storage["passwd"] = cinder_conf.get("DEFAULT",
                                                            "san_password")
                else:
                    svc_storage["passwd"] = None
                for prop in ('storwize_svc_volpool_name', 'san_ssh_port'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        svc_storage[prop] = cinder_conf.get('DEFAULT', prop)
                env["svc-storage"].append(svc_storage)
        except Exception as e:
            logger.exception(e)
    return env["svc-storage"]


def main():
    rc = 0
    # Parse the command line arguments
    rc = main_func()

    exit(rc)

if __name__ == "__main__":
    main()
