#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import json
from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv.resources import HMCResource
import glob
from powervc_ttv import _
from powervc_ttv import translate_message

SUCCESS = 0
WARN = 1
CRITICAL = 2


VIOS_STATE_RUNNING = "running"
VIOS_LEVEL_UNKNOWN = "Unknown"
POWER_VERSION_8 = "POWER8"

CodeToStatus_map = {SUCCESS: "Success", WARN: "Warning", CRITICAL: "Critical"}
StatusToCode_map = {"Success": SUCCESS, "Warning": WARN, "Critical": CRITICAL}

output_dict = {
    "ValidationID": "VIOS_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "vios",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}

# Log file
logFile = "vios_validator.log"

# IDs for validation checks
VIOS_OS_VERSION = "VIOS_OS_VERSION"
VIOS_RMC_STATE = "VIOS_RMC_STATE"
VIOS_MAXIMUM_VIRTUAL_SLOTS = "VIOS_MAXIMUM_VIRTUAL_SLOTS"
VIOS_CURRENT_PROFILE_SYNC_STATE = "VIOS_CURRENT_PROFILE_SYNC_STATE"
VIOS_MEDIA_REPOSITORY_FREE_SPACE = "VIOS_MEDIA_REPOSITORY_FREE_SPACE"
VIOS_SEA_STATE = "VIOS_SEA_STATE"
VIOS_SEA_COUNT = "VIOS_SEA_COUNT"


def filterHost(hmcManagedALLHosts, hostsUUID):
    hostsRegisteredIntoPowerVC = []
    for entry in hostsUUID:
        if entry in hmcManagedALLHosts:
            hostsRegisteredIntoPowerVC.append(entry)
    return hostsRegisteredIntoPowerVC


def prpare_mapping_between_hosts_and_hmcs(listOfHMCs, hostsUUID, logger):
    if (len(listOfHMCs) > 0):
        HMCCecsMappingDict = {}
        for entry in listOfHMCs:
            hmcip = entry["ip"]
            hmcUser = entry["userid"]
            hmcPasswd = entry["passwd"]
            hmcResource = HMCResource.HMCResource(hmcip, hmcUser, hmcPasswd,
                                                  logger, 120)
            (hmcManagedALLHosts) = hmcResource.getHMCManagedAllHosts()
            HMCCecsMappingDict[hmcip] = filterHost(hmcManagedALLHosts,
                                                   hostsUUID)

        return HMCCecsMappingDict


def addFailureResult(output_dict):
    correctiveAction = ""
    camsgplaceholders = []
    ttvcatalogmsgID = translate_message(
        ttvCatalog.HMC_CONFIGURATION_EXCEPTION,
        desired_locale)
    msgplaceholders = []
    messageID = messageIDCatalog.HMC_CONFIGURATION_ERROR
    msgSubstitues = ""
    verbose = ""
    status = "Critical"
    chkResult = utils.checkResult("",
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose)
    output_dict["ValidationDetail"]["ChecksList"].append(chkResult)
    return output_dict


def main_func():
    rc = 0
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)

    if rc == 1:
        return 1

    output_dict = {
        "ValidationID": "VIOS_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": translate_message(
            ttvCatalog.VIOS_VALIDATIONS,
            desired_locale),
        "TargetedResourceType": "vios",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }

    # Check for Powervc Edition
    powervc_edition = utils.getPowerVCEdition()
    if(powervc_edition.lower() != "hmc"):
        logger.info(_("PowerVC Edition is not HMC. The edition is {}".
                      format(powervc_edition)))
        print(json.dumps({"ValidationResult": output_dict}))
        return rc

    logger.info(_("PowerVC Edition is {}".format(powervc_edition)))

    validation_properties = utils.readJSONFile(
        "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json")["vios"]
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]

    # Default values for level, active_lpar_mobility_capable,
    # max_virtual_slots,mode are stored here. Values will be read from
    # IOS resource configuration file.
    expected = {}
    expected['min_no_of_vios'] = 0
    expected['level'] = "2.2.3.0"
    expected['rmc_connection_state'] = "active"
    expected['max_virtual_slots'] = 200
    expected['state'] = "operating"
    expected['vios_current_profile_sync'] = "On"
    expected['power8_level'] = "2.2.3.3"
    expected['vios_sea_state'] = "Available"
    expected['minimum_no_of_sea'] = 1

    # Get VIOS & PowerVM specific configurations - minimum number of VIOS
    if "min_no_of_vios" in validation_properties:
        expected['min_no_of_vios'] = \
            int(validation_properties["min_no_of_vios"])
    # Get VIOS & PowerVM specific configurations - RMC connection state
    if "rmc_connection" in validation_properties:
        expected['rmc_connection_state'] = \
            validation_properties["rmc_connection"]
    # Get VIOS & PowerVM specific configurations - version
    if "hmc_vios_version" in validation_properties:
        expected['level'] = validation_properties["hmc_vios_version"]
    # Get VIOS & PowerVM specific configurations - max_virtual_slots
    if "hmc_max_virtual_slots" in validation_properties:
        expected['max_virtual_slots'] = \
            int(validation_properties["hmc_max_virtual_slots"])
    # Get VIOS & PowerVM specific configurations -hmc_vios_current_profile_sync
    if "hmc_vios_current_profile_sync" in validation_properties:
        expected['vios_current_profile_sync'] = \
            validation_properties["hmc_vios_current_profile_sync"]
    # Get VIOS & PowerVM specific configurations for power8 - version
    if "power8_vios_version" in validation_properties:
        expected['power8_level'] = validation_properties["power8_vios_version"]
    # Get Sea specific configurations.
    if "sea_count" in validation_properties:
        expected['minimum_no_of_sea'] = int(validation_properties["sea_count"])
    if "vios_sea_state" in validation_properties:
        expected['vios_sea_state'] = validation_properties["vios_sea_state"]

    # Get the list of CECs
    cecs_uuids = get_registered_cecs()
    if len(cecs_uuids) == 0:
        logger.error("Unable to retrieve HMC Hosts Information OR "
                     "no host is registered.")
        print(json.dumps({"ValidationResult": output_dict}))
        exit(rc)

    # Get HMC Information from DB Query.
    env = {}
    try:
        env = utils.getAllHMCInfo()
    except Exception as e:
        logger.exception(e)
        output_dict = addFailureResult(output_dict)
        print(json.dumps({"ValidationResult": output_dict}))
        exit(rc)
    hmc_list = env["HMC"]

    # Get the Mapping of HMC and Managed Host
    HMCCecsMappingDict = prpare_mapping_between_hosts_and_hmcs(hmc_list,
                                                               cecs_uuids,
                                                               logger)
    if (len(hmc_list) == 0):
        logger.error("Error : " + "Not able to retrieve HMC Information")
        rc = 1
        exit(rc)

    for entry in hmc_list:
        hmc_ip = entry["ip"]
        hmc_user = entry["userid"]
        password = entry["passwd"]
        cec_ids = HMCCecsMappingDict[hmc_ip]
        if len(cec_ids) == 0:
            logger.error("Error : " +
                         "Not able to retrieve Hosts managed by " + hmc_ip)
            continue

        hmc = HMCResource.HMCResource(hmc_ip, hmc_user, password,
                                      logger, 120, cec_ids)
        rc, out, err, hmc_details = hmc.get_hmc_managed_resource_details()
        cecs = None
        if 'managed_systems' in hmc_details:
            cecs = hmc_details.get('managed_systems')

        for cec in cecs:
            vioses = cec.get('vios_list')
            # check for min_no_of_vios
            if len(vioses) < expected['min_no_of_vios']:
                host = cec['mgmt_SystemName']
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog.CA_POWERSERVER_NO_OF_VIRTUAL_SERVERS,
                    desired_locale)
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.POWERSERVER_NO_OF_VIRTUAL_SERVERS_FAILED,
                    desired_locale)
                messageID = messageIDCatalog.\
                    CA_POWERSERVER_NO_OF_VIRTUAL_SERVERS
                msgSubstitues = ""
                status = "Critical"
                verbose = ""

                result = utils.checkResult(host, correctiveAction,
                                           camsgplaceholders,
                                           ttvcatalogmsgID,
                                           msgplaceholders,
                                           messageID,
                                           msgSubstitues,
                                           status, verbose,
                                           "VIOS_MINIMUM_COUNT")
                chkResultList.append(result)
            else:
                host = cec['mgmt_SystemName']
                camsgplaceholders = []
                correctiveAction = ''
                msgplaceholders = [len(vioses)]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.POWERSERVER_NO_OF_VIRTUAL_SERVERS,
                    desired_locale)
                messageID = messageIDCatalog.\
                    POWERSERVER_HMC_CONFIGURATION
                msgSubstitues = ""
                status = "Success"
                verbose = ""

                result = utils.checkResult(host, correctiveAction,
                                           camsgplaceholders,
                                           ttvcatalogmsgID,
                                           msgplaceholders,
                                           messageID,
                                           msgSubstitues,
                                           status, verbose,
                                           "VIOS_MINIMUM_COUNT")
                chkResultList.append(result)

            expected_values = expected
            if cec.get('mgmt_SupportedPartitionProcessorCompatibilityModes'):
                supported_compatibility_modes = cec.\
                    get('mgmt_SupportedPartitionProcessorCompatibilityModes')
                if POWER_VERSION_8 in supported_compatibility_modes:
                    expected_values['level'] = expected['power8_level']

            sea_count_exp_state = 0
            sea_count_unexp_state = 0
            total_seas_available = {}
            total_seas_unavailable = {}
            for vios in vioses:
                vios['ttv_vios_display_name'] = cec['mgmt_SystemName'] + \
                    ":" + vios['vios_PartitionName']
                validate_vios(vios, chkResultList, validation_properties,
                              expected_values)
                vios_available_seas = []
                vios_unavailable_seas = []
                for sea in vios['seas']:
                    trunkadapters = sea['seatrunkVLANIDs']
                    logger.info(_("Trunk Adapters : "))
                    logger.info(trunkadapters)
                    sea_name = sea['vios_sea_InterfaceName']
                    current_sea_state = sea['vios_sea_state']
                    if current_sea_state == "true":
                        current_sea_state = "Available"
                    if current_sea_state:
                        if current_sea_state.lower() == \
                                expected['vios_sea_state'].lower():
                            sea_count_exp_state = sea_count_exp_state + 1
                            vios_available_seas.append(sea_name)
                        else:
                            sea_count_unexp_state = sea_count_unexp_state + 1
                            vios_unavailable_seas.append(sea_name)
                    else:
                        # Failed to get sea state
                        vios_unavailable_seas.append(sea_name)
                total_seas_available[vios['ttv_vios_display_name']] = \
                    vios_available_seas
                total_seas_unavailable[vios['ttv_vios_display_name']] = \
                    vios_unavailable_seas

            _handle_orphaned_veas(cec, logger, chkResultList)
            # Perform SharedEthernetAdapter count for all VIOS validation
            if sea_count_exp_state < expected['minimum_no_of_sea']:
                host = cec['mgmt_SystemName']
                camsgplaceholders = [sea_count_exp_state,
                                     expected['minimum_no_of_sea']]
                correctiveAction = translate_message(
                    ttvCatalog.CA_HMC_VIOS_SEA,
                    desired_locale)
                msgplaceholders = [sea_count_exp_state,
                                   expected['minimum_no_of_sea']]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.HMC_VIOS_SEA_FAILURE_SEA_COUNT,
                    desired_locale)
                messageID = messageIDCatalog.\
                    CA_HMC_VIOS_SEA_CONFIGURATION_ERROR
                msgSubstitues = ""
                status = "Critical"
                verbose = ""

                result = utils.checkResult(host, correctiveAction,
                                           camsgplaceholders,
                                           ttvcatalogmsgID,
                                           msgplaceholders,
                                           messageID,
                                           msgSubstitues,
                                           status, verbose, "VIOS_SEA_COUNT")
                chkResultList.append(result)
            if sea_count_exp_state >= expected['minimum_no_of_sea']:
                if sea_count_unexp_state > 0:
                    for vios in vioses:
                        vios_ttv_display_name = vios['ttv_vios_display_name']
                        seas_unavailable = \
                            total_seas_unavailable[vios_ttv_display_name]
                        camsgplaceholders = [sea_count_exp_state,
                                             expected['minimum_no_of_sea']]
                        correctiveAction = ""
                        msgplaceholders = []
                        messageID = messageIDCatalog.\
                            CA_HMC_VIOS_SEA_CONFIGURATION_ERROR
                        msgSubstitues = ""
                        status = "Warning"
                        verbose = ""

                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.HMC_VIOS_SEA_WARNING,
                            desired_locale)
                        result = utils.checkResult(host, correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues, status,
                                                   verbose, "VIOS_SEA_STATE")
                        chkResultList.append(result)

                        if len(seas_unavailable) > 0:
                            ttvcatalogmsgID = translate_message(
                                ttvCatalog.HMC_VIOS_SEA_AT_NOT_AVAILABLE_STATE,
                                desired_locale)
                            msgplaceholders = [host, str(seas_unavailable)]
                            result = utils.checkResult(host, correctiveAction,
                                                       camsgplaceholders,
                                                       ttvcatalogmsgID,
                                                       msgplaceholders,
                                                       messageID,
                                                       msgSubstitues, status,
                                                       verbose,
                                                       "VIOS_SEA_STATE")
                            chkResultList.append(result)
                else:
                    camsgplaceholders = []
                    correctiveAction = ""
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.HMC_SEA_SUCCESS,
                        desired_locale)
                    messageID = messageIDCatalog.VIOS_HMC_CONFIGURATION
                    msgSubstitues = ""
                    status = "Success"
                    verbose = ""
                    result = utils.checkResult(host, correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID, msgSubstitues,
                                               status, verbose,
                                               "VIOS_SEA_COUNT")
                    chkResultList.append(result)

    print(json.dumps({"ValidationResult": output_dict}))
    return 0


def _handle_orphaned_veas(cec, logger, chkResultList):

    orphanedvlans = []
    seavlans = []
    vioses = cec.get('vios_list')

    """ Get a list of all SEAs and their vlans """
    for vios in vioses:
        sea_list = vios['seas']
        pea_list = vios['peas']
        vea_list = vios['veas']
        vea_additional_list = vios['veas_additional']
        logger.info(_("Lists : "))
        logger.info(sea_list)
        logger.info(pea_list)
        logger.info(vea_list)
        logger.info(vea_additional_list)
        for sea in sea_list:
            seatrunkvlanids = sea['seatrunkVLANIDs']
            """ Get additional vlans and insert them in one list """
            for additional in sea['seatrunkAdditionalVLANIDs']:
                for each_a_l in additional:
                    if each_a_l not in seatrunkvlanids:
                        seatrunkvlanids.append(each_a_l)
            logger.info(_('Modified sea vlanids list : ' +
                        ' '.join(map(str, seatrunkvlanids))))
            for each_a_l in seatrunkvlanids:
                if each_a_l not in seavlans:
                    seavlans.append(each_a_l)
        logger.info(_('All sea vlan ids : ' +
                    ' '.join(map(str, seavlans))))

    """ Go through all trunk adapters in all VIOSes, collect vlanids """
    for vios in vioses:
        vea_list = vios['veas']
        """ Get additional vlan ids as well """
        vea_additional_list = vios['veas_additional']
        for additional in vea_additional_list:
            additional_list = additional.split()
            for each_a_l in additional_list:
                if each_a_l not in vea_list:
                    vea_list.append(each_a_l)
        logger.info(_('Modified vea vlan ids : ' +
                      ' '.join(map(str, vea_list))))

        for veavlanid in vea_list:
            """ Orphaned vlan, not in SEA """
            if veavlanid not in seavlans:
                """ Avoid duplicates """
                if veavlanid not in orphanedvlans:
                    orphanedvlans.append(veavlanid)

    if orphanedvlans:
        """ Orphaned vlans cound, message... """
        host = cec['mgmt_SystemName']
        name = cec['mgmt_SystemName']
        camsgplaceholders = []
        correctiveAction = ''
        msgplaceholders = [orphanedvlans, seavlans, name]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.HMC_VIOS_ORPHANED_VEAS,
            desired_locale)
        messageID = messageIDCatalog.\
            CA_HMC_VIOS_SEA_CONFIGURATION_ERROR
        msgSubstitues = ""
        status = "Critical"
        verbose = ""
        result = utils.checkResult(host, correctiveAction,
                                   camsgplaceholders,
                                   ttvcatalogmsgID,
                                   msgplaceholders,
                                   messageID,
                                   msgSubstitues,
                                   status, verbose, "VIOS_SEA_COUNT")
        chkResultList.append(result)
        return False
    return True


def validate_vios(vios, chkResultList, validation_properties, expected):
    exception_flag = False
    # Check vios media repository free space
    if "media_repository_free_space" in validation_properties:
        try:
            expected_free_space = \
                int(validation_properties["media_repository_free_space"])
            free_space_checker = \
                MediaRepositoryFreeSpaceChecker(vios,
                                                expected_free_space)
            (status, result) = free_space_checker.check()
            chkResultList.extend(result)
        except Exception:
            exception_flag = True

    # Check vios os level
    try:
        vios_level_checker = VIOSLevelChecker(vios, expected['level'])
        (status, result) = vios_level_checker.check()
        chkResultList.extend(result)
    except Exception:
        exception_flag = True

    # Check rmc connection state
    try:
        rmc_state_checker = \
            RMCStateChecker(vios, expected['rmc_connection_state'])
        (status, result) = rmc_state_checker.check()
        chkResultList.extend(result)
    except Exception:
        exception_flag = True

    # Check max_virtual_slots
    try:
        max_virtual_slots_checker = \
            MaxVirtualSlotsChecker(vios, expected['max_virtual_slots'])
        (status, result) = max_virtual_slots_checker.check()
        chkResultList.extend(result)
    except Exception:
        exception_flag = True

    # Check vios_current_profile_sync
    try:
        exp_profile_sync_state = expected['vios_current_profile_sync']
        profile_sync_state_checker = \
            CurrentProfileSyncStateChecker(vios,
                                           exp_profile_sync_state)
        (status, result) = profile_sync_state_checker.check()
        chkResultList.extend(result)
    except Exception:
        exception_flag = True

    if exception_flag:
        host = vios['ttv_vios_display_name']
        correctiveAction = "an exception occurred while " + \
                           "performing some or all validations of " + \
                           "this resource. check the log " + \
                           "for more details."
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.CA_VIOS_DETAILS_RETRIEVE_FAILED,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.VIOS_DETAILS_RETRIEVE_FAILED,
            desired_locale)
        messageID = ""
        msgSubstitues = ""
        status = "Critical"
        verbose = "verbose"
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               "VIOS_VALIDATION"))


class MediaRepositoryFreeSpaceChecker():

    def __init__(self, vios, expected_free_space):
        self.vios = vios
        self.expected_free_space = expected_free_space
        self.result = []

    def check(self):
        status = ""
        host = self.vios['ttv_vios_display_name']
        val_check_id = VIOS_MEDIA_REPOSITORY_FREE_SPACE

        if not ("vios_MediaRepositoryExists" in self.vios and
                "vios_RepositorySize" in self.vios):
            # Media library not existing
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_VIOS_DETAILS_RETRIEVE_FAILED,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_MEDIA_REPOSITORY_FREE_SPACE_CHECK_EXCEPTION,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 val_check_id))
            return (status, self.result)
        if ("vios_MediaRepositoryExists" in self.vios and
                "vios_RepositorySize" in self.vios):
            media_library_space = self.vios['vios_RepositorySize']
            used_media_space = self.vios['vios_TotalMediaSize']

        if not("vios_MediaRepositoryExists" in self.vios and
               "vios_RepositorySize" not in self.vios):
            free_space = float(media_library_space) - float(used_media_space)
            free_space = int(free_space * 1024)  # Convert into MB
            if free_space >= self.expected_free_space:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = [free_space, self.expected_free_space]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.VIOS_MEDIA_REPOSITORY_FREE_SPACE_SIZE_SUCCESS,
                    desired_locale)
                messageID = ""
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
            else:
                camsgplaceholders = [self.expected_free_space]
                correctiveAction = translate_message(
                    ttvCatalog.
                    CA_VIOS_AVAILABLE_MEDIA_REPOSITORY_FREE_SPACE_CHECK_EXPAND,
                    desired_locale)
                msgplaceholders = [free_space, self.expected_free_space]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.
                    VIOS_MEDIA_REPOSITORY_FREE_SPACE_SIZE_FAILED,
                    desired_locale)
                messageID = ""
                msgSubstitues = ""
                status = "Warning"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
        else:
            camsgplaceholders = []
            correctiveAction = messageIDCatalog.CHECK_LOG
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.
                VIOS_MEDIA_REPOSITORY_FREE_SPACE_CHECK_EXCEPTION,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 val_check_id))
        return (status, self.result)


class VIOSLevelChecker():

    def __init__(self, vios, expected_vios_level):
        self.vios = vios
        self.expected_vios_level = expected_vios_level
        self.result = []

    def check(self):
        status = ""
        val_check_id = VIOS_OS_VERSION
        host = self.vios['ttv_vios_display_name']
        vios_state = self.vios['vios_PartitionState']
        if vios_state and vios_state == "running":
            os_version = self.vios['vios_OperatiingSystemVersion']
            if os_version and os_version != "":
                if "VIOS" in os_version:
                    os_version = os_version.split(" ")[1]

                if os_version == VIOS_LEVEL_UNKNOWN:
                    camsgplaceholders = []
                    correctiveAction = translate_message(
                        ttvCatalog.CA_VIOS_MINIMUM_LEVEL_FAILED,
                        desired_locale)
                    msgplaceholders = [host, os_version,
                                       self.expected_vios_level]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.HMC_VIOS_LEVEL_UNKNOWN_FAILED,
                        desired_locale)
                    messageID = messageIDCatalog.VIOS_HMC_CONFIGURATION_ERROR
                    msgSubstitues = ""
                    status = "Critical"
                    verbose = ""
                    self.result.\
                        append(utils.checkResult(host, correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 val_check_id))
                else:
                    version_compare_result = \
                        compare_vios_levels(os_version,
                                            self.expected_vios_level)
                    if version_compare_result == 0:
                        camsgplaceholders = []
                        correctiveAction = ""
                        msgplaceholders = [host]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.HMC_VIOS_MINIMUM_LEVEL_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.VIOS_HMC_CONFIGURATION
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.\
                            append(utils.checkResult(host, correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
                    elif version_compare_result == -1:
                        camsgplaceholders = []
                        correctiveAction = ""
                        msgplaceholders = [host, os_version,
                                           self.expected_vios_level]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.VIOS_MINIMUM_LEVEL_CHECK_EXCEPTION,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            VIOS_HMC_CONFIGURATION_ERROR
                        msgSubstitues = ""
                        status = "Warning"
                        verbose = ""
                        self.result.\
                            append(utils.checkResult(host, correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
                    else:
                        camsgplaceholders = []
                        correctiveAction = translate_message(
                            ttvCatalog.CA_VIOS_MINIMUM_LEVEL_FAILED,
                            desired_locale)
                        msgplaceholders = [host, os_version,
                                           self.expected_vios_level]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.HMC_VIOS_MINIMUM_LEVEL_FAILED,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            VIOS_HMC_CONFIGURATION_ERROR
                        msgSubstitues = ""
                        status = "Critical"
                        verbose = ""
                        self.result.\
                            append(utils.checkResult(host, correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog.CA_UNABLE_TO_QUERY_VIOS_PROPERTIES,
                    desired_locale)
                msgplaceholders = [host]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.UNABLE_TO_QUERY_VIOS_PROPERTIES,
                    desired_locale)
                messageID = messageIDCatalog.\
                    VIOS_CONFIGURATION_INCORRECT
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.\
                    append(utils.checkResult(host, correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status, verbose,
                                             val_check_id))

        return (status, self.result)


class RMCStateChecker():

    def __init__(self, vios, expected_rmc_state):
        self.vios = vios
        self.expected_rmc_state = expected_rmc_state.lower().strip()
        self.result = []

    def check(self):
        status = ""
        val_check_id = VIOS_RMC_STATE
        host = self.vios['ttv_vios_display_name']
        rmc_state = self.vios['vios_ResourceMonitoringControlState']
        if rmc_state and rmc_state != "":
            if rmc_state.lower() == self.expected_rmc_state:
                # VIOS_RMC_CONN_STATE_SUCCESS
                # VIOS_HMC_CONFIGURATION
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = [host, rmc_state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.VIOS_RMC_CONN_STATE_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.VIOS_HMC_CONFIGURATION
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.\
                    append(utils.checkResult(host, correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status, verbose,
                                             val_check_id))
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog.CA_VIOS_RMC_CONN_STATE,
                    desired_locale)
                msgplaceholders = [host, rmc_state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.VIOS_RMC_CONN_STATE_FAILED,
                    desired_locale)
                messageID = ""
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.\
                    append(utils.checkResult(host, correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status, verbose,
                                             val_check_id))
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_UNABLE_TO_QUERY_VIOS_PROPERTIES,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.UNABLE_TO_QUERY_VIOS_PROPERTIES,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_CONFIGURATION_INCORRECT
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.\
                append(utils.checkResult(host, correctiveAction,
                                         camsgplaceholders,
                                         ttvcatalogmsgID,
                                         msgplaceholders,
                                         messageID,
                                         msgSubstitues,
                                         status, verbose,
                                         val_check_id))

        return (status, self.result)


class MaxVirtualSlotsChecker():

    def __init__(self, vios, expected_max_virtual_slots):
        self.vios = vios
        self.expected_max_virtual_slots = expected_max_virtual_slots
        self.result = []

    def check(self):
        status = ""
        val_check_id = VIOS_MAXIMUM_VIRTUAL_SLOTS
        host = self.vios['ttv_vios_display_name']
        curr_max_virtual_slots = int(self.vios['vios_MaximumVirtualIOSlots'])
        if curr_max_virtual_slots:
            if curr_max_virtual_slots >= self.expected_max_virtual_slots:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = [host]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.HMC_VIOS_MAX_VIRTUAL_SLOTS_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.VIOS_CONFIGURATION
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.\
                    append(utils.checkResult(host, correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status, verbose, val_check_id))
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog.HMC_CA_VIOS_MAX_VIRTUAL_SLOTS_FAILURE,
                    desired_locale)
                msgplaceholders = [curr_max_virtual_slots,
                                   self.expected_max_virtual_slots, host]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.HMC_VIOS_MAX_VIRTUAL_SLOTS_FAILURE,
                    desired_locale)
                messageID = messageIDCatalog.\
                    VIOS_MINIMUM_MAX_VIRTUAL_SLOTS_INCORRECT
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.\
                    append(utils.checkResult(host, correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status, verbose, val_check_id))
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.HMC_CA_VIOS_MAX_VIRTUAL_SLOTS_FAILURE,
                desired_locale)
            msgplaceholders = [curr_max_virtual_slots,
                               self.expected_max_virtual_slots, host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.HMC_VIOS_MAX_VIRTUAL_SLOTS_FAILURE,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_MINIMUM_MAX_VIRTUAL_SLOTS_INCORRECT
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.\
                append(utils.checkResult(host, correctiveAction,
                                         camsgplaceholders,
                                         ttvcatalogmsgID,
                                         msgplaceholders,
                                         messageID,
                                         msgSubstitues,
                                         status, verbose, val_check_id))
        return (status, self.result)


class CurrentProfileSyncStateChecker():

    def __init__(self, vios, expected_current_profile_sync_state):
        self.vios = vios
        self.expected_current_profile_sync_state = \
            expected_current_profile_sync_state.lower().strip()
        self.result = []

    def check(self):
        status = ""
        val_check_id = VIOS_CURRENT_PROFILE_SYNC_STATE
        host = self.vios['ttv_vios_display_name']
        current_profile_sync_state = self.vios['vios_CurrentProfileSync']
        if current_profile_sync_state and current_profile_sync_state != "":
            if current_profile_sync_state.lower() ==\
                    self.expected_current_profile_sync_state:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = [host, current_profile_sync_state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.VIOS_CURRENT_PROFILE_SYNC_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.VIOS_HMC_CONFIGURATION
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.\
                    append(utils.checkResult(host, correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status, verbose, val_check_id))
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog.CA_VIOS_CURRENT_PROFILE_SYNC,
                    desired_locale)
                msgplaceholders = [host, current_profile_sync_state,
                                   self.expected_current_profile_sync_state]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.VIOS_CURRENT_PROFILE_SYNC_FAILED,
                    desired_locale)
                messageID = messageIDCatalog.\
                    VIOS_HMC_CONFIGURATION_ERROR
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.\
                    append(utils.checkResult(host, correctiveAction,
                                             camsgplaceholders,
                                             ttvcatalogmsgID,
                                             msgplaceholders,
                                             messageID,
                                             msgSubstitues,
                                             status, verbose, val_check_id))
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_UNABLE_TO_QUERY_VIOS_PROPERTIES,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.UNABLE_TO_QUERY_VIOS_PROPERTIES,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_CONFIGURATION_INCORRECT
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.\
                append(utils.checkResult(host, correctiveAction,
                                         camsgplaceholders,
                                         ttvcatalogmsgID,
                                         msgplaceholders,
                                         messageID,
                                         msgSubstitues,
                                         status, verbose, val_check_id))
        return (status, self.result)


def get_registered_cecs():
    """ helper method to load HMC hosts information"""
    hmchosts = []
    if len(hmchosts) == 0:
        config_file_path = "/etc/nova"
        conffiles = glob.glob("{0}/nova-*.conf".format(config_file_path))

        for conffile in conffiles:
            if conffile == "/etc/nova/nova-health.conf":
                continue
            with open(conffile, "r") as f:
                lines = f.read()
            uuid = getValue('powervm_mgr_uuid', lines)
            if uuid is not None:
                hmchosts.append(uuid)
    return hmchosts


def getValue(key, buffer):
    """ helper method to fnd the value of key in buffer"""
    match = None
    for line in buffer.splitlines():
        if key in line:
            match = line
            break
    if match is not None:
        return match[match.find("=") + 2:].strip()
    else:
        return None


def compare_vios_levels(current_level, expected_level):
    if current_level is None or expected_level is None:
        return -1

    try:
        current_level_tokens = current_level.split(".")
        expected_level_tokens = expected_level.split(".")

        if len(current_level_tokens) != len(expected_level_tokens):
            return -1  # error in levels

        for current_level_token, expected_level_token \
                in zip(current_level_tokens, expected_level_tokens):
            if current_level_token > expected_level_token:
                return 0
            if current_level_token < expected_level_token:
                return 1
    except Exception:
        return -1

    return 0


def main():
    # Parse the command line arguments
    rc = main_func()
    exit(rc)

if __name__ == "__main__":
    main()
