#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import json
import glob
import os
from powervc_ttv.resources import XIVStorageResource
from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import _
from powervc_ttv import translate_message

output_dict = {
    "ValidationID": "XIVSTORAGE_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "xiv_storage",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}

XIV_STORAGE_REACHABILITY = "XIV_STORAGE_REACHABILITY"
XIV_STORAGE_MODEL = "XIV_STORAGE_MODEL"
XIV_STORAGE_MAX_HOST_LIMIT = "XIV_STORAGE_MAX_HOST_LIMIT"
XIV_STORAGE_FIRMWARE = "XIV_STORAGE_FIRMWARE"
XIV_STORAGE_VALIDATION = "XIV_STORAGE_VALIDATION"
NETWORK_QUALITY_CHK = "NETWORK_QUALITY_CHK"
logFile = "xivstorage_validator.log"


def main_func():
    rc = 0
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()
    if rc == 1:
        return 1

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.XIV_STORAGE_VALIDATIONS,
        desired_locale)
    resource_dict = readConf(logger)

    validationProp = utils.readJSONFile(
        "/opt/ibm/powervc/ttv-validation/"
        "conf/solution_spec.json")["xiv_storage"]

    for resource in resource_dict:
        try:
            validateResource(validationProp, resource, logger)
        except Exception as e:
            logger.exception(_(e))
    print(json.dumps({"ValidationResult": output_dict}))
    return rc


class ConfigFileError(Exception):
    pass


class ReachabilityChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        reachability = utils.checkReachability(host)
        if reachability == 0:
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = messageIDCatalog.XIV_RESOURCE_CREDENTIALS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_REACHABILITY))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = messageIDCatalog.XIV_RESOURCE_CREDENTIALS_FAIL

            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_REACHABILITY))
        return (status, self.result)


class ModelChecker():

    def __init__(self, resource, expmodel):
        self.resource = resource
        self.expmodel = expmodel
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        model = self.resource.getModel()
        if model in self.expmodel.split(","):
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.XIV_MODEL_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.XIV_MODEL_SUCCESS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_MODEL))
        else:
            camsgplaceholders = [self.model]
            correctiveAction = translate_message(
                ttvCatalog.CA_XIV_MODEL_FAIL,
                desired_locale)
            msgplaceholders = [self.expzmodel]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.XIV_MODEL_FAIL,
                desired_locale)
            messageID = messageIDCatalog.XIV_MODEL_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_MODEL))
        return (status, self.result)


class FirmwareChecker():

    def __init__(self, resource, expfirmware):
        self.resource = resource
        self.expfirmware = expfirmware
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        firmware = self.resource.getFirmware()
        if utils.compareVersion(firmware, self.expfirmware):
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.XIV_FIRMWARE_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.XIV_FIRMWARE_SUCCESS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_FIRMWARE))
        else:
            camsgplaceholders = [firmware, self.expfirmware]
            correctiveAction = translate_message(
                ttvCatalog.CA_XIV_FIRMWARE_FAIL,
                desired_locale)
            msgplaceholders = [firmware, self.expfirmware]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.XIV_FIRMWARE_FAIL,
                desired_locale)
            messageID = messageIDCatalog.XIV_FIRMWARE_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_FIRMWARE))
        return (status, self.result)


class MaxHostLimitChecker():

    def __init__(self, resource, explimit):
        self.resource = resource
        self.explimit = explimit
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        (limit, currentCount) = self.resource.getMaxHostLimit()
        actulimit = "false"
        if (limit - currentCount) > 0:
            actulimit = "true"
        if actulimit == self.explimit.lower():
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [limit, currentCount]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.XIV_HOST_EXHAUSTION_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.XIV_HOST_EXHAUSTION_SUCCESS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_MAX_HOST_LIMIT))
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_XIV_HOST_EXHAUSTION_FAIL,
                desired_locale)
            msgplaceholders = [limit, currentCount]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.XIV_HOST_EXHAUSTION_FAIL,
                desired_locale)
            messageID = messageIDCatalog.XIV_HOST_EXHAUSTION_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 XIV_STORAGE_MAX_HOST_LIMIT))
        return (status, self.result)


class NetworkQualityChecker():

    def __init__(self, ip):
        self.ip = ip
        self.result = []

    def check(self):
        status = ""
        host = self.ip
        quality = utils.checkNetworkQuality(self.ip)
        quality = int(quality)

        if quality == 100:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_BAD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality >= 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_DEGRADED,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        elif quality < 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_GOOD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))

        return (status, self.result)


def validateResource(validationProp, resource, logger=None):
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]
    exception = False
    host = resource["hostname"]
    xivresourcemgr = XIVStorageResource.XIVStorageResource(
        resource, logger)
    checkers = []
    checkers.append(NetworkQualityChecker(host))
    checkers.append(
        FirmwareChecker(xivresourcemgr, validationProp["firmware"]))
    checkers.append(ModelChecker(
        xivresourcemgr, validationProp["model"]))
    checkers.append(MaxHostLimitChecker(
        xivresourcemgr, validationProp["limit"]))
    status = ""
    result = []
    try:
        (status, result) = ReachabilityChecker(xivresourcemgr).check()
        chkResultList.extend(result)
    except Exception as e:
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        messageID = messageIDCatalog.XIV_RESOURCE_CREDENTIALS_FAIL
        msgSubstitues = ""
        status = "Critical"
        verbose = ""
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               XIV_STORAGE_REACHABILITY))
        logger.exception(_(e))
    if status == "Success" and not exception:

        for checker in checkers:
            status = ""
            result = []

            try:
                (status, result) = checker.check()
                chkResultList.extend(result)
            except Exception as e:
                logger.exception(_(e))
                exception = True

    if exception:
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.CA_XIV_DETAILS_RETRIEVE_FAILED,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.XIV_DETAILS_RETRIEVE_FAILED,
            desired_locale)
        messageID = messageIDCatalog.XIV_DETAILS_RETRIEVE_FAILED
        msgSubstitues = ""
        status = "Critical"
        verbose = "verbose"
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               XIV_STORAGE_VALIDATION))


def readConf(logger):
    env = {}
    env["xiv-storage"] = []
    for conf in glob.glob("/etc/cinder/cinder-*.conf"):
        if conf == "/etc/cinder/cinder-health.conf":
            continue
        try:
            logger.info(_("reading cinder.conf file {0}".format(conf)))
            cinder_conf = ConfigParser.ConfigParser()
            cinder_conf.read(conf)
            if cinder_conf.has_option('DEFAULT', 'host_type'):
                host_type = cinder_conf.get('DEFAULT', 'host_type')
            if host_type == 'xiv' or host_type == 'XIV':
                xiv_storage = {}
                # the san_ip may in fact be a hostname, so convert to IP as ECT
                # must have an IP address
                ip = cinder_conf.get("DEFAULT", "san_ip")
                xiv_storage["hostname"] = cinder_conf.get("DEFAULT", "san_ip")
                if not ip:
                    logger.error(_(("Name resolution failure in %(conf)s \
                    file - name is %(san_ip)s")
                                   % dict(conf=conf, san_ip=cinder_conf.
                                          get("DEFAULT", "san_ip"))))
                xiv_storage["ip"] = ip

                xiv_storage["userid"] = cinder_conf.get("DEFAULT", "san_login")

                if cinder_conf.has_option("DEFAULT", "san_private_key"):
                    if os.path.exists(cinder_conf.
                                      get("DEFAULT", "san_private_key")):
                        xiv_storage["key-file"] = \
                            cinder_conf.get("DEFAULT", "san_private_key")
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    xiv_storage["passwd"] = cinder_conf.get("DEFAULT",
                                                            "san_password")
                for prop in ('san_clustername', 'san_ssh_port'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        xiv_storage[prop] = cinder_conf.get('DEFAULT', prop)
                env["xiv-storage"].append(xiv_storage)

        except ConfigParser.NoOptionError, e:
            logger.error(_("cinder.conf file %(conf)s error: %(err_msg)s"
                           % dict(conf=conf, err_msg=e)))
        except ConfigFileError, e:
            logger.warning(_(e))

    return env["xiv-storage"]


def main():
    rc = 0
    # Parse the command line arguments
    rc = main_func()
    exit(rc)

if __name__ == "__main__":
    main()
