#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import re
from powervc_ttv.utils import utils
from powervc_oslo.wbem import pywbem
from powervc_ttv import _

base_nova_conf_file = "/etc/nova/nova.conf"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"
powervc_certs_path = "/opt/ibm/powervc/data/certs/"

CMD_LSPORTIP = 'svcinfo lsportip -nohdr -filtervalue'\
    ' "link_state=active":"state=configured" -delim !'
CMD_LSFABRIC = 'svcinfo lsfabric -nohdr -delim :'
CMD_SHOW_CLUSTER = 'svcinfo lscluster -nohdr -filtervalue name=* -delim :'
CMD_SHOW_VERSION = 'svcinfo lscluster '

"""
CIM Class Names for VMAX storage
Enumerate Instances
"""
# Release check
EMC_VMAX_RELEASE_CHECK = 'EMC_ArrayProduct'
# Version check
EMC_VMAX_VERSION_CHECK = 'EMC_ManagementServerSoftwareIdentity'
# Login check
EMC_VMAX_LOGIN_CHECK = 'EMC_UnifiedStoragePool'
# License related services
EMC_VMAX_LICENSE_SERVICE_NAMES = ['EMC_StorageConfigurationService',
                                  'EMC_ReplicationService',
                                  'Symm_ElementCompositionService',
                                  'EMC_ControllerConfigurationService',
                                  'EMC_StorageHardwareIDManagementService']
# Storage Port Groups
EMC_VMAX_PORT_GROUPS = 'CIM_TargetMaskingGroup'
# Storage ID Information
EMC_VMAX_SE_STORAGE_HW_ID = 'SE_StorageHardwareID'


class EMCVMAXStorageResource():

    def __init__(self, resource, logger=None, timeout=60):
        self.ip = resource["hostname"]
        if resource["port"] is None:
            self.port = 5989
        else:
            self.port = resource["port"]
        self.user = resource["userid"]
        self.pwd = utils.getClearTextString(resource["passwd"])
        self.LOG = logger
        self.con = None

    def checkCredential(self):
        self.LOG.debug("checking credential for resource: " + self.ip)
        credential = 1
        if self.con is None:
            self.con = self._get_ecom_connection(
                self.ip,
                self.user,
                self.pwd,
                self.port)
            credential = 0
        return credential

    # Establish Connection with the provider
    def _get_ecom_connection(self, provider_ip, provider_user, provider_pass,
                             provider_port=5989):

        self.LOG.debug("Get pywbem Ecom Connection")
        ip = provider_ip
        port = provider_port
        user = provider_user
        passW = provider_pass
        url = 'https://' + str(ip) + ':' + str(port)
        self.LOG.debug("The url to connect to pywbem to is " + url)
        cert = powervc_certs_path + re.sub('[:.]', '_', str(ip)) + '.crt'
        conn = pywbem.WBEMConnection(
            url,
            (user,
             passW),
            default_namespace='root/emc',
            ca_certs=cert)
        if conn is None:
            self.LOG.error(_("Request Not Found could not"
                             " establish ECOM connection"))
            self.LOG.debug(
                "Request Not Found, could not establish ECOM connection")

        return conn

    '''
    Reads VMAX version
    @param conn: A valid connection object
    '''

    def get_vmax_license_info(self):
        if self.con is None:
            self.con = self._get_ecom_connection()
        service_objects = []
        invalid_license = False
        try:
            self.LOG.debug("Fetching VMAX License Info")
            for service in EMC_VMAX_LICENSE_SERVICE_NAMES:
                service_obj = self.con.EnumerateInstanceNames(service)
                instance_detail = self.con.GetInstance(service_obj[0])
                service_objects.append(instance_detail)

            for service_obj in service_objects:
                # add a validation check here.
                self.LOG.debug("service_obj['enabledstate'] %s" %
                               service_obj['enabledstate'])
                if str(service_obj['enabledstate']).strip() != '5':
                    invalid_license = True
        except Exception as ex:
            self.LOG.exception(ex)
            self.LOG.error(_("Socket Error Connection Refused"))
            self.LOG.debug("Socket Error Connection Refused, "
                           "unable to communicate with provider")
            return False, 'Failed_to_get'

        if invalid_license:
            return False, 'Invalid'
        return True, 'Valid'

    '''
    Reads VMAX version
    @param conn: A valid connection object
    '''

    def get_vmax_version(self):
        try:
            if self.con is None:
                self.con = self._get_ecom_connection()
            self.LOG.debug("Fetching VMAX version info")
            versions = self.con.EnumerateInstances(EMC_VMAX_VERSION_CHECK)
        except:
            self.LOG.error(_("Socket Error Connection Refused"))
            return False, 'Failed_to_get'
        return True, versions[0]

    '''
    Validate EMC Symmetric Array
    '''

    def get_vmax_symmetric_array(self):
        ''' '''
        try:
            if self.con is None:
                self.con = self._get_ecom_connection()
            self.LOG.debug("Fetching VMAX Array info")
            arrays = self.con.EnumerateInstances(EMC_VMAX_RELEASE_CHECK)
        except:
            self.LOG.error(_("Socket Error Connection Refused"))
            return False, 'Failed_to_get'
        return True, arrays[0]

    '''
    getConnectivityList for all connected initiators
    '''

    def getConnectivityList(self):
        '''return FCAndHostConnectivityList'''
        # Add implementation for checking connectivity list
        fc_host_connectivity_dict = {}
        wwpn_set = set()
        host_wwpn_dict = {}
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set
        fc_host_connectivity_dict["host_wwpn_dict"] = host_wwpn_dict
        # hostname is empty now
        # as SE_StorageHardwareID does not return any name.
        hostname = ''
        host_wwpn_dict[hostname] = []

        if self.con is None:
            self.con = self._get_ecom_connection(
                self.ip,
                self.user,
                self.pwd,
                self.port)
        try:
            self.LOG.debug("Fetching Host WWPNs")
            se_hw_id_list = self.con.EnumerateInstances(
                EMC_VMAX_SE_STORAGE_HW_ID)
            wwn_set = set()
            for se_hw_id in se_hw_id_list:
                wwn_set.add((se_hw_id['StorageID']).upper())
            # prepare dict
            host_wwpn_dict[hostname] = []
            host_wwpn_dict[hostname] = wwn_set

            host_wwpn_dict[hostname] = host_wwpn_dict.get(hostname)
        except:
            self.LOG.error(_("Socket Error Connection Refused"))
            self.LOG.debug("Socket Error Connection Refused, "
                           "unable to communicate with provider")

        self.LOG.debug("System FC port for EMC VNX:%s" % self.ip)
        self.LOG.debug(fc_host_connectivity_dict)

        return fc_host_connectivity_dict

    '''
    get FC port wwpns of this storage
    '''

    def getSystemFCPorts(self):
        '''return SystemFCPorts'''
        self.LOG.debug("Checking System FC Port Info")
        fc_host_connectivity_dict = {}
        wwpn_set = set()
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set
        if self.con is None:
            self.con = self._get_ecom_connection(
                self.ip,
                self.user,
                self.pwd,
                self.port)
        try:
            mask_groups = self.con.EnumerateInstances(EMC_VMAX_PORT_GROUPS)
            for group in mask_groups:

                # get details of all ports
                port_info_list = self.con.AssociatorNames(group.path)

                for port_info in port_info_list:
                    if port_info['CreationClassName'] == \
                            'Symm_FCSCSIProtocolEndpoint':
                        wwpn_set.add((port_info['name']).upper())
        except:
            self.LOG.error(_("Socket Error Connection Refused"))
            self.LOG.debug("Socket Error Connection Refused, "
                           "unable to communicate with provider")

        self.LOG.debug("System FC port for EMC VNX:%s" % self.ip)
        self.LOG.debug(fc_host_connectivity_dict)
        return wwpn_set
