#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_ttv.utils import utils
from powervc_ttv import _
import os
import re
import subprocess

port = 22


class EMCVNXResource():

    def __init__(self, emcvnxip, emcvnxUser, emcvnxPasswd, keyFile,
                 navisecclipath, logger):
        self.emcvnxip = emcvnxip
        self.emcvnxUser = emcvnxUser
        self.emcvnxPasswd = emcvnxPasswd
        self.keyFile = keyFile
        self.navisecclipath = navisecclipath
        self.logger = logger

    def getFirmware(self):
        actualFirmWare = ""
        try:
            subcmd = "getagent"
            (returnCode, stdout, stderr) = self.runNavSecCLICommand(
                self.navisecclipath, self.emcvnxip, port, self.emcvnxUser,
                self.emcvnxPasswd, self.keyFile, subcmd)
            stdout = open("/tmp/output", "r").read().splitlines()
            os.remove("/tmp/output")

            if returnCode == 0 and stdout is not None:
                for line in stdout:
                    if "Revision:" in line:
                        elements = line.split(":")
                        actualVer = ".".join(
                            [elements[1].strip().split(".")[0],
                             elements[1].strip().split(".")[1]])
                        actualFirmWare = actualVer
        except:
            actualFirmWare = None
        return actualFirmWare

    def checkNavisecCLIPathExists(self):
        return os.path.exists(self.navisecclipath)

    def getLicensesDetails(self):
        enablers = []
        try:
            subcmd = "ndu -list"
            (returnCode, stdout, stderr) = self.runNavSecCLICommand(
                self.navisecclipath, self.emcvnxip, port, self.emcvnxUser,
                self.emcvnxPasswd, self.keyFile, subcmd)

            stdout = open("/tmp/output", "r").read().splitlines()
            os.remove("/tmp/output")
            enabler_pat = r'Name of the software package:\s*(\S+)\s*'
            if returnCode == 0 and stdout is not None:
                out = "\n".join(stdout)
                self.logger.debug(stdout)
                for m in re.finditer(enabler_pat, out):
                    enablers.append(m.groups()[0])
        except Exception as e:
            self.logger.exception(_(
                "Exception Occurred. during fetching license details"
                "details of exception: {0}".format(str(e))))
        self.logger.info(_("Returning enablers.{0}".format(enablers)))
        return enablers

    def runNavSecCLICommand(self, navisecclipath, emcvnxip, port, emcvnxUser,
                            emcvnxPasswd, keyFile, subcmd):
        emcvnxPasswd = utils.getClearTextString(emcvnxPasswd)
        if keyFile is None:
            cmd = '%s -Address %s -User %s -Password %s -Scope 0 -Timeout ' +\
                '30 %s'
            cmd = cmd % (navisecclipath, emcvnxip,
                         emcvnxUser, emcvnxPasswd, subcmd)
        else:
            cmd = 'sudo -u cinder %s -Address %s -secfilepath %s -Timeout ' +\
                '30 %s'
            cmd = cmd % (navisecclipath, emcvnxip, keyFile, subcmd)

        self.logger.info(_("executing NaviSecCLI command:{0}".format(subcmd)))
        try:
            out_file = "/tmp/output"
            pipe_output = file(out_file, 'w')

            p = subprocess.Popen(
                cmd.split(), stdout=pipe_output, stderr=subprocess.PIPE)
            p.wait()
            stderr = p.communicate()[1]
            returnCode = p.returncode
            if (returnCode > 0):
                raise Exception("unable to execute NaviSecCLI command")
            if pipe_output:
                pipe_output.close()

            output = []
            err_output = stderr.splitlines()

            self.logger.info(_(
                "EMC VNX NaviSecCLI command [{0}] returned returnCode: "
                "{1} stdout: {2} stderr: {3}".format(subcmd, returnCode,
                                                     output, err_output)))
            return (returnCode, output, err_output)
        except Exception as e:
            self.logger.exception(_(
                "Error while running command {0}: {1}".format(subcmd, str(e))))
            raise e

        '''
    getConnectivityList
    '''

    def getConnectivityList(self):
        '''return FCAndHostConnectivityList'''
        fc_host_connectivity_dict = {}
        wwpn_set = set()
        host_wwpn_dict = {}
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set
        fc_host_connectivity_dict["host_wwpn_dict"] = host_wwpn_dict

        subcmd = "port -list -hba"
        if not os.path.exists(self.navisecclipath):
            return
        returnCode, stdout, stderr = self.runNavSecCLICommand(
            self.navisecclipath,
            self.emcvnxip,
            port,
            self.emcvnxUser,
            self.emcvnxPasswd,
            self.keyFile,
            subcmd)

        stdout = open("/tmp/output", "r").read().splitlines()
        os.remove("/tmp/output")

        self.logger.debug("vnx connecitivity returnCode %s" % returnCode)
        if returnCode == 0 and stdout is not None:
            stdout = "\n".join(stdout)
            eachhbas = stdout.split("Information about each HBA:")
            for hbainfo in eachhbas:
                testhost = 'empty_unknown_zoned_unconfigured_ttv99999'
                lines = hbainfo.splitlines()
                wwpn = ''
                hostname = ''
                loggedin = False
                for line in lines:
                    if "HBA UID:" in line:
                        uid = line[
                            line.index("HBA UID:") + len("HBA UID:"):].strip()
                        wwpn = (uid.replace(":", "")[16:]).upper()
                    if "Server Name:" in line:
                        hostname = line[
                            line.index("Server Name:") + len("Server Name:"):]\
                            .strip()
                        if uid == hostname:
                            hostname = testhost
                    if "Logged In:" in line and "YES" in line:
                        loggedin = True
    #                if "Defined:" in line and "YES" in line and "":
    #                    wwpn_set.add((wwpn).upper())
                if loggedin:
                    if hostname in host_wwpn_dict.keys():
                        if host_wwpn_dict.get(hostname):
                            host_wwpn_dict.get(hostname).add(wwpn)
                        else:
                            wwpnset = set()
                            wwpnset.add(wwpn)
                            host_wwpn_dict[hostname] = wwpnset
                    else:
                        wwn_set = set()
                        wwn_set.add(wwpn)
                        host_wwpn_dict[hostname] = []
                        host_wwpn_dict[hostname] = wwn_set

                    host_wwpn_dict[hostname] = host_wwpn_dict.get(hostname)

        self.logger.debug("Connectivity list EMC VNX:%s" % self.emcvnxip)
        self.logger.debug(fc_host_connectivity_dict)

        return fc_host_connectivity_dict

    '''
    get FC port wwpns of this storage
    '''

    def getSystemFCPorts(self):
        '''return SystemFCPorts'''
        fc_host_connectivity_dict = {}

        wwpn_set = set()
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set

        subcmd = "port -list -sp"
        if not os.path.exists(self.navisecclipath):
            return
        returnCode, stdout, stderr = self.runNavSecCLICommand(
            self.navisecclipath,
            self.emcvnxip,
            port,
            self.emcvnxUser,
            self.emcvnxPasswd,
            self.keyFile,
            subcmd)

        stdout = open("/tmp/output", "r").read().splitlines()
        os.remove("/tmp/output")

        if returnCode == 0 and stdout is not None:
            stdout = "\n".join(stdout)
            eachhbas = stdout.split("Information about each HBA:")
            for hbainfo in eachhbas:
                lines = hbainfo.splitlines()
                wwpn = ''
                for line in lines:
                    if "SP UID:" in line:
                        uid = line[
                            line.index("SP UID:") + len("SP UID:"):].strip()
                        wwpn = (uid.replace(":", "")[16:]).upper()
                    if "Port Status:" in line and "Online" in line:
                        wwpn_set.add((wwpn).upper())

        self.logger.debug("System FC port for EMC VNX:%s" % self.emcvnxip)
        self.logger.debug(fc_host_connectivity_dict)
        return wwpn_set
