#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ast
import subprocess
from powervc_ttv import _


class cmdHandler:

    def __init__(self, logger, timedOut=60):
        self.logger = logger
        self.timedOut = timedOut

    # Internal method to execute a command
    # RC 0 - Cmd execution with zero return code
    # RC 1 - Cmd executed with non zero return code
    # RC 2 - Runtime error / exception
    def cmd_run(self, cmd):
        cmd_output = ''
        cmd_err = ''
        rc = 0

        # self.logger.info("CommmandExec \"{0}\"".format(cmd))
        try:
            #TODO: Defect 47986 - this was flagged in a Bandit Scan.
            #Will need to fix subprocess issue in 2Q
            process = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE, shell=True)
            (cmd_output, cmd_err) = process.communicate()
#        except subprocess.TimeoutExpired as err:
#            cmd_err = "Command {0} Timedout".format(cmd)
#            self.logger.error(cmd_err)
#            rc = 2
        except Exception as err:
            self.logger.error(_("Command execution failed"))
            self.logger.error(_(err))
            cmd_err = "Command execution failed"
            rc = 2
        else:
            if process.returncode == 0:
                rc = 0
            else:
                rc = 1
            self.logger.info(_("CmdReturnCode \"{0}\"".format(str(rc))))
            self.logger.info(_("CmdOutput \"{0}\"".format(cmd_output)))
            self.logger.info(_("CmdError \"{0}\"".format(cmd_err)))
        return (rc, cmd_output, cmd_err)


class HMCResource():

    def __init__(self, hmcip, hmcUser, hmcPasswd, logger, timeout,
                 hostsUUID=[]):
        self.hmcip = hmcip
        self.hmcUser = hmcUser
        self.hmcPasswd = hmcPasswd
        self.hostsUUID = hostsUUID
        self.logger = logger
        self.cmdHandle = cmdHandler(self.logger, timeout)

    def getHMCVersionDetails(self):
        hmcResource = {}
        # Invokes the script that calls K2 api.
        cmd = 'python /usr/lib/python2.7/site-packages/powervc_ttv/utils/' + \
            'powervc_k2_scripts/hmc_version.py -i \"' + \
            self.hmcip + '\" -u \"' + self.hmcUser + \
            '\" -p \"' + self.hmcPasswd + '\"'
        self.logger.info(_("Going to execute hmc_version.py"))
        # gets back the return code and results in the output stream
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)

        if rc == 0:
            # parse the results string and get the information in a dict
            hmcResource = self.parse_command_output(cmdOutput)

        self.logger.info(_("HMC Resource Details: {0}".format(hmcResource)))
        return (rc, cmdOutput, cmdErr, hmcResource)

    def parse_command_output(self, cmdOutput):
        hmcResource = {}
        hmcResource['id'] = None
        hmcResource['name'] = None
        hmcResource['version'] = None
        hmcResource['release'] = None
        hmcResource['maintenance'] = None
        cmdOutputList = cmdOutput.splitlines()
        for currLine in cmdOutputList:
            split = currLine.split("=")
            splitLen = len(split)

            if "Start querying for ManagementConsole" in currLine:
                hmcResource = {}
            elif "hmc_id" in currLine and splitLen == 2:
                hmcResource['id'] = split[1].strip()
            elif "HMC_Name" in currLine and splitLen == 2:
                hmcResource['name'] = split[1].strip()
            elif "HMC_Version" in currLine and splitLen == 2:
                hmcResource['version'] = split[1].strip()
            elif "HMC_Release" in currLine and splitLen == 2:
                hmcResource['release'] = split[1].strip()
            elif "HMC_Maintenance" in currLine and splitLen == 2:
                hmcResource['maintenance'] = split[1].strip()

        return hmcResource

    def getHMCManagedResourcesDetails(self):
        hmcManagedResourceList = []
        self.cecIDs = ",".join(self.hostsUUID)

        # Invokes the script that calls K2 api.
        cmd = 'python /usr/lib/python2.7/site-packages/powervc_ttv/utils/' + \
            'powervc_k2_scripts/hmc_managedsystem.py -i \"' + \
            self.hmcip + '\" -u \"' + self.hmcUser + \
            '\" -p \"' + self.hmcPasswd + '\" -c \"' + self.cecIDs + '\"'

        self.logger.info(_("Going to execute hmc_managedsystem.py"))
        # gets back the return code and results in the output stream
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        if rc == 0:
            # parse the results string and get the HMC Host details
            hmcManagedResourceList =\
                self.parse_managed_resource_command_output(cmdOutput)

        self.logger.info(_("HMC Managed Resource Details: {0}".format(
            hmcManagedResourceList)))
        return (rc, cmdOutput, cmdErr, hmcManagedResourceList)

    def get_hmc_managed_resource_details(self):
        hmc_managed_resource_list = []
        self.cecIDs = ",".join(self.hostsUUID)

        # Invokes the script that calls K2 api.
        cmd = 'python /usr/lib/python2.7/site-packages/powervc_ttv/utils/' + \
            'powervc_k2_scripts/hmc_details.py -i \"' + \
            self.hmcip + '\" -u \"' + self.hmcUser + \
            '\" -p \"' + self.hmcPasswd + '\" -c \"' + self.cecIDs + '\"'

        self.logger.info(_("Going to execute hmc_details.py"))
        # gets back the return code and results in the output stream
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        if rc == 0:
            # parse the results string and get the HMC Host details
            hmc_managed_resource_list =\
                self.parse_hmc_managed_resource_details(cmdOutput)

        return (rc, cmdOutput, cmdErr, hmc_managed_resource_list)

    def parse_hmc_managed_resource_details(self, cmdOutput):
        hmcResource = {}
        hmcResource['id'] = None
        hmcResource['name'] = None
        hmcResource['version'] = None
        hmcResource['release'] = None
        hmcResource['maintenance'] = None
        hmcResource['managed_systems'] = None

        hmc_managed_cecs = []
        vioses_managed_by_cec = []

        cmdOutputList = cmdOutput.splitlines()
        for curr_line in cmdOutputList:
            split = curr_line.split("=")
            split_len = len(split)

            if "Start querying for ManagementConsole" in curr_line:
                pass
            elif "hmc_id" in curr_line and split_len == 2:
                hmcResource['id'] = split[1].strip()
            elif "HMC_Name" in curr_line and split_len == 2:
                hmcResource['name'] = split[1].strip()
            elif "HMC_Version" in curr_line and split_len == 2:
                hmcResource['version'] = split[1].strip()
            elif "HMC_Release" in curr_line and split_len == 2:
                hmcResource['release'] = split[1].strip()
            elif "HMC_Maintenance" in curr_line and split_len == 2:
                hmcResource['maintenance'] = split[1].strip()

            elif "Start querying for ManagedSystem" in curr_line:
                hmcManagementSystem = {'vios_list': []}
                vioses_managed_by_cec = []
            elif "mgmt_id" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_id'] = split[1].strip()
            elif "mgmt_MachineType" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_MachineType'] = split[1].strip()
            elif "mgmt_Model" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_Model'] = split[1].strip()
            elif "mgmt_SerialNumber" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_SerialNumber'] = split[1].strip()
            elif "mgmt_MTMS" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_MTMS'] = split[1].strip()
            elif "mgmt_SystemName" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_SystemName'] = split[1].strip()
            elif "mgmt_State" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_State'] = split[1].strip()
            elif "mgmt_PrimaryIPAddress" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_PrimaryIPAddress'] = split[1].strip()
            elif "mgmt_ConfigurableSystemMemory" in curr_line \
                    and split_len == 2:
                hmcManagementSystem['mgmt_ConfigurableSystemMemory'] = \
                    split[1].strip()
            elif "mgmt_ConfigurableSystemProcessorUnits" in curr_line \
                    and split_len == 2:
                hmcManagementSystem['mgmt_ConfigurableSystemProcessorUnits'] =\
                    split[1].strip()
            elif "mgmt_SupportedPartitionProcessorCompatibilityModes" in \
                    curr_line and split_len == 2:
                MODES = 'mgmt_SupportedPartitionProcessorCompatibilityModes'
                hmcManagementSystem[MODES] = split[1].strip()
            elif "mgmt_VirtualIOServerCapable" in curr_line and split_len == 2:
                hmcManagementSystem['mgmt_VirtualIOServerCapable'] = \
                    split[1].strip()
            elif "mgmt_MicroLogicalPartitionCapable" in curr_line \
                    and split_len == 2:
                hmcManagementSystem['mgmt_MicroLogicalPartitionCapable'] = \
                    split[1].strip()
            elif "mgmt_ActiveLogicalPartitionMobilityCapable" in curr_line \
                    and split_len == 2:
                LPM_CAPABILITY = 'mgmt_ActiveLogicalPartitionMobilityCapable'
                hmcManagementSystem[LPM_CAPABILITY] = split[1].strip()
            elif "PowerVMLogicalPartitionSimplifiedRemoteRestartCapable" \
                    in curr_line and split_len == 2:
                HOST_SRR_CAPABILITY = \
                    'PowerVMLogicalPartitionSimplifiedRemoteRestartCapable'
                hmcManagementSystem[HOST_SRR_CAPABILITY] = split[1].strip()
            elif "Start querying details of VIOS" in curr_line:
                vios = {}
                sea_list = []
                pea_list = []
                vea_list = []
                vea_additional_list = []
            elif "vios_id" in curr_line and split_len == 2:
                vios['vios_id'] = split[1].strip()
            elif "vios_PartitionName" in curr_line and split_len == 2:
                vios['vios_PartitionName'] = split[1].strip()
            elif "vios_PartitionID" in curr_line and split_len == 2:
                vios['vios_PartitionID'] = split[1].strip()
            elif "vios_MaximumVirtualIOSlots" in curr_line and split_len == 2:
                vios['vios_MaximumVirtualIOSlots'] = split[1].strip()
            elif "vios_OperatiingSystemVersion" in curr_line \
                    and split_len == 2:
                vios['vios_OperatiingSystemVersion'] = split[1].strip()
            elif "vios_CurrentProfileSync" in curr_line and split_len == 2:
                vios['vios_CurrentProfileSync'] = split[1].strip()
            elif "vios_PartitionState" in curr_line and split_len == 2:
                vios['vios_PartitionState'] = split[1].strip()
            elif "vios_ResourceMonitoringControlState" in curr_line and \
                    split_len == 2:
                vios['vios_ResourceMonitoringControlState'] = split[1].strip()
            elif "vios_MediaRepositoryExists" in curr_line and split_len == 2:
                vios['vios_MediaRepositoryExists'] = split[1].strip()
            elif "vios_RepositorySize" in curr_line and split_len == 2:
                vios['vios_RepositorySize'] = split[1].strip()
            elif "vios_TotalMediaSize" in curr_line and split_len == 2:
                vios['vios_TotalMediaSize'] = split[1].strip()
            elif "vios_PhysicalFiberChannelWWPNs" in curr_line and \
                    split_len == 2:
                vios['vios_PhysicalFiberChannelWWPNs'] = split[1].strip()
            elif "vios_VirtualFiberChannelClientWWPNs" in curr_line and \
                    split_len == 2:
                vios['vios_VirtualFiberChannelClientWWPNs'] = split[1].strip()
            elif "vios_vea_PortVLANID" in curr_line and split_len == 2:
                vea_list.append(split[1].strip())
            elif "vios_vea_AdditionalVLANIDs" in curr_line and split_len == 2:
                vea_additional_list.append(split[1].strip())
            elif "Start querying SharedEthernetAdapter properties" \
                    in curr_line:
                sea = {}
                sea_trunk_vlanids = []
                sea_trunk_additional_vlanids = []
                self.logger.info('start querying sea is')
            elif "trunkPortVLANID" in curr_line and split_len == 2:
                sea_trunk_vlanids.append(split[1].strip())
                self.logger.info('sea_trunk_vlanids is')
                self.logger.info(sea_trunk_vlanids)
            elif "AdditionalVLANIDs" in curr_line and split_len == 2:
                sea_trunk_additional_vlanids.append(split[1].strip().split())
            elif "vios_sea_PortVLANID" in curr_line and split_len == 2:
                sea['vios_sea_PortVLANID'] = split[1].strip()
            elif "vios_sea_HighAvailabilityMode" in curr_line \
                    and split_len == 2:
                sea['vios_sea_HighAvailabilityMode'] = split[1].strip()
            elif "vios_sea_InterfaceName" in curr_line and split_len == 2:
                sea['vios_sea_InterfaceName'] = split[1].strip()
            elif "vios_sea_JumboFramesEnabled" in curr_line and split_len == 2:
                sea['vios_sea_JumboFramesEnabled'] = split[1].strip()
            elif "vios_sea_PortVLANID" in curr_line and split_len == 2:
                sea['vios_sea_PortVLANID'] = split[1].strip()
            elif "vios_sea_QualityOfServiceMode" in curr_line \
                    and split_len == 2:
                sea['vios_sea_QualityOfServiceMode'] = split[1].strip()
            elif "vios_sea_ThreadModeEnabled" in curr_line and split_len == 2:
                sea['vios_sea_ThreadModeEnabled'] = split[1].strip()
            elif "vios_sea_state" in curr_line and split_len == 2:
                sea['vios_sea_state'] = split[1].strip()
            elif "vios_sea_BackingDeviceName" in curr_line and split_len == 2:
                sea['vios_sea_BackingDeviceName'] = split[1].strip()
            elif "vios_sea_BackingDevice_PhysicalLocation" in curr_line \
                    and split_len == 2:
                sea['vios_sea_BackingDevice_PhysicalLocation'] = \
                    split[1].strip()
            elif "End querying SharedEthernetAdapter properties" in curr_line:
                sea['seatrunkVLANIDs'] = sea_trunk_vlanids
                sea['seatrunkAdditionalVLANIDs'] = sea_trunk_additional_vlanids
                sea_list.append(sea)
            elif "Start querying PhysicalEthernetAdapter properties" \
                    in curr_line:
                pea = {}
            elif "vios_pea_AdapterID" in curr_line and split_len == 2:
                pea['vios_pea_AdapterID'] = split[1].strip()
            elif "vios_pea_DeviceName" in curr_line and split_len == 2:
                pea['vios_pea_DeviceName'] = split[1].strip()
            elif "vios_sea_PortVLANID" in curr_line and split_len == 2:
                pea['vios_pea_PhysicalLocation'] = split[1].strip()
            elif "vios_pea_PhysicalLocation" in curr_line and split_len == 2:
                pea['vios_sea_QualityOfServiceMode'] = split[1].strip()
            elif "vios_pea_InterfaceName" in curr_line and split_len == 2:
                pea['vios_pea_InterfaceName'] = split[1].strip()
            elif "vios_pea_State" in curr_line and split_len == 2:
                pea['vios_pea_State'] = split[1].strip()
            elif "End querying PhysicalEthernetAdapter properties" \
                    in curr_line:
                pea_list.append(pea)
            elif "End querying details of VIOS" in curr_line:
                vios['seas'] = sea_list
                vios['peas'] = pea_list
                vios['veas'] = vea_list
                vios['veas_additional'] = vea_additional_list
                vioses_managed_by_cec.append(vios)
            elif "End querying for ManagedSystem" in curr_line:
                hmcManagementSystem['vios_list'] = vioses_managed_by_cec
                hmc_managed_cecs.append(hmcManagementSystem)

        # vios_list = vios_list.append(vios)
        hmcResource['managed_systems'] = hmc_managed_cecs
        return hmcResource

    def parse_managed_resource_command_output(self, cmdOutput):
        cmdOutputList = cmdOutput.splitlines()
        startIndex = cmdOutputList.index("managed systems details start")
        endIndex = cmdOutputList.index("managed systems details end")

        if endIndex == startIndex + 2:
            hmcManagedResourceString = cmdOutputList[startIndex + 1]

        hmcManagedResourceList = ast.literal_eval(hmcManagedResourceString)
        return hmcManagedResourceList

    def getHMCManagedAllHosts(self):
        hmcManagedALLHosts = []

        # Invokes the script that calls K2 api.
        cmd = 'python /usr/lib/python2.7/site-packages/powervc_ttv/utils/' + \
            'powervc_k2_scripts/hmc_managed_all_hosts_details.py -i \"' + \
            self.hmcip + '\" -u \"' + self.hmcUser + \
            '\" -p \"' + self.hmcPasswd + '\"'

        self.logger.info(_("Going to execute hmc_managed_all_host_details.py"))

        # gets back the return code and results in the output stream
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)

        if rc == 0:
            # parse the results string and get the information in a dict
            hmcManagedALLHosts = self.getAllManagedHosts(cmdOutput)

        self.logger.info(_("HMC Managed All host Details: {0}".format(
            hmcManagedALLHosts)))
        return (hmcManagedALLHosts)

    def getAllManagedHosts(self, cmdOutput):
        hmcManagedAllHosts = []
        cmdOutputList = cmdOutput.splitlines()
        for currLine in cmdOutputList:
            split = currLine.split("=")
            splitLen = len(split)
            if "id" in currLine and splitLen == 2:
                hmcManagedAllHosts.append(split[1].strip())
        return hmcManagedAllHosts
