#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================


from powervc_ttv import _
from powervc_ttv.utils import utils

CMD_SYSTEM_CPU = 'pvmctl logicalpartition list -d LogicalPartition.id' +\
    ' is_mgmt_partition has_dedicated ded.desired desired_units' +\
    ' desired_virtual -w "LogicalPartition.is_mgmt_partition=True"'
CMD_SYSTEM_MEMORY = 'pvmctl logicalpartition list -d logicalpartition.id' +\
    ' is_mgmt_partition mem.desired -w' +\
    ' "LogicalPartition.is_mgmt_partition=True"'
CMD_FIRMWARE = "lsmcode"
CMD_GET_UID = "cat /etc/passwd | grep "
CMD_GET_USER_GROUP = "groups "
CMD_VIOS_RMC_STATE = "pvmctl virtualioserver list -d ios.id ios.name " +\
    "ios.uuid state VirtualIOServer.rmc_state"
CMD_SEA_LIST = "pvmctl sea list --d SharedEthernetAdapter.pvid"
CMD_GET_MGMT_VSWITCH = "pvmctl vswitch list -d VirtualSwitch.name"


class NOVALINKResource():

    def __init__(self, resource, logger, timeout=60):
        self.ip = resource["powervm_host_ip"]
        self.user = resource["powervm_host_user"]
        self.pwd = resource["powervm_host_passwd"]
        self.keyFile = resource["powervm_host_private_key"]
        self.host_id = resource["host_uniqueid"]
        self.host_disp_name = resource["host_display_name"]
        self.host_dhcp_domain = resource["dhcp_domain"]
        self.LOG = logger
        self.ssh = None
        self.port = 22
        # TODO read from neutron.conf keystone_authtoken group on neo
        self.neutron_user = 'neutron'

    def checkCredential(self):
        self.LOG.debug("checking credential for powervm host: " + self.ip)
        is_credential_valid = False
        if self.ssh is None:
            try:
                self.ssh = utils.getSSHConnection(self.ip, self.user,
                                                  self.port, self.pwd,
                                                  self.keyFile)
                is_credential_valid = True
            except Exception as exp:
                self.LOG.error(exp)
        return is_credential_valid

    def execute_command(self, cmd):
        ssh_conn = utils.getSSHConnection(self.ip, self.user, self.port,
                                          self.pwd, self.keyFile)
        if ssh_conn:
            return utils.runSSHCommand(ssh_conn, cmd)

    def get_actual_CPU_details(self):
        rc, stdout, stderr = self.execute_command(CMD_SYSTEM_CPU)
        return rc, stdout, stderr

    def get_SEA_list(self):
        rc, stdout, stderr = self.execute_command(CMD_SEA_LIST)
        return rc, stdout, stderr

    def get_MGMTSWITCH(self):
        rc, stdout, stderr = self.execute_command(CMD_GET_MGMT_VSWITCH)
        return rc, stdout, stderr

    def get_actual_available_memory(self):
        rc, stdout, stderr = self.execute_command(CMD_SYSTEM_MEMORY)
        return rc, stdout, stderr

    def check_required_packages(self, req_pkg_cmd_list):
        pkg_status_dict = {}
        try:
            for pkg, pkg_cmd in req_pkg_cmd_list:
                (rc, stdout, stderr) = self.execute_command(pkg_cmd)
                if(rc == 0):
                    status = self._parse_pkg_cmd_std_output(stdout)
                elif(rc == 1):
                    status = "NOT_INSTALLED"
                else:
                    status = "UNABLE_TO_DETERMINE_PKG_STATUS"
                pkg_status_dict[pkg] = status
        except Exception as e:
            self.LOG.error(e)
        return pkg_status_dict

    def _parse_pkg_cmd_std_output(self, stdout):
        status = "NOT_INSTALLED"
        for entry in stdout:
            entry_val = entry.split(":")
            if (entry_val[0].lower().strip() == "status" and
                    entry_val[1].lower().strip() == "install ok installed"):
                status = "INSTALLED"
                break
        return status

    def check_services_status(self, req_services_cmd_list):
        service_status_dict = {}
        try:
            for service, service_cmd in req_services_cmd_list:
                status = ""
                (rc, stdout, stderr) = self.execute_command(service_cmd)
                if(rc == 0 or rc == 3):
                    status = self._parse_service_cmd_std_output(stdout)
                else:
                    status = "UNABLE_TO_DETERMINE_SERVICE_STATUS"
                service_status_dict[service] = status
        except Exception as e:
            self.LOG.error(e)
        return service_status_dict

    def _parse_service_cmd_std_output(self, stdout):
        status = ""
        for entry in stdout:
            if ("Active:" in entry):
                status = entry[entry.find(":") + 1:entry.find(")") + 1].strip()
                break
        return status

    def is_user_privileged(self, exp_novalink_user_grp):
        is_user_privileged = False
        uid = -1
        grp_name = ""
        try:
            (rc, stdout, stderr) = self.execute_command(
                CMD_GET_UID + self.user)
            if(rc == 0):
                uid = self._parse_uid_cmd_std_output(stdout)
                self.LOG.info("NovaLink User {0} has uid {1}".format(
                    self.user, uid))

            (rc, stdout, stderr) = self.execute_command(
                CMD_GET_USER_GROUP + self.user)
            if(rc == 0):
                grp_name = self._parse_grp_name_cmd_std_output(stdout)
                self.LOG.info("NovaLink User {0} belongs to {1} user group"
                              .format(self.user, grp_name))

            if (int(uid) == 0 or exp_novalink_user_grp in grp_name.strip()):
                is_user_privileged = True

        except Exception as e:
            self.LOG.error(e)
        return is_user_privileged

    def is_neutron_user_privileged(self, exp_novalink_user_grp):
        is_neutron_user_privileged = False
        uid = -1
        grp_name = ""
        try:
            (rc, stdout, stderr) = self.execute_command(
                CMD_GET_UID + self.neutron_user)
            if(rc == 0):
                uid = self._parse_uid_cmd_std_output(stdout)

            (rc, stdout, stderr) = self.execute_command(
                CMD_GET_USER_GROUP + self.neutron_user)
            if(rc == 0):
                grp_name = self._parse_grp_name_cmd_std_output(stdout)

            if (uid == 0 or exp_novalink_user_grp in grp_name.strip()):
                is_neutron_user_privileged = True

        except Exception as e:
            self.LOG.error(e)
        return is_neutron_user_privileged

    def _parse_uid_cmd_std_output(self, stdout):
        return stdout[0].split(":")[2]

    def _parse_grp_name_cmd_std_output(self, stdout):
        return stdout[0].split(':')[1]

    def get_vios_rmc_state(self):
        is_error = False
        rc, stdout, stderr = self.execute_command(CMD_VIOS_RMC_STATE)
        vios_list = []
        if(rc == 0):
            vios_list = self._parse_vios_cmd_std_output(stdout)
        else:
            is_error = True
        return (is_error, vios_list)

    def _parse_vios_cmd_std_output(self, stdout):
        vios_list = []
        try:
            for entry in stdout:
                single_vios__dict = {}
                outer_val = entry.split(",")
                for inner_entry in outer_val:
                    inner_val = inner_entry.split("=")
                    single_vios__dict[inner_val[0]] = inner_val[1]
                vios_list.append(single_vios__dict)
        except Exception as exp:
            self.LOG.exception(exp)
            self.LOG.error("Exception occurred while parsing vios stdout")
        return vios_list
