#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_ttv.utils import utils

base_nova_conf_file = "/etc/nova/nova.conf"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"

CMD_LSPORTIP = 'svcinfo lsportip -nohdr -filtervalue'\
    ' "link_state=active":"state=configured" -delim !'
CMD_LSFABRIC = 'svcinfo lsfabric -nohdr -delim :'
CMD_SHOW_VERSION = 'svcinfo lssystem '

'''
Resource object represent a SVC resource
'''


class SVCStorageResource():

    def __init__(self, resource, logger=None, timeout=60):
        self.svcip = resource["hostname"]
        self.port = 22
        self.svcUser = resource["userid"]
        self.svcPwd = resource["passwd"]
        self.keyFile = resource["key-file"]
        self.LOG = logger
        self.ssh = None

    def checkCredential(self):
        self.LOG.debug("checking credential for resource: " + self.svcip)
        credential = 1
        if self.ssh is None:
            self.ssh = self._getSVCConnection()
            credential = 0
        return credential

    '''
    get SSH connection of remote host
    '''

    def _getSVCConnection(self):
        self.ssh = utils.getSSHConnection(self.svcip, self.svcUser, self.port,
                                          self.svcPwd, self.keyFile)
        return self.ssh

    '''
    get firmware version
    '''

    def getFirmware(self):
        self.LOG.debug("retrieving SVC storage firmware")
        if self.ssh is None:
            self.ssh = self._getSVCConnection()
        firmware = ""
        returnCode, stdout, stderr = utils.runSSHCommand(
            self.ssh, CMD_SHOW_VERSION)
        self.LOG.debug("SSH command [%s] returned returnCode: "
                       "%s stdout: %s stderr: %s" %
                       (CMD_SHOW_VERSION, returnCode, stdout, stderr))
        if returnCode == 0:
            for line in stdout:
                if "code_level" in line:
                    elements = line.split()
                    if len(elements) > 2:
                        firmware = elements[1]
        return firmware

    '''
    get node iscsi ips
    '''

    def getNodeISCSIIPs(self):
        '''validation for svc node ISCSI IPs'''
        if self.ssh is None:
            self.ssh = self._getSVCConnection()
        returnCode, stdout, stderr = utils.runSSHCommand(self.ssh,
                                                         CMD_LSPORTIP)
        nodeIPDict = {}
        ipindex = 3
        if utils.is_ipv6_install():
            ipindex = 6
        if returnCode == 0 and len(stdout) > 0:
            for line in stdout:
                nodeID = line.split("!")[1]
                nodeip = line.split("!")[ipindex]
                if(len(nodeip) > 0):
                    if nodeIPDict.get(nodeID) is not None:
                        existingnodeiplist = nodeIPDict.get(nodeID)
                        existingnodeiplist.append(nodeip)
                        nodeIPDict[nodeID] = existingnodeiplist
                    else:
                        nodeiplist = []
                        nodeiplist.append(nodeip)
                        nodeIPDict[nodeID] = nodeiplist
        output = self.svcip + "----"
        for nodeid in nodeIPDict:
            output += "Node" + nodeid + "!!"
            output += ",".join(nodeIPDict[nodeid])
            output += "###"
        return output

    '''
    Get logged in initiator wwpns
    '''

    def getlsfabric(self):
        '''validation for svc lsfabric'''
        if self.ssh is None:
            self.ssh = self._getSVCConnection()
        returnCode, stdout, stderr = utils.runSSHCommand(self.ssh,
                                                         CMD_LSFABRIC)
        lsfabric_dict = {}
        wwpn_dict = {}
        host_wwpn_dict = {}
        lsfabric_dict["wwpn_dict"] = wwpn_dict
        lsfabric_dict["host_wwpn_dict"] = host_wwpn_dict

        if returnCode == 0 and stdout:
            for line in stdout:
                data = line.split(":")
                host_wwpn = data[0]
                host_name = data[8]
                '''
                if hostname is empty in lsfabric output that means the host
                is zoned but not yet configured to have a host object in svc.
                Assign an temporary hostname to identify this scenario'''
                if host_name == '':
                    host_name = 'empty_unknown_zoned_unconfigured_ttv99999'
                type = data[10]
                if (type == 'host'.lower() or type == 'unknown'.lower()):
                    if host_name in host_wwpn_dict.keys():
                        if host_wwpn not in host_wwpn_dict[host_name]:
                            host_wwpn_dict[host_name].append(host_wwpn)
                    else:
                        host_wwpn_dict[host_name] = [host_wwpn]
                    if host_wwpn in wwpn_dict.keys():
                        type = wwpn_dict[host_wwpn].get("type")
                        node_list = wwpn_dict[host_wwpn].get("node_list")
                        host_name = wwpn_dict[host_wwpn].get("host_name")
                        if type is None:
                            wwpn_dict[host_wwpn]["type"] = type
                        if node_list is None:
                            node_list = []
                            node_list.append(data[3])
                        else:
                            if data[3] not in node_list:
                                node_list.append(data[3])
                        wwpn_dict[host_wwpn]["node_list"] = node_list

                        if host_name is None:
                            wwpn_dict[host_wwpn]["host_name"] = host_name
                    else:
                        wwpn_dict[host_wwpn] = {}
                        wwpn_dict[host_wwpn]["type"] = type
                        wwpn_dict[host_wwpn]["node_list"] = [data[3]]
                        wwpn_dict[host_wwpn]["host_name"] = host_name
        return lsfabric_dict

    '''
    get SVC storage system wwpns
    '''

    def getSystemFCPorts(self):
        if self.ssh is None:
            self.ssh = self._getSVCConnection()
        cmd = 'lsportfc -filtervalue'\
            ' type=fc:status=active:attachment=switch -delim : -nohdr'
        fc_host_connectivity_dict = {}
        wwpn_set = set()
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set

        returnCode, stdout, stderr = utils.runSSHCommand(self.ssh, cmd)
        if returnCode == 0 and stdout:
            for line in stdout:
                elements = line.split(":")
                wwpn = elements[7]
                wwpn_set.add(wwpn.upper())

        self.LOG.debug("fc_host_connectivity_dict for svc:%s" % self.svcip)
        self.LOG.debug(fc_host_connectivity_dict)
        return wwpn_set
