#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import subprocess
import re
from powervc_ttv.utils.utils import getSSHConnection
from powervc_ttv.utils.utils import runSSHCommand
from powervc_ttv import _
from powervc_ttv.utils import utils

"""
CLI Commands for FC zoning operations.
"""
GET_ACTIVE_ZONE_CFG = 'terminal length 0 ; show zoneset active vsan '
CMD_SHOW_ROLE = "show role name "
CMD_SHOW_USER = "show user-account  "
CMD_SHOW_VERSION = "show version"
CMD_SHOW_VSAN_DB_STATS = "show zone analysis active vsan "

"""
Common constants used by Cisco FC Zone Driver.
"""
YES = 'y'
ACTIVE_ZONE_CONFIG = 'active_zone_config'
CFG_ZONESET = 'zoneset'
CFG_ZONE = 'zone'
CFG_ZONE_MEMBER = 'pwwn'
CFG_ZONE_VSAN = 'vsan'
CFG_ZONES = 'zones'
OPENSTACK_CFG_NAME = 'OpenStack_Cfg'
SUCCESS = 'Success'
TRANS_ABORTABLE = 'It is abortable'


class CiscoFabric(object):
    def __init__(self, name, ip, user, password=None, port=22, vsan=None,
                 key=None, uniqueID=None, logger=None):
        self.name = name
        self.ip = ip
        self.user = user
        self.password = password
        self.port = port
        self.vsan = vsan
        self.key = key
        self.unique_id = uniqueID
        self.logger = logger

    def execute_command(self, cmd):
        ssh_conn = getSSHConnection(self.ip, self.user, self.port,
                                    self.password, self.key)
        if ssh_conn:
            return runSSHCommand(ssh_conn, cmd)

    def is_reachable(self):
        if utils.is_ipv6_install():
            cmdping = ["ping6", "-c 4 ", "-w 10 ", self.ip]
        else:
            cmdping = ["ping", "-c 4 ", "-w 10 ", self.ip]
        p = subprocess.Popen(cmdping, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        p.communicate()
        p_status = p.wait()
        if p_status == 0:
            return True
        else:
            return False

    def login(self, user, password=None, key=None):
        if password is not None or key is not None:
            ssh_conn = getSSHConnection(self.ip, user, self.port,
                                        self.password, self.key)
        if ssh_conn:
            return ssh_conn

    def get_firmware_version(self):
        returnCode, stdout, stderr = self.execute_command(CMD_SHOW_VERSION)
        if returnCode == 0 and stdout:
            for line in stdout:
                line = line.strip()
                if "system:" in line and "version" in line:
                    line = line.split(":")[1].strip()
                    startIndex = line.find("version ") + 8
                    endIndex = len(line)
                    if "(" in line and ")" in line:
                        endIndex = line.find("(")
                    version = line[startIndex:endIndex]
                    return version
        else:
            self.logger.debug(
                "Error while running command. Error=%s" % str(stderr))

    def get_user_role(self, user=None):
        returnCode, stdout, stderr = self.execute_command(CMD_SHOW_USER +
                                                          self.user)
        if returnCode == 0 and stdout is not None:
            role = None
            for line in stdout:
                line = line.strip()
                if "roles:" in line:
                    line = line.split(":")
                    role = line[1]
                    return role
        else:
            self.logger.debug(
                "Error while running command. Error=%s" % str(stderr))

    def get_rules(self, role):
        returnCode, stdout, stderr = self.execute_command(CMD_SHOW_ROLE +
                                                          role)
        rules = []
        if returnCode == 0 and stdout is not None:
            for line in stdout:
                line = line.strip()
                words = line.split()
                if len(words) >= 4 and \
                        ("permit" in words[1] or "deny" in words[1]):
                    rule = {"rule_no": words[0], "type": words[1],
                            "command-type": words[2], "feature": words[3]}
                    rules.append(rule)
            return rules
        else:
            self.logger.debug(
                "Error while running command. Error=%s" % str(stderr))

    def get_zoning_memory_space_in_MB(self):
        cmd = CMD_SHOW_VSAN_DB_STATS + self.vsan
        returnCode, stdout, stderr = self.execute_command(cmd)
        if returnCode == 0 and stdout:
            for line in stdout:
                line = line.strip()
                if "Formatted size:" in line:
                    line = line.split(":")[1].strip()
                    if " / " in line:
                        line = line.split("/")
                        allocated_memory = line[1].strip()
                        if allocated_memory.lower()\
                                .endswith(('kb', 'kilobytes', 'kilo bytes')):
                            allocated_memory = float(
                                allocated_memory.split()[0]) / 1024
                        elif allocated_memory.lower()\
                                .endswith(('mb', 'megabytes', 'mega bytes')):
                            allocated_memory = \
                                float(allocated_memory.split()[0])
                        elif allocated_memory.lower()\
                                .endswith(('gb', 'gigabytes', 'giga bytes')):
                            allocated_memory = float(
                                allocated_memory.split()[0]) * 1024
                        elif allocated_memory.lower()\
                                .endswith(('bytes', 'b')):
                            allocated_memory = float(
                                allocated_memory.split()[0]) / (1024 * 1024)

                        actual_memory = line[0].strip()
                        if actual_memory.lower()\
                                .endswith(('kb', 'kilobytes', 'kilo bytes')):
                            actual_memory = float(
                                actual_memory.split()[0]) / 1024
                        elif actual_memory.lower()\
                                .endswith(('mb', 'megabytes', 'mega bytes')):
                            actual_memory = float(actual_memory.split()[0])
                        elif actual_memory.lower()\
                                .endswith(('gb', 'gigabytes', 'giga bytes')):
                            actual_memory = float(
                                actual_memory.split()[0]) * 1024
                        elif actual_memory.lower()\
                                .endswith(('bytes', 'b')):
                            actual_memory = float(
                                actual_memory.split()[0]) / (1024 * 1024)

                        return {"actual_memory": actual_memory,
                                "allocated_memory": allocated_memory}
            else:
                self.logger.debug("Error while running command. "
                                  "Error=%s" % str(stderr))

    def get_member_wwpns(self):
        """Return the active zone configuration.

        Return active zoneset from fabric. When none of the configurations
        are active then it will return empty map.

        :returns: Map -- active zone set map in the following format
        {
            'zones':
                {'openstack50060b0000c26604201900051ee8e329':
                    ['50060b0000c26604', '201900051ee8e329']
                },
            'active_zone_config': 'OpenStack_Cfg'
        }
        """
        zone_set = {}
        zone = {}
        zone_member = None
        zone_name = None
        zone_set_name = None

        cmd = 'show zoneset active vsan '
        if self.vsan:
            cmd = cmd + self.vsan

        returnCode, stdout, stderr = self.execute_command(cmd)
        if returnCode == 0 and stdout:
            try:
                for line in stdout:
                    # Split on non-word characters,
                    line_split = re.split('[\s\[\]]+', line)
                    if CFG_ZONESET in line_split:
                        # zoneset name [name] vsan [vsan]
                        zone_set_name = \
                            line_split[line_split.index(CFG_ZONESET) + 2]
                        zone_set[CFG_ZONE_VSAN] = \
                            line_split[line_split.index(CFG_ZONESET) + 4]
                        continue
                    if CFG_ZONE in line_split:
                        # zone name [name] vsan [vsan]
                        zone_name = \
                            line_split[line_split.index(CFG_ZONE) + 2]
                        zone[zone_name] = list()
                        continue
                    if CFG_ZONE_MEMBER in line_split:
                        # Examples:
                        # pwwn c0:50:76:05:15:9f:00:12
                        # * fcid 0x1e01c0 [pwwn 50:05:07:68:02:20:48:04] \
                        # [V7K_N1P2]
                        zone_member = \
                            line_split[
                                line_split.index(CFG_ZONE_MEMBER) + 1]
                        zone_member_list = zone.get(zone_name)
                        zone_member_list.append(zone_member.replace(':', ''))

                zone_set[CFG_ZONES] = zone
                zone_set[ACTIVE_ZONE_CONFIG] = zone_set_name
            except Exception as ex:
                # Incase of parsing error here,
                # it should be malformed cli output.
                msg = _("Malformed zone configuration: (switch=%(switch)s "
                        "zone_config=%(zone_config)s)."
                        ) % {'switch': self.ip,
                             'zone_config': stdout}
                self.logger.error(msg)
                self.logger.exception(ex)

            return zone_set
        else:
            self.logger.debug(
                "Error while running command. Error=%s" % str(stderr))

    def get_loggedin_host_ports(self):
        fc_host_connectivity_dict = {}
        wwpn_set = set()
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set

        cmd = 'show fcns database vsan ' + self.vsan

        returnCode, stdout, stderr = self.execute_command(cmd)
        if returnCode == 0 and stdout:
            for line in stdout:
                if " N " in line:
                    elements = line.split()
                    if len(elements) > 2 and \
                            len(elements[2].replace(":", "")) == 16:
                        wwpn = elements[2].replace(":", "")
                        wwpn_set.add(wwpn.upper())
            self.logger.debug(
                "fc_host_connectivity_dict for ciscofabric:%s" % self.ip)
            self.logger.debug(fc_host_connectivity_dict)
            return wwpn_set
        else:
            self.logger.debug(
                "Error while running command. Error=%s" % str(stderr))
